;;; GOLD Lisp Macro Package for GNU Emacs,     >>> MAIN ENTRY FILE <<< !
;;; Written by Markus Ast,  ast@intes.de        Germany
;;;
;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###                                                                      ###
;;;###    gold.el - Emacs Lisp Macro (super-EDT-like)  - by Markus Ast      ###
;;;###   ===============================================================    ###
;;;###                                                                      ###
;;;### The GOLD additions to GNU-Emacs are split into many files. This is   ###
;;;### the main entry file including primary definitions and the autoload   ###
;;;### commands for various other basic GOLD facilities. This splitting is  ###
;;;### designed to make the start-up loading time be as short as possible.  ###
;;;###                                                                      ###
;;;### This files are not part of GNU-Emacs, but they are a conglomerate    ###
;;;### of own developments and some extended macro elements out of the      ###
;;;### standard GNU-Emacs.el files (i.e. the alpha version of gold.el was   ###
;;;### just a derivate of edt.el with improved keybindings).                ###
;;;###                                                                      ###
;;;### For activation place   (load-library "gold.elc")   in your system    ###
;;;### default.el or your personal .emacs file (be sure not to overwrite    ###
;;;### those definitions -> better place it at bottom than top of file).    ###
;;;### If you wish to redefine GOLD macros or perform special key bindings  ###
;;;### put those personal definitions into a file called  .emacs_gold  on   ###
;;;### your home directory (will be loaded AFTER gold.el automatically).    ###
;;;### Note that a number of non-standard GNU-Emacs extensions are defined  ###
;;;### to be autoloaded within gold.el (e.g. calc, ispell etc.) and there-  ###
;;;### fore may overwrite the corresponding - and now obsolete - autoloads  ###
;;;### made in your original default.el for this.                           ###
;;;###                                                                      ###
;;;###           Bug reports and suggestions are always welcome !           ###
;;;###                                                                      ###
;;;############################################################################
(defconst GOLD-last-change "25-Nov-11" "Date of last change in gold.el")
;;;------------------------***********----------------------------------------
(defvar xemacs (string-match "XEmacs\\|Lucid" emacs-version)
   "Non-nil if the running emacs is XEmacs (in opposite to FSF's GNU Emacs)")
(if xemacs (progn
    (setq startup-message "*W* GOLD package not fully operational with XEmacs !")
    (message startup-message) (sleep-for 1))
;; else
  (if (string-lessp emacs-version "22.0") (progn (beep)
      (setq startup-message "*E* GOLD package needs Emacs-23.0 or higher !!!")
      (message startup-message) (sleep-for 1) (beep) (sleep-for 1) (beep)))
  (or (string-lessp emacs-version "23.4") (progn
      (setq startup-message "*W* GOLD package not yet tested with Emacs-23.4 or higher (use at own risk) !")
      (message startup-message)
      (if (or (equal user-login-name "markus")
              (equal user-login-name "ast")) nil (beep) (sleep-for 1))))
)

;;; GOLD-Version:
(defconst GOLD-version (concat "GNU-Emacs " emacs-version
  " with gold.el from " GOLD-last-change " !") "No documentation.")
(defvar startup-message (format
   "GOLD-GNU-Emacs (%s):  Type the HELP key (PF2 key) to get Help !"
    GOLD-last-change) "Standard message printed to echo area after install")

;;; Extend load-path:
(setq save-load-path load-path)
(defun cut-path (path) (substring path 0 (string-match "[^/]*/?$" path)))
(defvar gold-load-path nil "Load-path to find gold-package")
(defvar calc-load-path nil "Load-path to find calc-package")
(defvar main-load-path nil "Load-path to find 'new' packages")
(let ((rest load-path))
     (while (and rest (not main-load-path))
            (if (file-executable-p (concat (car rest) "/gold"))
                (setq main-load-path (car rest))
                (setq rest (cdr rest))
)))
(or main-load-path (setq main-load-path "."))
(or gold-load-path (setq gold-load-path (concat main-load-path "/gold")))
(or calc-load-path (setq calc-load-path (concat main-load-path "/calc")))
;;
;; Definition of Load-Path (1.gold, 2.site-lisp, 4.calc, 5.rest):
(setq load-path (append (list gold-load-path calc-load-path
                              (concat main-load-path "/jde")
                              (concat main-load-path "/jde/lisp")
                              (concat main-load-path "/semantic")
                              (concat main-load-path "/speedbar")
                              (concat main-load-path "/elib")
                              (concat main-load-path "/eieio")
                              ) load-path
                        (list (concat main-load-path "/auctex"))))
;;
;; Ignore default.el: This is a no-op when called from default.el, but makes
(setq inhibit-default-init t); "gold" library loadable from .emacs file also !

;; Set file-size-warning to 200 MB (was 10 MB, >200MB is too much for 32bit)
(setq large-file-warning-threshold 200000000)
;setq large-file-warning-threshold 1000000000)

;;;----------------------------------------------------------------------------
(defun diary-update ()
  "Update & display global diary (sum up company diary and all holiday files)."
  (interactive) (if (functionp 'my-calendar) (my-calendar) (calendar)))
;;;
;;;
;;; Loading Urgent-Stuff:
;;;-----------------------
(load "gold_debug"    nil t) ; Site-init dependent debugging/watching
(require 'gold_whoami)       ; WHOAMI functions to watch current settings
(require 'gold_simple)       ; Simple move&scroll functions (primary defs).
(load "gold_term"     nil t) ; Terminal-Dependent Functions and Key-Bindings
;;;                           ; (loaded before ANY substitute-key-definitions)
;;;
;;; Kill Klipper !
;;;----------------
(defun get-pid (process) "Get PROCESS' ID (or nil)." (interactive)
   (let ((pid (string-to-number (shell-command-to-string
         (format "ps x | grep %s | grep -v grep | head -1 | sed -e 's/ [^0-9].*//'" process)))))
        (if (> pid 0) (eval pid))))

(defun kill-klipper () "Kill KDE's klipper (causes strange `Timer notify events`)."
   (interactive)
   (let ((pid (get-pid "klipper")))
        (if pid (progn (message "Killing KDE's klipper !") (beep)
                (call-process shell-file-name nil 0 nil "-fc"
                      (format "kill %d" pid)) (sleep-for 1)))))
;;
(if (string-lessp emacs-version "21.2") (kill-klipper)) ;; Allow klipper on newer systems
;;;
;;;
;;; Remove icon tool-bar for pre-22 and skip Emacs-22 splash-screen
;;;-----------------------------------------------------------------
(if (< emacs-major-version 22)
    (if (featurep 'tool-bar) (tool-bar-mode -1))
    (setq save-abbrevs 'silently)  ;; silently read/write ~/.abbrev_defs
    (setq inhibit-splash-screen t) ;; same as --no-splash
)

;; Only show startup if we are called without any arguments
(add-hook 'after-init-hook 'command-arg-dependent-startup t) ;; Call AFTER .emacs
(defun command-arg-dependent-startup ()
  "Only show startup if we are called without any arguments."
  (if (nth 1 command-line-args)
      (setq inhibit-startup-screen t)     ; drop *scratch* startup-message
      (setq inhibit-startup-screen nil))) ; show *scratch* startup-message

;;;
;;; Unconditional Loading:
;;;------------------------
(if (string-lessp emacs-version "21.0") ;;;???
    (load "gold_complete" nil t) ; must be loaded after term/keyboard definitions
    (load "gold_emacs21" nil t)  ; gold_complete does not work with Emacs >=21 ?!
)
(load "gold_comint"   nil t) ; Additions/modifications to comint.el
;;
(defvar proportional-fonts nil "*nil inhibits faces with variable font width.")
(if (or x-terminal windows)
  (load "gold_face"     nil t) ; Face customizations for font-lock mode
)
(or xemacs
  (load "gold_hilit"   nil t) ; Old hiliting customizations (obsolete?)
)
(load "gold_replace"   nil t) ; Query-Replace with ADVANCE/BACKUP mode
(load "gold_doc"       nil t) ; On-Line documentation (Basis for On-Line-Help)
(load "gold_help"      nil t) ; GOLD help facilities (context-help...)
(load "gold_view"      nil t) ; PostCript-Viewer for context helps
(load "gold_compile"   nil t) ; Compile actual buffer contents.
(load "gold_tex"       nil t) ; Extra definitions to (La)TeX-Modes
(load "gold_tags"      nil t) ; Additional TAGS features
(load "gold_macro"     nil t) ; Keyboard-macro utilities
(load "gold_bookmark"  nil t) ; Installing Bookmark possibilities:
(load "gold_calendar"  nil t) ; Installing calendar/diary/holiday/appointments
(load "gold_company"   nil t) ; Functions for company/user-init-files
(if xemacs nil
(load "gold_rmail"     nil t) ; Installing RMail with rmail-output-file-alist
(load "gold_mail"      nil t) ; Installing Mail and loading mail-aliases
(load "gold_gnus"      nil t) ; Installing GNUS news reading environment.
)
(load "gold_print"     nil t) ; Extra print command (autoload won't work "r")
(load "gold_shell"     nil t) ; Load shell mode additions
(load "dbfrobs"         t  t) ; Useful enhancements for debugging
(load "uniquify"       nil t) ; Installing unique buffer names:
(setq uniquify-separator "<")
;;;
;;;
;;; Autoload definitions:
;;;-----------------------
(defun AL (symbol file doc) "Short Autoload-Def" (autoload symbol file doc t))
(AL 'fortran-mode       "gold_f7x"     "Load Fortran-mode"             )
(AL 'f77-mode           "gold_f7x"     "Load Fortran-mode"             )
(AL 'f7x-mode           "gold_f7x"     "Load Fortran-mode"             )
(AL 'f7x2f77-mode       "gold_f7x"     "Load Fortran-mode"             )
(AL 'gprof-mode         "gold_gprof"   "Viewing mode for Unix gprof files")
(AL 'kmail-edit-mode    "kmail"        "Edit kmail's mail bodies (KDE)")
(AL 'permas-dat-mode    "permas_dat"   "Editing mode for PERMAS DAT files")
(AL 'permas-inp-mode    "permas_inp"   "Viewing mode for ABAQUS *.inp files")
(AL 'permas-log-mode    "permas_log"   "Viewing mode for PERMAS LOG files")
(AL 'permas-nas-mode    "permas_nas"   "Mode for NASTRAN input files to PERMAS")
(AL 'permas-pqa-mode    "permas_pqa"   "Viewing mode for PERMAS PQA log files")
(AL 'permas-pro-mode    "permas_pro"   "Viewing mode for PERMAS PRO files")
(AL 'permas-psum-mode   "permas_psum"  "Viewing mode for PERMAS PSUM files")
(AL 'permas-res-mode    "permas_res"   "Viewing mode for PERMAS RES files")
(AL 'permas-t19-mode    "permas_t19"   "Mode for Marc t19 result files")
(AL 'permas-uci-mode    "permas_uci"   "Editing mode for PERMAS UCI files")
(AL 'permas-unv-mode    "permas_unv"   "Mode for I_DEAS Universal-Files")
(AL 'permas-fmt-mode    "permas_fmt"   "Edit/View Mode PERMAS format files")
(AL 'visper-tip-mode    "visper_tip"   "Mode for VisPer Tooltip Files")
(AL 'python-mode        "python-mode"  "Python-Mode")
(AL 'show-permas-fmt-group "permas_fmt" "Show a PERMAS FMT Group.")
(AL 'permas-formats-buffer "permas_fmt" "Switch to [reloaded] PERMAS/PERMNEW formats buffer.")
(AL 'self-exec-server   "gold_server"  "Self executable(s) for emacsclient.")
;(AL 'named-shell        "gold_shell"   "Run a named inferior shell.")
;(AL 'new-shell          "gold_shell"   "Create a new shell (with increment)")
;(AL 'root-shell         "gold_shell"   "Start a *root* shell (or switch to)")
;(AL 'telnet-shell       "gold_shell"   "Start a shell on given MACHINE")
;(AL 'shell-mode         "gold_shell"   "Start shell-mode"           )
;(AL 'switch-shell       "gold_shell"   "Load shell-mode"            )
;;;
;(AL 'send-pr-template   "gold_gnats"   "Command to create and send a problem report.")
;(AL 'send-pr-mode       "gold_gnats"   "Major mode for sending problem reports.")
;(AL 'send-pr            "gold_gnats"   "Command to create and send a problem report.")
(AL 'edit-pr            "gold_gnats"   "Command to edit a problem report.")
(AL 'view-pr            "gold_gnats"   "Command to view a problem report.")
(AL 'unlock-pr          "gold_gnats"   "Unlock a problem report.")
(AL 'query-pr           "gold_gnats"   "Command to query info about problem reports.")
(AL 'gnats-view-mode    "gold_gnats"   "gnats-view-mode.")
(AL 'pr-view-mode       "gold_gnats"   "Interactive gnats-view-mode.")
;;;
(AL 'all                "all"           "(X)Edit ALL lines containing REGEXP.")
(AL 'c++-mode           "cc-mode"       "C++ Editing Mode")
(AL 'c-mode             "cc-mode"       "C Editing Mode")
(AL 'cc-mode            "cc-mode"       "Objective-C Editing Mode")
(AL 'objc-mode          "cc-mode"       "Objective-C Editing Mode")
(AL 'flash-matching-mode "flashparen"   "Flash matching character")
(AL 'turn-on-eldoc-mode "eldoc"         "Electric-Lisp-Documentation")
(AL 'follow-mode        "follow"        "Synchronized windows")
(AL 'ftelnet            "ftelnet"       "Remote login interface")
(AL 'listbuf            "listbuf"       "Buffer-menue with Rev & Proc")
(AL 'gnu-plot-mode      "gnuplot"       "Load gnuplot facilities")
(AL 'plot-mode          "gold_plot"    "Load GOLD plot facilities")
(AL 'GOLD-news-help     "gold_news"    "Load GOLD help facilities")
(AL 'gold-keypad-help   "gold_news"    "Load GOLD help facilities")
(AL 'help-bulk-keyword  "permas_nas"   "GOLD help for NAS-Files")
(AL 'adr-mode           "gold_adr"     "DB like mode for ASCII-Address-Files")
(AL 'client-mode        "gold_client"  "DB like mode for ASCII-Client-Files")
(AL 'ediff-buffers      "ediff"   "Intelligent Emacs interface to diff")
(AL 'ediff-files        "ediff"   "Intelligent Emacs interface to diff")
(AL 'ediff-files-remote "ediff"   "Intelligent Emacs interface to diff")
(AL 'crontab            "crontab" "Edit your current crontab file")
(AL 'double-mode        "double"  "Toggle double-key-mode.")
(AL 'elock              "elock"   "Lock current emacs-session.")
(AL 'igrep              "igrep"   "Run `grep' to match EXPRESSION in FILES...")
(AL 'egrep              "igrep"   "Run `egrep'...")
(AL 'fgrep              "igrep"   "Run `fgrep'...")
(AL 'igrep-recursively  "igrep"   "Run `grep' recursively...")
(AL 'egrep-recursively  "igrep"   "Run `egrep' recursively...")
(AL 'fgrep-recursively  "igrep"   "Run `fgrep' recursively...")
;(AL 'turn-on-lazy-lock  "lazy-lock" "Unconditionally turn on Lazy Lock mode")
;;; 'html-mode          "hm--html-menu" "HTML major mode.")
(AL 'sgml-mode          "psgml"   "Major mode for editing SGML files.")

;;;
;;****************************************************************************
;;***** Setting global variables ! *******************************************
;;****************************************************************************
(setq auto-mode-alist (append
  '(("\\.adr\\b"           . adr-mode)       ; Address-DB
    ("\\.aux\\b"           . LaTeX-mode)
    ("/perm.*\\.client\\b" . perminfo-mode)  ; Permas-Client-Info file
    ("/perm.*\\.config\\b" . perminfo-mode)  ; Permas-Configurations-Info file
    ("/perm.*\\.doc\\b"    . perminfo-mode)  ; Permas-Revision-Info file
    ("/perm.*\\.rev\\b"    . perminfo-mode)  ; Permas-Revision-Info file
    ("\\.bdf[0-9]*\\b"     . permas-nas-mode); PERMAS-NAS
    ("\\.client$"          . client-mode)    ; Kunden-DB
    ("\\.C$"               . c++-mode)
    ("\\.c$"               . c-mode)         ; to edit C code
    ("\\.cc$"              . c++-mode)
    ("\\.cu$"              . c++-mode)
    ("/xy.*\\.dat"         . plot-mode)      ; PERMAS-XY
    ("/XY.*\\.DAT"         . plot-mode)      ; PERMAS-XY
    ("\\.caso?\\b"         . permas-dat-mode); PERMAS-DAT
    ("\\.dat[0-9]*\\b"     . permas-dat-mode); PERMAS-DAT
    ("\\.dato[0-9]*\\b"    . permas-dat-mode); PERMAS-DAT
    ("\\.DFT[0-9]*\\b"     . permas-uci-mode); PERMAS-UCI (Defaults-File)
    ("\\.dpv[45]?\\b"      . permas-dat-mode); PERMAS-DAT
    ("FILES/formats\\b"    . permas-fmt-mode); PERMAS-FMT
    ("etc/formats\\b"      . permas-fmt-mode); PERMAS-FMT
    ("/formats.base"       . permas-fmt-mode); PERMAS-FMT
    ("/formats.local"      . permas-fmt-mode); PERMAS-FMT
    ("/formats.new"        . permas-fmt-mode); PERMAS-FMT
    ("\\.fmt?\\b"          . permas-fmt-mode); PERMAS-FMT
    ("\\.tpl\\b"           . visper-tip-mode); VisPer-Tip
    ("\\.vpt\\b"           . visper-tip-mode); VisPer-Tip
    ("visper/.*[Tt]ip"     . visper-tip-mode); VisPer-Tip
    ("VPT[ip]*/.*"         . visper-tip-mode); VisPer-Tip
    ("\\.[dl]tx\\b"        . LaTeX-mode)
    ("\\.e\\b$"            . eiffel-mode)    ; Eiffel(3)-programs
    ("\\.el\\b"            . emacs-lisp-mode); Emacs
    ("/\\.emacs\\b"        . emacs-lisp-mode); Emacs init-files
    ("\\.fd\\b  "          . LaTeX-mode)
    ("\\.fif[0-9]*\\b"     . permas-nas-mode); PERMAS-NAS
    ("\\.fnt\\b"           . text-mode)      ; Font-Definitions
    ("\\.fortran\\b"       . fortran-mode)
    ("\\.f77.f\\b"         . f7x2f77-mode)
    ("\\.f[7tn]*\\b"       . f7x-mode)
    ("\\.f[79]*x\\b"       . f7x-mode)
    ("\\.fsav"             . f7x-mode)
    ("\\.f90\\b"           . f90-mode)
    ("\\.gp\\b"            . gnu-plot-mode)  ; Edit gnuplot files
    ("\\.gprof\\b"         . gprof-mode)     ; Unix gprof file
    ("\\.h\\b"             . c-mode)         ; to edit C code
    ("\\.hh\\b"            . c++-mode)       ; to edit C++ code
    ("\\.inp[0-9]*\\b"     . permas-inp-mode); ABAQUS-INPut
;   ("/kmail.*\\.tmp"      . kmail-edit-mode); kmail-edit-server
;   ("/knode.*\\.tmp"      . kmail-edit-mode); kmail-edit-server
    ("\\.log[0-9]*\\b"     . permas-log-mode); PERMAS-LOG
    ("\\.lsp\\b"           . emacs-lisp-mode); Better than just lisp-mode
    ("\\.m\\b"             . objc-mode)      ; to edit C code
    ("\\.nas[0-9]*\\b"     . permas-nas-mode); PERMAS-NAS
    ("\\.post\\b"          . permas-dat-mode); PERMAS-DAT
    ("/@/[0-9]+\\b"        . permas-pqa-mode); PERMAS-PQA
    ("\\.pro[0-9]*\\b"     . permas-pro-mode); PERMAS-PRO
    ("\\.pro2uci[0-9]*\\b" . permas-uci-mode); PERMAS-UCI
    ("\\.psum[0-9]*\\b"    . permas-psum-mode); PERMAS-PSUM
    ("/psum"               . python-mode)    ; Python-mode
    ("\\.py[0-9]*\\b"      . python-mode)    ; Python-mode
    ("\\.res[0-9]*\\b"     . permas-res-mode); PERMAS-RES
    ("\\.sty\\b"           . LaTeX-mode)
    ("\\.tex\\b"           . TeX-mode)
    ("\\.tty[0-9]*\\b"     . text-mode)      ;
    ("\\.uci[0-9]*\\b"     . permas-uci-mode); PERMAS-UCI
    ("\\.t19\\b"           . permas-t19-mode); Marc-t19 result files
    ("\\.uni?v[0-9]*\\b"   . permas-unv-mode); IDEAS-UNIVERSAL
    ("\\.unvo[0-9]*\\b"    . permas-unv-mode); IDEAS-UNIVERSAL
    ("\\.xy[0-9]*\\b"      . plot-mode)      ; PERMAS-XY
    ("\\.XY[0-9]*\\b"      . plot-mode)      ; PERMAS-XY
    ("/gnats/gnats-db/.*/[0-9]+\\b"  . gnats-view-mode)
    ;("\\.mk"              . makefile-mode)
    ;("[mM]akefile"        . makefile-mode)
  ) auto-mode-alist))
;;
;; Extend magic-line-detection (no need to pre-load extra packages)
(setq interpreter-mode-alist (append
  '(("python" . python-mode)
  ) interpreter-mode-alist))
;;
;;**************************
(load "gold_bugfix" nil t) ; Bugfixes for standard emacs lisp-files.
(load "gold_inst" nil t)   ; Installation/customization of additional packages
(load "gold_more" nil t)   ; Just MORE gold definitions
;;**************************

;;;****************************************************************************
;;;***** Special macro functions for switching GOLD-emulation ON/OFF ! *******
;;;****************************************************************************
(add-hook 'term-setup-hook 'GOLD-emulation-on t) ;; Call AFTER Terminal-Setup
(defun GOLD-emulation-on ()
  "Begin emulating DEC's EDT editor.
Certain keys are rebound; including nearly all keypad keys.
Use \\[GOLD-emulation-off] to undo all rebindings except the keypad keys.
Note that this function does not work if called directly from .emacs file !
Instead, the .emacs file should do (setq term-setup-hook 'edt-emulation-on)
Then this function will be called at the time when it will work."
  (interactive)
  (gold-debug "Emacs: term-setup-hook" 'no-save)
  (setq truncate-lines t) (advance-direction)
;e22/23:(if x-terminal (or xemacs (self-exec-server)))
  (or x-terminal (application-keys-on t))
  (define-key esc-map    "?"     'apropos)              ; Esc-<?>
  (define-key esc-map    "\C-m"  'multiply-current-line); Esc-<Return>
  (message startup-message)
  ;; ***** Avoid display-garbage on startup of resized-windows ********
  (if window-system (redraw-display-h))
;;  (if xemacs (error "XEmacs: Stop term-setup in order to keep GOLD-Keys !"))
  (remap-function-keys) ;; Rename function keys (after term-setup)
)
(or (assq 'edt-direction-string minor-mode-alist)
    (setq minor-mode-alist (cons '(edt-direction-string edt-direction-string)
                                   minor-mode-alist)))
;;
;; Look for additional company and/or user-definitions !
;;-------------------------------------------------------
(load-company/user-init-files)
(if (not proportional-fonts) (progn            ; After user-init, before term-setup:
   (setq font-latex-fontify-script nil)        ; NO super/subscript
   (setq font-latex-title-fontify      `color) ; NO proportional \section{title}    (colored)
   (setq font-latex-fontify-sectioning 'color) ; NO proportional \subsection{title} (colored)
))
;;
;; Do some additional settings if we have a valid backup-file-dir
;; Must be done after .emacs_gold (since user may have re-set it)
(if (and backup-file-dir (file-accessible-directory-p backup-file-dir))
    (progn
        ;; Force non-empty path name to have a trailing "/"
        (setq backup-file-dir (file-name-as-directory backup-file-dir))
        (setq version-control t)) ; Make numbered-backup !
    (setq backup-file-dir nil))
;;;
(setq gold-EOF-reached t)     ; Debug control-variable
(gold-debug "Emacs: rest-init (garbage)" 'no-save)
;;###########################-End-of-gold.el-#################################
(provide 'gold)
