;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  X-Terminal dependent definitions (Key/Mouse-Bindings and -Functions)###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_Xterm")

;; Must have a non-nil DISPLAY environment variable:
;;---------------------------------------------------
(or DISPLAY (stringp DISPLAY) (setq DISPLAY "unknown"))
(if (or (equal DISPLAY "unknown") (equal DISPLAY "undefined"))
    (progn (beep) (setq startup-message
    "Please set Your $DISPLAY environment variable for X-Window Emacs !")
    (message startup-message)))


;;; Defuns 'quasi-asynchronous' processes to (re)set propper X-KeySymbols:
;;;------------------------------------------------------------------------
(defun numlock-rename ()
  "Rename Num_Lock keysym to KP_F1 (without clearing modifier)."
  (interactive)
  (call-process shell-file-name nil 0 nil "-fc"
     (concat xmodmap-program " -e \"keysym Num_Lock = KP_F1\"")))

;; xmodmap -e "clear mod2" may corrupt Alt-Button-Events on german xfce,
;; hence we try to avoid actually clearing Num_Lock modifier (see above)!
(defun numlock-clear-and-rename ()
  "Clear Num_Lock modifier (if any), get mod-ID to be cleared automatically."
  (interactive)
  (call-process shell-file-name nil 0 nil "-fc"
     (concat xmodmap-program " -pm"
         " | grep -i \"num.*lock\"" ;;??? should we use remove instaed of clear?
         " | sed -e \"s/\\(mod[^ ]*\\) .*/clear \\1/\""
         " | " xmodmap-program " -"
          ;; Also rename 'Num_Lock' to be 'KP_F1' symbol
         " ; " xmodmap-program " -e \"keysym Num_Lock = KP_F1\"")))

(defun clear-strange-KP-keys ()
  "Rather use [kp-0123456789] respectively [kp-period/decimal] (if available)
(if [kp-n] is available, but isn't the unmodified default KeySym,
then get KeyCode of [kp-n] and reassign [kp-n] to be it's default)."
  (interactive)
  (call-process shell-file-name nil 0 nil "-fc"  ; sed does not have \| => search
     (concat xmodmap-program " -pk"            ; KP_Decimal but not KP_Down/Delete
         " | grep -i \"(.*).*(KP_\\([0-9]\\|Period\\|Decimal\\))\""  ; or KP_Prior ...
         " | sed -e \"s/^ *\\([0-9][0-9]*\\).*(\\(KP_\\([0-9]\\|Period\\|Decimal\\)\\)).*"
                      "/keycode \\1 = \\2 /\""
         " | " xmodmap-program " -")))


;;; Get initial xmodmap info:
;;;---------------------------
(gold-debug "gold_xmodmap-pm/pk")
(xmodmap-pm) ; Get 'xmodmap-pm' output from startyp (test on Num_Lock etc.)
(xmodmap-pk) ; Get 'xmodmap-pk' output from startup (test on Adiaresis etc.)
(or xmodmap-pk (stringp xmodmap-pk) (setq xmodmap-pk "Command not found."))
(if (and (< (length xmodmap-pk) 100)
         (string-match "ommand not found" xmodmap-pk))
    (progn (beep) (setq startup-message
 "Couldn't execute Unix/X11 xmodmap (check Your $PATH environment variable) !")
    (message startup-message)))


;; Clear Num_Lock (if any):
;;--------------------------
(gold-debug "gold_NumLock")
;(if (string-match "Num_Lock" xmodmap-pk) (numlock-rename))
(if (string-match "Num_Lock" xmodmap-pk) (numlock-clear-and-rename))


;; Get Keyboard language (optimized for SGI/PC like keyboards):
;;--------------------------------------------------------------
(gold-debug "keyboard-language")
;(defun xkeysym-pos1/2 (keysym xmodmap)
;   "Returns t if KEYSYM is in 1st or 2nd column of XMODMAP (nil otherwise)."
;   (if (string-match keysym xmodmap)
;       (string-match (concat "^[^(]*(?[^(]*(" keysym ")") xmodmap)
;));; optimized ???
(defun xkeysym-pos1/2 (keysym xmodmap)
   "Returns t if KEYSYM is in 1st or 2nd column of XMODMAP (nil otherwise)."
   (if (string-match keysym xmodmap)
       (or (string-match (concat "^[^(]+(" keysym ")") xmodmap)
           (string-match (concat "^[^(]+([^(]+(" keysym ")") xmodmap))
))
;;
(setq keyboard-language (cond
    ; Matching keysym must be (first) or (second) keysym in line !
    ((xkeysym-pos1/2 "ssharp"    xmodmap-pk) "DEU");1
    ((xkeysym-pos1/2 "ntilde"    xmodmap-pk) "ESP");2
    ((xkeysym-pos1/2 "guillemot" xmodmap-pk) "PRT");3
    ((xkeysym-pos1/2 "brokenbar" xmodmap-pk) "GBR");4
    ((xkeysym-pos1/2 "igrave"    xmodmap-pk) "ITA");5
    ((xkeysym-pos1/2 "eacute"    xmodmap-pk) "FRA");=BEL
    ((xkeysym-pos1/2 "Ooblique"  xmodmap-pk) "NOR");?DNK
    ((xkeysym-pos1/2 "Aring"     xmodmap-pk) "SWE");=FIN
    ( t                                      "USA")))


;; Make special key-(re)definitions for AltGrp-Symbols on "foreign" keypads:
;;---------------------------------------------------------------------------
(gold-debug "gold_AltGrp")
(defun remap-AltGrp-key (key gold-vector)
    (define-key esc-map key key)
    (define-key GOLD-map (concat "\033" key) (key-binding gold-vector))
)
(if (string= keyboard-language "USA") nil
    (remap-AltGrp-key "~"  [gold ?~])
    (remap-AltGrp-key "@"  [gold ?@])
    (remap-AltGrp-key "$"  [gold ?$])
    (remap-AltGrp-key "`"  [gold ?`])
    (remap-AltGrp-key "'"  [gold ?'])
    (remap-AltGrp-key "^"  [gold ?^])
    (remap-AltGrp-key "|"  [gold ?|])
    (remap-AltGrp-key "{"  [gold ?{])
    (remap-AltGrp-key "}"  [gold ?}])
    (remap-AltGrp-key "["  [gold ?[])
    (remap-AltGrp-key "]"  [gold ?]])
    (remap-AltGrp-key "\\" [gold ?\\])
    (remap-AltGrp-key ""  [gold pound])
;;  (remap-AltGrp-key ""  [gold ?])
)


;; X-Window Machines (make them sending vt100/320-KeyCodes):
;;-----------------------------------------------------------
(cond ((string-match "hpterm"       XTERM) (load "gold_Xhpterm"))
      ((string-match "dtterm"       XTERM) (load "gold_Xsun"))
      ((string-match "sgi\\|iris"   XTERM) (load "gold_Xsgi"   ))
      ((or (string-match "Num_Lock"     xmodmap-pk)
           (string-match "Mode_switch"  xmodmap-pk)
           (string-match "Scroll_Lock"  xmodmap-pk)
           (string-match "Xpc\\|ibm"    XTERM)) (load "gold_Xpc"))
      ((string-match "sun\\|sparc"  XTERM) (load "gold_Xsun"))
      ((string-match "PC-"          keyboard-type) (load "gold_Xpc"))
      (t (message "Standard-X-Window only !")
        ;(clear-strange-KP-keys)
         (define-key global-map [delete] 'delete-previous-char) ;      <BackSp>
         (define-key GOLD-map   [delete] 'delete-other-window)  ; GOLD <BackSp>
         (define-key global-map [remove] 'delete-current-char)  ;      <Remove>
         (define-key GOLD-map   [remove] 'cut-context-to-paste) ; GOLD <Remove>
         (define-key global-map [f5]     (key-binding [f11]))
         (sleep-for 1)))


;;; Mouse-Bindings/Functions and -Menues:
;;;---------------------------------------
(load "gold_Xmouse" nil t)


;;; Some additional X11-Settings:
;;;------------------------------
(setq transient-mark-mode t)    ; Use temporary region highlighting
(setq mark-even-if-inactive t)  ; But do not remove mark after unhighlighting
(require 'paren)
;(or xemacs (load "multi-click" t t)) ; Relaxed (require 'multi-click) for XEmacs

;;----------------------------------------------------------------------------
(provide 'gold_Xterm)
