;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Compilation interface and utilities !                               ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;;
;;;----------------------------------------------------------------------------
(gold-debug "gold_compile")
(require 'compile)
;;;----------------------------------------------------------------------------
(defvar compile-buffer-function nil
"Function called (if non-nil) to perform `compile-buffer'.
Each buffer has its own value of this variable.")
;;
(make-variable-buffer-local 'compile-buffer-function)
;;
(setq-default compile-buffer-function nil)
;;;----------------------------------------------------------------------------
(defun eval-buffer-or-region ()
"Execute the region or - if no region defined - the current buffer as Lisp code
(short-cut interface to `eval-region' respectively `eval-current-buffer')."
  (interactive)
  (let ((type "REGION") (start (region-beginning)) (end (region-end)))
       (if (or (not mark-active)
               (region-equal-buffer start end)) (setq type "BUFFER"))
       (message "Executing Lisp code in current %s !" type)
       (eval-region start end nil) (setq mark-active nil)))
;;
(defun recompile-dir ()
"Recompile every `.el' file in DIRECTORY that needs recompilation.
(short-cut interface to `byte-recompile-directory')."
  (interactive)
  (let ((save-case-fold-search case-fold-search))
       (save-excursion (call-interactively 'byte-recompile-directory))
       (setq case-fold-search save-case-fold-search)
))
;;;----------------------------------------------------------------------------
(defun compile-buffer () " " (interactive)
  (cond (compile-buffer-function (funcall compile-buffer-function))
        ((equal major-mode 'jde-mode) (jde-compile))
        (t (call-interactively 'compile))))
;;
(defun my-compile-buffer (command)
  (save-window-excursion (save-buffer)
                         (setq compile-command command)
  (compile-internal (concat command " " (buffer-name)) "No more errors")))

;; Called when compilation process changes state.
(defun compilation-sentinel (proc msg)
  "Sentinel for compilation buffers."
  (let ((buffer (process-buffer proc)))
    (if (memq (process-status proc) '(signal exit))
        (progn
          (if (null (buffer-name buffer))
              ;; buffer killed
              (set-process-buffer proc nil)
            (let ((obuf (current-buffer))
                  omax opoint)
              ;; save-excursion isn't the right thing if
              ;; process-buffer is current-buffer
              (unwind-protect
                  (progn
                    ;; Write something in the compilation buffer
                    ;; and hack its mode line.
                    (set-buffer buffer)
                    (let ((buffer-read-only nil))
                      (setq omax (point-max)
                            opoint (point))
                      (goto-char omax)
                      ;; Record where we put the message, so we can ignore it
                      ;; later on.
                      (insert ?\n mode-name " " msg)
                      (forward-char -1)
                      (insert " at " (substring (current-time-string) 0 19))
                      (forward-char 1)
                      (setq mode-line-process
                            (concat ": "
                                    (symbol-name (process-status proc))))
                      ;; Since the buffer and mode line will show that the
                      ;; process is dead, we can delete it now.  Otherwise it
                      ;; will stay around until M-x list-processes.
;!!!
                 (let ((visible-bell t)) (beep) ;;;;;!!!!!!!!!!!!
                      (message (concat ">>> Compilation "
                               (substring msg 0 -1) " !"))
                      (sit-for 1)(beep)) ;;;;;!!!!!!!!!!!!
;!!!
                      (delete-process proc)
                      ;; Force mode line redisplay soon.
                      (set-buffer-modified-p (buffer-modified-p)))
                    (if (and opoint (< opoint omax))
                        (goto-char opoint))
                    (if compilation-finish-function
                        (funcall compilation-finish-function buffer msg)))
                (set-buffer obuf))))
          (setq compilation-in-progress (delq proc compilation-in-progress))
          ))))
;;-----------------------------------------------------------------------------
(provide 'gold_compile)
