;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein INTES
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Additional debug functionality to debug site-dependent init-files.  ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
;; Save actual Start-Time (for debug-marks):
;;-------------------------------------------
(defun float-time () (let ((time (current-time)))
                     (+ (nth 1 time) (/ (nth 2 time) 1000000.0))))
(defvar start-time (float-time) "'float-time' at start of gold.el(c)")
(defvar last-time start-time "'float-time' aof last gold-debug-mark")

;; Test on "emacs '*debug*' ..." commandline
;;-------------------------------------------
(defvar debug-site-init nil "Non-nil debugs emacs site-initialization.")
(if (equal (nth 1 command-line-args) "*debug*")
    (let ((buffer (get-buffer-create "*debug*")))
         (setq command-line-args (cons (car command-line-args)
                                       (nthcdr 2 command-line-args)))
         (setq debug-site-init t)
         (setq debug-on-error t)
         (princ "SECONDS for    STEP      SUM   > DEBUG-STRING" buffer)
         (display-buffer buffer)
         (add-hook 'window-setup-hook 'debug-site-endmark t)))
;;;
(defun debug-site-endmark ()
  (switch-to-buffer "*debug*") (delete-other-windows)
  (let ((save (point-max)))
       (gold-debug "End-Of-Site-Init ---------------------")
       (recenter -1) (goto-char save))
  (set-buffer-modified-p nil)
  (setq debug-site-init nil))

(add-hook 'window-setup-hook 'debug-whoami-on-error t)
(defun debug-whoami-on-error () "GOLD debug utility."
  (if gold-EOF-reached nil (beep) (whoami 'debug)
      (message (format "Error after gold-debug mark: %s" last-gold-debug))))

;; Standard debug-mark for site-init debugging:
;;----------------------------------------------
(setq inhibit-startup-message t) ; is reset to nil when gold-EOF is reached !
(defvar gold-EOF-reached nil "Set to t if gold EOF is reached without error")
(defvar last-gold-debug "gold-debug" "Holds last valid 'gold-debug' call")
(defun gold-debug (string &optional no-save) " "
   (or no-save (setq last-gold-debug string))
   (if debug-site-init (progn
      (let ((this-time (float-time)) (buffer (get-buffer "*debug*")))
           (set-buffer buffer) (goto-char (point-max))
           (princ (format "\ngold-debug%10f%10f> %s"
                          (- this-time last-time)
                          (- this-time start-time) string) buffer)
           (setq last-time this-time)))))
;;
(defun intes-debug ()
  (message "`intes-debug' is an obsolete command, please remove it !"))

;;----------------------------------------------------------------------------
(provide 'gold_debug)
