;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Interface to on-line dokumentations, e.g. PERMAS documentation.     ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_doc")
;-----------------
(defun doc-mode ()
  "Just normal text-mode with an own symbol to enable special context-help."
  (interactive) (text-mode) (setq mode-name "Doc-tty")
  (setq major-mode 'doc-mode) (run-hooks 'doc-mode-hook))
;-----------------
;;
(defvar permas-doc-dir "/doc/archive/permas/"
  "*Main directory for PERMAS documentation files (tty, ps, On-Line Help etc.)")
;;                     /usr/intes/doc ???? analyze 'which permas_doc' ????
;;
(defvar permas-doc-version "v13"
  "*Preferred version for PERMAS documentation, e.g. \"intern\" or \"v13\",
\"v13\", etc. (nil means default version, i.e. no version string).")
;;  Maybe we should grab the default from what is newest in permas-doc-dir ?
;;
(defun permas-doc-file (name ext)
  "Return the full name of a permas-document according to `permas-doc-version'."
   (if (stringp permas-doc-version)
       (or (permas-doc-find-file (concat name "_" permas-doc-version ext))
           (permas-doc-find-file (concat name ext)))
       (permas-doc-find-file (concat name ext))))
;;
(defun permas-doc-find-file (document)
   "Search for DOCUMENT file in `permas-doc-dir' (nil if not found)."
   (let ((path (list permas-doc-dir)) (name))
        (while (and (not name) path)
               (setq name (find-document-x (car path) document))
               (setq path (cdr path)))
        name))
;;
(defun document-search-path () "Get a list of directories to search docu."
  (append (list permas-doc-dir)
         '("/doc/archive/extern/GNU"
           "/usr/local/share/doc" "/usr/share/doc")))
;;
(defvar document-alist '(
  ("RRZN_F77"            . "RRZN: FORTRAN-77 Sprachumfang (Vol.6)")
  ("RRZN_F90"            . "RRZN: FORTRAN-90")
  ("auc-tex"             . "AUC-TeX Manual")
  ("calc"                . "GNU-Emacs Calc Manual")
  ("cl"                  . "GNU-Emacs Common Lisp Extensions")
  ("cpp"                 . "C-Pre-Processor for GCC")
  ("dejagnu"             . "Deja-GNU Test Frame")
  ("dired"               . "GNU-Emacs Directory Editor")
  ("edb"                 . "EDB Manual (GNU-Emacs DataBase Editor)")
  ("elisp"               . "GNU-Emacs Lisp Reference Manual")
  ("emacs"               . "GNU-Emacs Manual")
  ("emacs-FAQ"           . "GNU-Emacs FAQ: Frequently Asked Questens with answers)")
  ("f77"                 . "RRZN: FORTRAN-77 Sprachumfang (Vol.6)")
  ("fe_resources"        . "Internet FE-Resources")
  ("forms"               . "GNU-Emacs Forms Manual")
  ("gawk"                . "GNU-AWK Manual")
  ("gcc"                 . "GNU C(++)-Compiler Manual")
  ("gdb"                 . "GNU Source Level Debugger")
  ("giostream"           . "GNU I/O-Stream for C++")
  ("ghostscript"         . "Ghostscript Manual")
  ("ghostview"           . "Ghostview Manual")
  ("gmake"               . "GNU-Make Manual")
  ("gnats"               . "Original GNATS Manual")
  ("gnu-bulletin"        . "GNU Bulletin")
  ("gnuplot"             . "GNUPLOT Manual")
  ("gnuplot-FAQ"         . "GNUPLOT FAQ (Frequently Asked Questens with answers)")
  ("gnus"                . "GNUS Manual (GNU-Emacs NetNews-Interface)")
  ("gperf"               . "GNU Perf for C++")
  ("hypb"                . "Hyperbole Manual")
  ("ilisp"               . "Lucid-Emacs Lisp Reference Manual")
  ("info"                . "GNU Info Manual")
  ("intex"               . "LaTeX at INTES")
  ("ispell"              . "GNU-Emacs ISPELL Manual")
  ("la"                  . "PERMAS-LA")
  ("latex"               . "LaTeX Macro Package")
  ("lemacs"              . "Lucid-Emacs Manual")
  ("libg++"              . "User's Guide fo GNU C++ Library")
  ("motif++"             . "Motif++ (XPO.APO)")
  ("motif-FAQ"           . "Motif FAQ (Frequently Asked Questens with answers)")
  ("mpi-FAQ"             . "MPI FAQ (Message Passing Interface)")
  ("na-FAQ"              . "Numerical Analysis Resources and the Internet")
  ("na-digest"           . "Numerical Analysis (na-digests starting from 1987)")
  ("nas67err"            . "MSC/NASTRAN-V67: Current Error List")
  ("netlib_FAQ"          . "Netlib FAQ (Frequently Asked Questens with answers)")
  ("oleo"                . "OLEO Manual")
  ("opt"                 . "PERMAS-OPT")
  ("perl"                . "PERL Manual File (screen-formatted)")
  ("pvm"                 . "PVM User's Guide and Reference Manual")
  ("pvm-FAQ"             . "PVM FAQ: Frequently Asked Questens with answers)")
  ("recode"              . "GNU's Character-Code Recoding")
  ("send-pr"             . "SEND-PR Manual for GNATS")
  ("supercite"           . "Supercite Manual")
  ("tex-macros"          . "ASCII-List of (La)TeX and INTEX-Macros for Online-Help")
  ("texi"                . "TeX-Info Manual")
  ("transfig"            . "Trabsfig/Fig2dev Manual")
  ("vip"                 . "GNU-Emacs VIP Manual")
  ("winterp"             . "Winterp: The OSF/Motif Widget Interpreter")
  ("xlisp"               . "X-Lisp Manual")
  ("xlisp-ref"           . "X-Lisp Reference Manual")
  ("xref"                . "GNU Emacs: Xref Manual")
;
  ("asimuth97"           . "High Performance Computing in Structural Analysis")
  ("bad97"               . "???")
  ("bad98"               . "???")
  ("estec96"             . "The Parallelization of PERMAS")
  ("estec98"             . "???")
  ("hpcn95"              . "A General Approach for an Automatic Parallelization Applied to the Finite Element Code PERMAS")
  ("hpcn96"              . "Experiences and Achievements with the Parallelization of a Large Finite Element System")
  ("nafems97em"          . "A General Approach to Electromagnetic Field Analysis")
  ("nafems97par"         . "The Efficient Use of Parallel Computing with PERMAS")
  ("nasa97"              . "Run-Time Parallelization of LARGE FEM Analyses with PERMAS")
  ("elsevier98"          . "Run-Time Parallelization of LARGE FEM Analyses with PERMAS")
  ("vdi98"               . "???")
  ("vdi98col"            . "???")
  ("vdi98ecol"           . "???")
  ("vdi98e"              . "???")
;
  ("adl_secure"          . "SECURE Kurzbeschreibung")
  ("ams"                 . "PERMAS: Authorization Management System")
  ("bench"               . "PERMAS: Benchmark (English)")
  ("benchG"              . "PERMAS: Benchmark (German)")
  ("bool"                . "New Boolean Books: Concept, Data Structure & Call Reference")
  ("cached_obj"          . "PERMAS: Cached Data Objects")
  ("callref"             . "PERMAS: Call Reference")
  ("callref_intern"      . "PERMAS: Call Reference (internal)")
  ("caprog"              . "PERMAS: CA Programmers Manual")
  ("catiadoor"           . "PERMAS: Catia-Door User's Manual")
  ("catiadoor_intern"    . "PERMAS: Catia-Door User's Manual (intern)")
  ("catiadoor_v13"       . "PERMAS: Catia-Door User's Manual (v13)")
  ("cm"                  . "PERMAS: Core Memory Slot Manager")
  ("dataref"             . "PERMAS: Data Structures")
  ("data_basdef"         . "PERMAS: Basic Definitions")
  ("data_basmod"         . "PERMAS: Basic Module")
  ("data_conmod"         . "PERMAS: Connectivity Module")
  ("data_cstblk"         . "PERMAS: Constraint Definition Block")
  ("data_cstmod"         . "PERMAS: Constraint Module")
  ("data_felix"          . "PERMAS-Felix: Data Structure & Calls")
  ("data_lodblk"         . "PERMAS: Loading Definition Block")
  ("data_lodmod"         . "PERMAS: Loading Module")
  ("data_modblk"         . "PERMAS: Modification Definition Block")
  ("data_modmod"         . "PERMAS: Modification Module")
  ("data_resblk"         . "PERMAS: Result Specification Block")
  ("data_resmod"         . "PERMAS: Result Module")
  ("data_sysblk"         . "PERMAS: System Definition Block")
  ("data_sysmod"         . "PERMAS: System Module")
  ("data_utiblk"         . "PERMAS: Utility Data Block")
  ("dataref_intern"      . "PERMAS: Data Structures (internal)")
  ("ddms"                . "PERMAS: Distributed Data Management System")
  ("ddms_intern"         . "PERMAS: DDMS Internals")
  ("dev_request"         . "PERMAS: Development Requests")
  ("excalls"             . "PERMAS: Export Calls")
  ("FAQ"                 . "PERMAS: Frequently Asked Questens (with answers)")
  ("f7x"                 . "F7X for PERMAS - Some F90 Extensions to FORTRAN77")
  ("feti"                . "FETI method in PERMAS")
  ("f90"                 . "Fortran 90 for PERMAS")
  ("gold_emacs"          . "Emacs: GOLD-Keypad and -functions")
  ("ideascall"           . "I-DEAS Door: Call Reference & Data Structures")
  ("ideasdoor"           . "PERMAS: I-DEAS-Door User's Manual")
  ("ideasdoor_intern"    . "PERMAS: I-DEAS-Door User's Manual (intern)")
  ("ideasdoor_v13"       . "PERMAS: I-DEAS-Door User's Manual (v13)")
  ("info"                . "PERMAS: Short Introduction (English)")
  ("infoG"               . "PERMAS: Kurzbeschreibung (German)")
  ("infoF"               . "PERMAS: Description Resumee (French)")
  ("unix_install"        . "PERMAS: Installation on Unix")
  ("unix_install_v13"    . "PERMAS: Installation on Unix")
  ("kolbenbolzen"        . "???")
  ("lower_calls"         . "PERMAS: Lower Level Calls")
  ("maintenance"         . "PERMAS: Maintenance")
  ("matio"               . "PERMAS: Matrix I/O")
  ("matlab"              . "PERMAS: Matlab output")
  ("medidoor"            . "PERMAS: MEDINA-Door User's Manual")
  ("medidoor_intern"     . "PERMAS: MEDINA-Door User's Manual (intern)")
  ("medidoor_v13"        . "PERMAS: MEDINA-Door User's Manual (v13)")
  ("multigrid"           . "Autobench: GMD Multigrid-Solver in PERMAS (internal)")
  ("nascall"             . "NASTRAN-Door: Call Reference & Algorithms")
  ("nasdat"              . "NASTRAN-Door: Data Structure")
  ("nasdoor"             . "PERMAS: NASTRAN-Door User's Manual")
  ("nasdoor_intern"      . "PERMAS: NASTRAN-Door User's Manual (intern)")
  ("nasdoor_v13"         . "PERMAS: NASTRAN-Door User's Manual (v13)")
  ("output"              . "PERMAS: Output")
  ("patcall"             . "PATRAN-Door: Call Reference & Data Structures")
  ("patdoor"             . "PATRAN-Door: User's Manual")
  ("patdoor_intern"      . "PATRAN-Door: User's Manual (intern)")
  ("patdoor_v13"         . "PATRAN-Door: User's Manual (v13)")
  ("pcgsolv"             . "Block-PCG method in PERMAS")
  ("perm4_FAQ"           . "PERMAS-4: Frequently Asked Questens (with answers)")
  ("perm4_ca"            . "PERMAS-4: CA - Analysis of Linear Contact Problems")
  ("perm4_fsexa"         . "PERMAS-FS: Examples Maunal")
  ("perm4_fstheory"      . "PERMAS-FS: Theory Maunal")
  ("perm4_fsuser"        . "PERMAS-FS: User Maunal")
  ("perm4_la"            . "PERMAS-4: LA User Manual")
  ("perm4_ra"            . "PERMAS-4: RA - Reliability Analysis")
  ("perm4_steps"         . "PERMAS-4: STEPS")
  ("perm520_relnote"     . "Release Note PERMAS 5.2")
  ("perm6_relnote"       . "Release Note PERMAS 6")
  ("perm605_relnote"     . "New CA-Features in PERMAS Version 6.05")
  ("perm610_relnote"     . "Release Note PERMAS 6.1")
  ("pipe"                . "Pipe Element (UM403)")
  ("pmp"                 . "PERMAS: Message Passing & Tag Catalog")
  ("pom"                 . "PERMAS: Parallel Operation Manager")
  ("ptm"                 . "PTM, Parallel Task Manager: Call Reference")
  ("ptm_intern"          . "PTM, Parallel Task Manager: Call Reference (internal)")
  ("relnote"             . "PERMAS Release note")
  ("relnote_v10"         . "PERMAS Release note (v10)")
  ("relnote_v11"         . "PERMAS Release note (v11)")
  ("relnote_v12"         . "PERMAS Release note (v12)")
  ("relnote_v13"         . "PERMAS Release note (v13)")
  ("suspit"              . "PERMAS Subspace Iteration")
  ("verify"              . "PERMAS System Verification Manual")
  ("um150"               . "PERMAS Programmer's Manual (um150)")
  ("um150_v13"           . "PERMAS Programmer's Manual (um150_v13)")
  ("um450"               . "PERMAS User's Reference Manual (um450)")
  ("um450_intern"        . "PERMAS User's Reference Manual (internal)")
  ("um450_v13"           . "PERMAS User's Reference Manual (um450_v13)")
  ("um460"               . "FELIX User's Reference Manual (um460)")
  ("um460_intern"        . "FELIX User's Reference Manual (internal)")
  ("um460_v12"           . "FELIX User's Reference Manual (um460_v12)")
  ("um470"               . "VisPER User's Manual")
  ("um470_v2"            . "VisPER User's Manual (um470_v2)")
  ("um550"               . "PERMAS Examples Manual (um550)")
  ("um550_intern"        . "PERMAS Examples Manual (internal)")
  ("um550_v13"           . "PERMAS Examples Manual (um550_v13)")
  ("unix"                . "???")
  ("virtuallab_edo"      . "PERMAS EDO-Interface (Virtual Lab)")
;
  ("patdoor"             . "PATRAN-Door: User's Manual")
  ("pat3dat"             . "PATRAN-Door: Call Reference & Data Structures")
  ("patint"              . "PERMIT: PATRAN/PERMAS-Interface")
  ("patran3-pref"        . "PERMAS Preferences fo PATRAN-3")
  ("supdoor"             . "SUPERTAB-Door: User's Manual")
  ("supint"              . "PERMIT: SUPERTAB/PERMAS-Interface")
)  "Association list of known documents in form: (ABBREV . TITLE)")
;;
(defun find-document (document)
   "Search for DOCUMENT file in `document-search-path' (nil if not found)."
   (let ((path (document-search-path)) (name))
        (while (and (not name) path)
               (setq name (find-document-x (car path) document))
               (setq path (cdr path)))
        name))
;;
(defun find-document-x (dir file) "Auxiliary function for `find-document'."
   (let ((name (if (string-equal "/" (substring dir -1))
                   (concat dir file) (concat dir "/" file))))
        (if (file-readable-p name) name
            (if (file-readable-p (concat name ".gz")) (concat name ".gz")
                (if (file-readable-p (concat name ".Z")) (concat name ".Z")
)))))
;;
(defun insert-document (document)
  "Variable extension must be valid (must be a non-nil string) ! "
  (let ((file (find-document document)))
       (if file (insert-file-contents file))))
;;
(defun load-document (abbrev)
  "Load document into its buffer if not loaded (returns nil if not loadable)."
  (let ((buffer (concat "*" abbrev "*"))
        (doc-alist (assoc abbrev document-alist)))
       (if (get-buffer buffer) (set-buffer buffer)
           (unconditional-new-buffer buffer nil)
           (if (insert-document (concat (car doc-alist) ".tty"))
               (progn (doc-mode) (set-buffer-modified-p nil)
                                 (setq buffer-read-only t))
               (message "Sorry, document \"%s\" not available !" abbrev)
               (kill-buffer buffer) nil))))
;;
(defun show-document () " "
   (interactive)
   (show-Document (completing-read "Abbreviation for document: "
                    document-alist 'find-document-p)))
;;
(defun find-document-p (alist) (find-document (concat (car alist) ".tty")))
;;
(defun show-Document (abbrev) " "
   (interactive)
   (let ((buffer (concat "*" abbrev "*"))
         (doc-alist (assoc abbrev document-alist)))
        (if (get-buffer buffer) (switch-to-buffer buffer)
            (unconditional-new-buffer buffer t)
            (if (insert-document (concat (car doc-alist) ".tty"))
                (message "Viewing \"%s\"." (cdr doc-alist))
                (message "Sorry, document not available !"))
            (set-buffer-modified-p nil) (setq buffer-read-only t)
            (goto-char (point-min)) (advance-direction) (doc-mode))))
;
(defun print-headline-and-page (abbrev &optional pre key underline)
"Print next available Headline with page-number and document-title.
Underline the headline if optional UNDERLINE (4th arg) is non-nil."
  (let ((line "No Title")) (end-of-line)
       (if (re-search-backward "^[$0-9A-Za-z]" nil t)
           (setq line (buffer-linestring)))
       (if key (progn
               (princ (concat pre " \"" key "\":  " line))
               (if underline (princ (concat "\n"
                  (make-string (+ (length pre) 1) ?-)
                  (make-string (+ (length key) 3) ?=)
                  (make-string (+ (length line) 3) ?-))))
               (princ (concat "\nPage " (page-number) " of \""
                      (cdr (assoc abbrev document-alist)) "\"")))
          (if (> (length line) 60) (setq line (substring line 0 61)))
          (princ (concat line ":  Page " (page-number) " of \""
                  (cdr (assoc abbrev document-alist)) "\""))
          (if underline (princ (concat "\n"
              (make-string (+ (length line) 1) ?-)))))))
;
(defun page-number () "Return TeX page number of actual tty-page"
  (save-excursion
  (let ((head)(foot))
       (if (not (search-forward "^L" nil t)) (goto-char (point-max)))
       (forward-line -1) (setq foot (buffer-linestring))
       (if (string-match "[0-9A-Z]+-[0-9]+" foot)
           (substring foot (match-beginning 0) (match-end 0))
           (if (not (search-backward "^L" nil t)) (goto-char (point-min)))
           (forward-line 1) (setq head (buffer-linestring))
           (if (string-match "[0-9A-Z]+-[0-9]+" head)
               (substring head (match-beginning 0) (match-end 0))
               (if (string-match "\\b[iIvVxX]+\\b" foot)
                   (substring foot (match-beginning 0) (match-end 0))
                   (if (string-match "\\b[iIvVxX]+\\b" head)
                       (substring head (match-beginning 0) (match-end 0))
                       (eval "?-??"))
       ))))))
;
(defun page-number-at-point ()
"Returns a string with the next available page-number around point."
  (condition-case () (save-excursion
   (let ((pos1)(pos2))
        (if (equal (following-char) ? ) (forward-word 1))
        (forward-char 1) (forward-word -1)
        (if (equal (preceding-char) ?-) (forward-word -1))
        (setq pos1 (point))
        (skip-chars-forward "0-9A-Z\-") (setq pos2 (point))
        (if (< pos1 pos2) (buffer-substring pos1 pos2) (eval "0"))))
   (error nil)))
;
(defun show-page ()
  "Show (goto) the page for the page-number read from actual point position."
   (let*((page (page-number-at-point))                  ;; Page numbers at:
         (regexp1 (concat "\\b" page "\\b.*\n\\^L\n"))  ;; -> bottom of page
         (regexp2 (concat "\n\\^L\n.*\\b" page "\\b"))) ;; -> top of page
        (if (not (or (if (or (re-search-backward regexp1 nil t)
                             (progn (re-search-forward regexp1 nil t)
                                    (search-backward "^L" nil t) t))
                         (progn (search-backward "^L" nil t) t))
                     (re-search-backward regexp2 nil t)
                     (re-search-forward regexp2 nil t)))
            (message (concat "No such page " page " !"))
            (forward-line 1) (line-to-top-of-window)
            (message (concat "Cursor at top of page " page " !")))))

;;----------------------------------------------------------------------------
(provide 'gold_doc)
