;;; gold_f7x.el --- fortran/f77/f90-mode modifications for gold-package

;; Copyright (C) 1999 Free Software Foundation, Inc.

;; Author: Markus Ast <ast@intes.de>
;; Last Change: 13-Sep-11
;; Keywords: gold, fortran-mode, f77-mode, f7x-mode, f90-mode, editing

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; Some additions/modifications to fortran/f77/f90-mode
;; To be used with gold-library (gold.el[c]).

;;; Code:

(gold-debug "gold_f7x")
;;----------------------------------------------------------------------------
(require 'fortran)
(require 'f90)
;;
(defvar f7x-regexp
   "^ *!\\|^ *module\\|^ *use\\|^ [^!']*::"
   "REGEXP to detect f7x code.")
;;
;;;###autoload
(defun f77-mode () "Major mode for editing FORTRAN77 code."
  (interactive) (fortran-mode)
  (save-excursion (goto-char (point-min))
    (if (search-forward "\n!" nil t) (progn (beep) (message
       "%s also contains some f7x or f90 extensions !" (buffer-name)))))
  (setq mode-name "F77") (setq major-mode 'f77-mode)
  (setq comment-line-start-skip "^[Cc*!#]") (setq comment-line-start "C") ; emacs20
  (setq comment-start-skip "^[Cc*!#]") (setq comment-start "C")           ; emacs21
  (setq fortran-comment-region "C>")
  (f7x-mode-settings))
;;
;;;###autoload
(defun f7x-mode () "Major mode for editing Fix-Field F77/Fortran90 code."
  (interactive) (fortran-mode)
  (setq mode-name "f7x") (setq major-mode 'f7x-mode)
  (setq comment-line-start-skip "^[Cc*!#]") (setq comment-line-start "!") ; emacs20
  (setq comment-start-skip "^[Cc*!#]") (setq comment-start "!")           ; emacs21
  (setq fortran-comment-region "!>")
  (f7x-mode-settings))
;;
(defun f7x-mode-settings () "Common settings for `f7x-mode' and `f77-mode'."
  (local-set-key "\e\C-a" 'beginning-of-fortran-subprogram)
  (local-set-key "\e\C-e" 'end-of-fortran-subprogram)
  (setq fortran-continuation-string "&")
  (set-syntax-table f7x-mode-syntax-table)
  (setq font-lock-defaults '(f7x-font-lock-keywords t t))
)
;;
;;;###autoload
(defun f7x2f77-mode () "Major mode for viewing f7x2f77 converted fortran-code."
  (interactive) (f7x-mode)
  (setq mode-name "f7x2f77") (setq major-mode 'f7x2f77-mode)
  (setq buffer-read-only t))
;;
(defvar f7x-mode-syntax-table nil
  "Syntax table in use in f7x-mode buffers.")
;;
(if f7x-mode-syntax-table
  ()
  (setq f7x-mode-syntax-table (make-syntax-table))
  (modify-syntax-entry ?\! "<"  f7x-mode-syntax-table)  ; beg. comment
  (modify-syntax-entry ?\n ">"  f7x-mode-syntax-table)  ; end comment
  (modify-syntax-entry ?\r " "  f7x-mode-syntax-table)  ; return is whitespace
  (modify-syntax-entry ?\" "\"" f7x-mode-syntax-table)  ; string quote
  (modify-syntax-entry ?\' "\"" f7x-mode-syntax-table)  ; string quote
  (modify-syntax-entry ?\' "."  f7x-mode-syntax-table)  ; ADL-Header is no string
  (modify-syntax-entry ?\% "."  f7x-mode-syntax-table)  ; Separate field names
  (modify-syntax-entry ?\; "w"  f7x-mode-syntax-table)  ; Abbrev starter
  (modify-syntax-entry ?+  "."  f7x-mode-syntax-table)
  (modify-syntax-entry ?-  "."  f7x-mode-syntax-table)
  (modify-syntax-entry ?=  "."  f7x-mode-syntax-table)
  (modify-syntax-entry ?*  "."  f7x-mode-syntax-table)
  (modify-syntax-entry ?/  "."  f7x-mode-syntax-table)
  (modify-syntax-entry ?_  "w"  f7x-mode-syntax-table)
)
;;
(setq f7x-font-lock-keywords-1
  (list
    '("^[C!][*](.*$" .  'font-lock-match-face)          ; ADL separator line
    '("^[C!]TOC .*$" .  'font-lock-string-face)         ; ADL TOC line
    ;;
    ;; Special highlight for ENTRY names
    '("^     *\\(ENTRY\\) *\\([^ ()]*\\)" (1 'font-lock-keyword-face) (2 'font-lock-match-face t))
    ;;
    ;; Copyright-lines
    '("^\\([!Cc*] (C) Copyright.*\\) \\([0-9]+\\)$"
      (1 'font-lock-comment-face) (2 'font-lock-reference-face))
    '("^\\([!Cc*].*\\) \\(Copyright.*INTES.*\\)$"
      (1 'font-lock-comment-face) (2 'font-lock-mismatch-face))
    ;;
    ;; Declaration comments
    '("^\\([!Cc*]  ?[<>-][%?-]+[<>-]\\)  ?\\([^ :-]+\\)\\(.*\\)$"
      (1 'font-lock-comment-face) (2 'font-lock-variable-name-face)
      (3 'font-lock-comment-face nil t))
    '("^[!Cc*].*$"  .  'font-lock-comment-face)         ; F77/90 comment lines
    '(fortran-match-!-comment . font-lock-comment-face) ; Trailing F90 comments
    '("'[^'\n]*'?"  . font-lock-string-face)            ; Strings ???
    '("%[a-z0-9_]*" . 'font-lock-field-name-face)       ; F90 field names
    '("^     \\([^ 0]\\)" 1 'font-lock-string-face)     ; Continuation character
    ;;
    ;; Character*(*) variables
    '("^[ \t]*\\(character[(*)0-9]*\\)\\(.*::\\|[ \t]*(.*)\\)?\\([^!'\n]+\\)"
     (1 font-lock-type-face) (3 font-lock-variable-name-face))
    '("\\([<>]\\|[<=/>]=\\)" . 'font-lock-keyword-face)
    '("^[ \t]*data"          . 'font-lock-keyword-face)
    '("^[ \t]*\\(common\\)[ \t]*\/ *\\([^ ]*\\)? *\/"
      (1 'font-lock-keyword-face) (2 'font-lock-reference-face nil t))
    ;;
    ;; Variable declarations (double XXXX missing in f90.el)
    '("^[ \t]*\\(double *\\(precision\\|complex\\)\\)\\(.*::\\|[ \t]*(.*)\\)?\\([^!'\n]+\\)"
      (1 'font-lock-type-face) (4 'font-lock-variable-name-face nil t))
    ;; Type() variables
    '("^[ \t]*\\(type\\) *(\\([^)]*\\))\\(.*::\\|[ \t]*(.*)\\)?\\([^!'\n]+\\)"
     (1 'font-lock-type-face) (2 'font-lock-field-name-face nil t)
                              (4 'font-lock-variable-name-face nil t))
    ;;
    ;; Obsolete calls/constructs (to be replaced)
    '("\\(CALL\\) \\(CV[^ ]*A4[^ ]*\\)\\b"
        (1 'font-lock-keyword-face) (2 'font-lock-mismatch-face))
    '("\\(CALL\\) \\(CVCH[^ ]*A4\\) *("
        (1 'font-lock-keyword-face) (2 'font-lock-mismatch-face))
;
   '("\\(CALL\\) *\\(ED\\(INIT\\|FIND\\|GLOD\\|LOAD\\|NAME\\|READ\\|TERM\\)\\)"
     (1 'font-lock-keyword-face) (2 'font-lock-mismatch-face))
;
    ;; FMT-formats
;   '("'.*[^%]\\(%%\\)[^%]" (1 'font-lock-keyword-face t))
    '("%&"                                                (0 'font-lock-keyword-face t))
    '("%[#0<=>+-]* ?[#0<=>+-]*[0-9.]*[DLP]?[sefgibox]\\b" (0 'font-lock-keyword-face t))
;   '("%[ #0<=>+-]*[0-9.]*[DhlLP]?[bcdefginopsux]\\b"     (0 'font-lock-keyword-face t))
    '("[^%]\\(%[#+-<=> ]?[0-9.]*([^)]*)\\)"               (1 'font-lock-keyword-face t))
;    '("%[a-z0-9]*" (0 'font-lock-field-name-face t))    ; F90 field names
))
;;
(setq f7x-font-lock-keywords-x ; post-highlighting that will
  (list                        ; overwrite all previous ones
    ;;
    ;; Generic tokens
    '("^ +\\(@VAR *{.*[^ ]\\) *$" (1 'font-lock-gold-face t))
    '("<[a-z][][a-z2()]*>"        (0 'font-lock-gold-face t))
    ;;
    ;; mark parts beyond 72nd column
    (list (concat (make-string 72 ?.) "\\(.*\\)")       ; mark parts beyond
         '(1 'font-lock-mismatch-face t))               ;       72nd column
))
;;
(setq f90-font-lock-keywords-3
  (append f90-font-lock-keywords-2
   (list
    f90-keywords-level-3-re
    f90-operators-re
    (list f90-procedures-re '(1 font-lock-function-name-face))
    "\\<real\\>"                        ; Avoid overwriting real defs.
    )))
;;
(defvar f7x-font-lock-keywords nil
  "Highlight declarations, do-loops and other constructions")
(setq f7x-font-lock-keywords
  (append f7x-font-lock-keywords-1 f90-font-lock-keywords-3 f7x-font-lock-keywords-x))
;;
;;From: simon@gnu (Simon Marshall)
;; Find the next ! not in a string.
(defun fortran-match-!-comment (limit)
  (let (found)
    (while (and (setq found (search-forward "!" limit t))
                (fortran-is-in-string-p (point))))
    (if (not found)
        nil
      ;; Cheaper than `looking-at' "!.*".
      (set-match-data
       (list (1- (point)) (progn (end-of-line) (min (point) limit))))
      t)))
;;
;;----------------------------------------------------------------------------
(setq fortran-tab-mode-string "FTAB") ;; String to appear in mode line when TAB format mode is on.")
(setq fortran-do-indent 3)            ;; Extra indentation applied to DO blocks
(setq fortran-if-indent 3)            ;; Extra indentation applied to IF blocks
(setq fortran-continuation-indent 15) ;; Extra indent to continuation lines
(setq fortran-comment-indent-style nil) ;;
(setq fortran-comment-line-column 6)  ;; Indentation for text in comment lines
(setq fortran-minimum-statement-indent 6) ;; Minimum indentation for Fortran statements.")
(setq fortran-line-number-indent 5)
(setq fortran-check-all-num-for-matching-do t)
(setq fortran-continuation-string "&")
;;;
;;----------------------------------------------------------------------------
(defun beginning-of-fortran-subprogram ()
  "Moves point to the beginning of the current Fortran subprogram."
  (interactive)
  (let ((case-fold-search t))
    (beginning-of-line 0) (re-search-backward
    "^[ \t0-9]*end[ \t]*\\(fun\\|sub\\|mod\\|pro\\|$\\)\\|^[cC*!]\\*(\\$) "
      nil 'move)
    (if (looking-at "^[ \t0-9]*end") (forward-line 1)))
  (eval (point)))
;;;
(defun end-of-fortran-subprogram ()
  "Moves point to the end of the current Fortran subprogram."
  (interactive)
  (let ((case-fold-search t))
    (beginning-of-line 2) (re-search-forward
    "^[ \t0-9]*end[ \t]*\\(fun\\|sub\\|mod\\|pro\\|$\\)\\|^[cC*!]\\*(\\$) "
    nil 'move)
    (goto-char (match-beginning 0)) (forward-line 1))
  (eval (point)))
;;
(defun end-of-fortran-header ()
  "Moves point to the end of the current Fortran subprogram's header."
  (interactive)
  (let (end (beginning-of-fortran-subprogram))
       (beginning-of-fortran-subprogram)
       (re-search-forward "opyright" end t)
       (beginning-of-line -1))
  (eval (point)))
;;
;;----------------------------------------------------------------------------
(defun commenttab () "Tabstop at column 30 for comments in FORTRAN programs"
  (interactive) (beginning-of-line 1) (move-or-extend 29))
;;
(defun gototab () "Tabstop at column 50 for GO TO in FORTRAN programs"
  (interactive) (beginning-of-line 1) (move-or-extend 49))
;;
;;----------------------------------------------------------------------------
(add-hook 'fortran-mode-hook 'fortran-mode-ON nil)
(defun fortran-mode-ON () "Additionals when changing to fortran-modes"
  ;;
  (define-key fortran-mode-map "\t" 'fortran-tab-left-right)
  (define-key fortran-mode-map "\n" 'scroll-up-down) ; Qume-"LFD/QUME"
  (substitute-key-definition 'split-line          'fortran-split-line
                                                   fortran-mode-map global-map)
  (define-key fortran-mode-map [gold f7] 'fortran-join-line)
  (substitute-key-definition 'newline-and-indent  'fortran-newline-indent
                                                   fortran-mode-map global-map)
  (substitute-key-definition 'self-insert-command 'fortran-self-insert-command
                                                   fortran-mode-map global-map)
  (substitute-key-definition 'fortran-electric-line-number
                                                  'fortran-self-insert-command
                                                   fortran-mode-map global-map)
  ;;
  (setq context-help-function   'fortran-context-help)
  (setq context-update-function 'fortran-context-update)
  (setq context-work-function   'fortran-to-tex)
  (setq compile-buffer-function 'fortran-compile-buffer)
  (make-local-variable 'comment-start) (setq comment-start "*")
  (modify-syntax-entry ?. "." fortran-mode-syntax-table)
  ;;
  (make-local-variable 'kf-def-regexp)
  (setq kf-def-regexp "^       *\\(MODULE\\SUBROUTINE\\|FUNCTION\\) ")
  (abbrev-mode +1))    ;Standard is abbreviation-table ON
;;----------------------------------------------------------------------------
;;
(defun fortran-compile-buffer () " "
  ;; kill optionally existing xxxx.f77.f buffer visiting previous f7x outputs
  (let* ((f77-file (concat (file-name-sans-extension (buffer-file-name)) ".f77.f"))
         (f77-buffer (get-file-buffer f77-file)))
        (if f77-buffer (kill-buffer f77-buffer)))
  ;; Invoke compile command
  (my-compile-buffer fortran-compile-command)
  (message (concat fortran-compile-message " of " (buffer-name) " started !")))
;;
(defvar fortran-compile-command
  (cond ((file-executable-p "/usr/local/bin/ftest") (concat "ftest -" DEFPROG))
        ((and (file-executable-p "/usr/local/bin/f7x2f77")
              (file-executable-p "/usr/local/bin/f7x")) "f7x")
        ((equal system-type 'hpux) "f77 -c -w66")
        (t "f77 -c")) "*Local compile-command.")
;
(defvar fortran-compile-message "F77-Compilation" "*Local compile-message.")
;;----------------------------------------------------------------------------
;; Context-help:
;;--------------
;;;###autoload
(defun fortran-context-help ()
  "Show a context-sensitive help for point position in fortran-mode."
  (interactive)
  (setq this-command 'fortran-context-help)
  (cond
   ;; 'fortran-context-help' typed twice at same position => describe-mode
   ((eq last-command 'help-permas-format-group)
        (describe-mode) (setq this-command 'dummy))
   ;;
   ((eq last-command this-command)
    (if (save-excursion (beginning-of-line 1)
        (looking-at ".*call *\\(err\\|out\\)[c0-9]* *(.+, *[0-9][0-9][0-9] *[0-9][0-9][0-9][0-9]"))
        (help-permas-format-group)
        (describe-mode) (setq this-command 'dummy)))
   ;;
   ((save-excursion (beginning-of-line 1)
    (looking-at "[!Cc*]\\(fmt\\|[*]10\\|[*]600\\)"))
    (help-permas-format-group))
   ;;
   ((and (eq last-command 'help-permas-format-group))
         (save-excursion (looking-at "call *\\(err\\|out\\)"))
    (help-permas-format-group) (setq this-command 'help-permas-format-group))
   ;;
   ;; in f7x2f77 output: try to locate original source code based on line number
   ((equal major-mode 'f7x2f77-mode)
    (let ((f7x-file (concat (file-name-sans-extension
                            (file-name-sans-extension (buffer-file-name))) ".f"))
          (line (save-excursion (end-of-line) (string-to-number (integer-at-point)))))
         (find-file-other-window f7x-file) (goto-line line)))
   ;;
   ;; Any other position: Try help-for-call
   (t (help-fortran-call) (setq this-command 'fortran-context-help)))
)
;;
(defun help-permas-format-group () "Developer's help for PERMAS Error Group."
  (interactive)
  (setq this-command 'help-permas-format-group)
  (let ((bound (save-excursion (beginning-of-line -3) (point))))
  (if (save-excursion (end-of-line 1) (re-search-backward
   "call *\\(err\\|out\\)[c0-9]* *(.+, *\\([0-9][0-9][0-9]\\) *\\([0-9][0-9][0-9][0-9]\\)" bound t))
      (let ((my-buffer (buffer-name)) (remark "!")
            (file "unknown") (local "unknown")
            (caltyp   (buffer-substring (match-beginning 1) (match-end 1)))
            (group    (buffer-substring (match-beginning 2) (match-end 2)))
            (number   (buffer-substring (match-beginning 3) (match-end 3))))
      (if (equal number "") (setq number "0000"))
      (save-excursion (beginning-of-line 1)
        (if (equal group "010")
            (if (looking-at "[!Cc]fmt")    (setq remark "(should be a `!*10' line) !"))
            (if (looking-at "[!Cc][*]10")  (setq remark "(must *NOT* be a `!*10' line) !")))
        (if (equal group "600")
            (if (looking-at "[!Cc]fmt")    (setq remark "(should be a `!*600' line) !"))
            (if (looking-at "[!Cc][*]600") (setq remark "(must *NOT* be a `!*600' line) !"))))
      (if (permas-formats-buffer) (progn
          (if (re-search-forward (format "^ +%s +%s" group number) nil t)
              (message (format "OLD format %s %s %s" group number remark))
              (re-search-forward (format "<GRP ID=%s" group) nil t)
              (message (format "NEW format %s %s %s" group number remark)))
          (line-to-top-of-window)
          (switch-to-buffer-other-window my-buffer)))
))))

;;----------------------------------------------------------------------------
;; Context-Update:
;;-----------------
(defun fortran-context-update ()
  "Context sensitive update of data around point"
  (cond
    ;;
    ;; Error-line ?  -> Update error format
    ((save-excursion (beginning-of-line 1)
     (looking-at "[!Cc*]fmt \\|[!Cc][*]10 \\|[!Cc][*]600\\|.*call *\\(err\\|out\\)[c0-9]* *(.+, *[0-9][0-9][0-9] *[0-9][0-9][0-9][0-9]"))
     (let ((save (point))) (update-permas-format) (goto-char save)))
    ;;
    ;; ADL-Header-Line ?  -> Update NAMSUB
    ((save-excursion (beginning-of-line 1)
     (or (looking-at "^[!C][*]([$]) ") (looking-at "^[^\n]*NAMSUB *=")
         (looking-at "^      +\\(sub\\|.*fun\\|mod\\|pro\\)")))
     (fortran-name-update))
    ;;
    ;; Common ?  -> Update common
    ((save-excursion (beginning-of-line 1) (looking-at "^      +COMMON */.*/"))
     (fortran-common-update))
    ;;
    ;; Copyright-line -> Update copyright/version/date
    ((save-excursion (beginning-of-line 1) (looking-at "^.*copyright.*intes"))
     (fortran-update-copyright))
    ((save-excursion (beginning-of-line 2) (looking-at "^.*copyright.*intes"))
     (fortran-update-copyright))
    ((save-excursion (beginning-of-line 1) (looking-at "^[!c]!"))
     (fortran-update-copyright))
    ;;
    ;; Else ?  -> Update/insert date/time
    (t (fortran-update-date))))
;;;
(defun fortran-update-date () "Update date and (optionally) author"
  (interactive)
  (update-day-month-year)
  (let (eol)
     (end-of-line) (setq eol (point)) (beginning-of-line)
     (if (re-search-forward "-[A-Z][A-Z]-" eol t)
         (replace-match (format "-%s-" user-two-letter-name)))))
;;;
(defun fortran-update-copyright () "Update INTES copyright lines."
  (beginning-of-line 1)
  (if (save-excursion (beginning-of-line 0) (looking-at "^.*copyright.*intes")) (beginning-of-line 0))
  (if (save-excursion (beginning-of-line 2) (looking-at "^.*copyright.*intes")) (beginning-of-line 2))
  (let (author month year (version "1") (save (point)))
       ;;
       ;; guess author/month/year/version from current line(s)
       ;; -> should this be made with /source....routine (if available) ?
       (if (re-search-forward "[ -]\\([A-Z][A-Z]\\)[ -]" (+ 100 (point)) t)
           (setq author (match-string 1)))
       (or author (if (re-search-forward "kirchg"  (+ 100 (point)) t) (setq author "BK")))
       (or author (if (re-search-forward "fischer" (+ 100 (point)) t) (setq author "RF")))
       (or author (if (re-search-forward "moehring" (+ 100 (point)) t) (setq author "WM")))
       (or author (setq author "??"))
       ;;
       (if (re-search-forward "[ -]\\([a-z][a-z][a-z]\\)[ -]\\([0-9][0-9][0-9][0-9]\\)[ -]" (+ 50 (point)) t)
           (progn (setq month (match-string 1)) (setq year (match-string 2)))
           (if (re-search-forward "[ -]\\([0-9][0-9][0-9][0-9]\\)[ -]" (+ 50 (point)) t)
               (setq year (match-string 1))
               (if (re-search-forward " \\([a-z][a-z][a-z]\\)/\\([0-9][0-9]\\)" (+ 50 (point)) t)
                   (progn (setq month (match-string 1)) (setq year (concat "19" (match-string 2)))))))
;;
       (or year (setq year (substring (current-time-string) 20)))
       (if (re-search-forward "   \\([0-9]+\\)$" (+ 50 (point)) t)
           (setq version (match-string 1)))
      ;;
      ;; Remove old copyright line(s)
       (goto-char save) (insert (format (if (equal major-mode 'f7x-mode)
"!-----------------------------------------------------------------------
! (C) Copyright %s, %s, %s   %s    %4s%4s   %6s\n"
"C-----------------------------------------------------------------------
C (C) Copyright %s, %s, %s   %s    %4s%4s   %6s\n")
      company-address-name company-town company-state
      author (if month (concat month "-") "    ") year version))
      (fortran-modification-line)
      ;;
      ;; Remove old copyright line(s)
      (if (looking-at "^[!c].*copyright") (delete-current-line 1))
      (if (looking-at "^[!c]!") (delete-current-line 1))
      (if (looking-at "^[!c]--") (delete-current-line 1))
      (beginning-of-line -3)
      (if (looking-at "^[!c]--") (delete-current-line 1))
      (goto-char save)
))
;;;
(defun update-permas-format () "Update current PERMAS FMT-Format"
  (interactive)
  (let ((bound (save-excursion (beginning-of-line -3) (point))))
  (if (save-excursion (end-of-line 1) (re-search-backward
      "\\(err\\|out\\)[c0-9]* *(.+, *\\([0-9][0-9][0-9]\\) *\\([0-9][0-9][0-9][0-9]\\)" bound t))
      (let ((call (match-beginning 0)) (new-message "!FMT  ???")
            (my-buffer (buffer-name)) (file "unknown") (local "unknown")
            (caltyp   (match-string 1))
            (group    (match-string 2))
            (number   (match-string 3))
            (fxcomment-start comment-start))
      (if (equal number "") (setq number "0000"))
      (save-window-excursion
         (if (permas-formats-buffer)
         (if (re-search-forward (format "^ +%s +%s" group number) nil t)
             (setq new-message (permas-error-message-text number))
             (message (format "Format %s %s not found !" group number)))))
      (save-excursion (goto-char call) (beginning-of-line 2)
           (or (looking-at "^     [^ ]") (beginning-of-line 0))
           (new-line 1) (insert new-message)
           (beginning-of-line 2) (while (looking-at "[!Cc]\\(fmt\\|[*]10\\|[*]600\\)")
                                        (delete-current-line 1)))
      (if (equal new-message "!FMT  ???") (help-permas-format-group)))
)))
;;;
;;(defun update-permas-err-message () "Update current PERMAS-ERR to ERF Error Text"
;;  (interactive)
;;  (if (save-excursion (end-of-line 1) (re-search-backward
;;      "er[^(]*( *\\([0-5]\\) *\\([0-9][0-9][0-9]\\) *\\([0-9]+\\)" nil t))
;;      (let ((call (match-beginning 0)) start end text)
;;      (goto-char call)
;;      (re-search-forward "^[cC!]" nil t) (setq start (match-beginning 0))
;;      (re-search-forward "^.[^E]" nil t) (setq end (+ (match-beginning 0) -1))
;;;;
;;;;    Update ERR->ERF-Text
;;      (setq text (buffer-substring-no-properties start end))
;;      (setq text (replace-regexp-in-string "^.er[rf]"   "!FMT"  text))
;;      (setq text (replace-regexp-in-string "99*[.]999"  "%.3f"  text))
;;      (setq text (replace-regexp-in-string "[.]3f99*"   "f"     text))
;;      (setq text (replace-regexp-in-string "99*[.]99*"  "%f"    text))
;;      (setq text (replace-regexp-in-string "fE[+]?99*"  "E"     text))
;;      (setq text (replace-regexp-in-string ".3E"        "G"     text))
;;      (setq text (replace-regexp-in-string "99*"        "%i"    text))
;;      (setq text (replace-regexp-in-string "XXX*"       "%s"    text t))
;;      (delete-region start end) (goto-char start) (insert text)
;;;;
;;;;    Convert old ERR to new ERF-Call
;;      (setq end (- start 1))
;;      (setq text (buffer-substring-no-properties call end))
;;      (setq text (replace-regexp-in-string "^ERR"       "ERF"   text))
;;;     (setq text (replace-regexp-in-string "^ERF\\b"    "ERF0"  text))
;;;     (setq text (replace-regexp-in-string "^ERFC\\b"   "ERFC0" text))
;;      (setq text (replace-regexp-in-string "( "         "("     text))
;;      (setq text (replace-regexp-in-string " )"         ")"     text))
;;      (setq text (replace-regexp-in-string ", "         ","     text))
;;      (setq text (replace-regexp-in-string " ,"         ","     text))
;;;;
;;      (delete-region call end) (goto-char call) (insert text)
;;      (goto-char (+ call 3))
;;      (if (looking-at "[^()]*([0-5] *010 *5") (update-permas-format))
;;)))
;;;
(defun permas-error-message-text (number)
   (let ((start (match-beginning 0))
         (end   (match-end 0)))
        (if (re-search-forward "^ [0-9]\\|\\'" nil t)
            (progn (end-of-line 0) (setq end (point))))
        (replace-regexp-in-string "^[^!C]........." (concat fxcomment-start
            (if (equal group "010") "*10  " (if (equal group "600") "*600 " "FMT  ")))
            (buffer-substring start end))
))
;;----------------------------------------------------------------------------
(defun fortran-name-update (&optional newnam)
  "Update NAMSUB of current subroutine/function around point."
  (interactive) (save-excursion
  (let ((save (point)) (name (or newnam (fortran-sub-fun-name)))
        start end      (type (fortran-sub-fun-type)))
       (end-of-fortran-subprogram) (setq end (point))
       (beginning-of-fortran-subprogram) (setq start (point))
       ;; 1st replace end-of-sub (because a longer NAMSUB may fade out `end')
       (if (re-search-forward "^[ \t0-9]*end[ \t]*\\(fun\\|sub\\|mod\\|pro\\).*\n" end t)
           (replace-match (format "      END %s %s\n" type name)))
       (goto-char start)  ;; Now start replacing from top
       (if (looking-at "^[!C][*]([$]) .*\n")
           (replace-match (fortran-sub-fun-cx-line name)))
       (if (re-search-forward "\\(PARAMETER.*NAMSUB *= *'\\).*'" end t)
           (replace-match (concat "\\1" name "'")))
)))
;;----------------------------------------------------------------------------
(defun fortran-common-update (&optional noerror)
  "Update current Common Block around point."
  (beginning-of-line) (if (looking-at "^      +COMMON */.*/")
  (let ((save (point)) (name (fortran-next-common)) (text) (updated))
       (if name (setq text (fortran-common-text name)))
       (if text (progn (delete-region fortran-common-start fortran-common-end)
                       (insert text) (setq updated t))
          (if noerror nil (beep) (message
          (format "Sorry: No update found for common block /%s/ !" name))))
       (if (re-search-backward "^      +COMMON */" nil t)
           (goto-char (match-end 0)) (goto-char save))
       (eval updated)
)))
;;
(defun update-common-blocks ()
  "Update all Common Blocks with search starting at point."
  (interactive)
  (let ((found 0) (updated 0))
       (while (re-search-forward "^      +COMMON */.*/" nil t)
              (setq found (+ found 1))
              (if (fortran-common-update t) (setq updated (+ updated 1))))
       (if (< updated found)
           (message (format "%d commons found, %d of them up-to-date." found updated))
           (message (format "%d common blocks found & up-to-date." updated)))
))
;;
(defvar fortran-common-start nil "Start position of common searched for.")
(defvar fortran-common-end   nil "End position of common searched for.")
(defvar fortran-decline-regexp (concat "^      +\\(CHARACTER[*()0-9]*\\|COMPLEX\\|"
   "DIMENSION\\|DOUBLE *PRECISION\\|DOUBLE *COMPLEX\\|EQUVALENCE\\|"
   "INTEGER\\|LOGICAL\\|PARAMETER\\|REAL\\|TYPE *([^()]*)\\)")
   "*Regular expression to match F77 declaration lines")
;;
(defun fortran-next-common (&optional name)
  "Search next common block with optional NAME.
If NAME is a string then only common that blocks are searched for.
Sets `fortran-common-start' and `fortran-common-end' to the start/end
position. Returns the name of the common block found (nil if none)."
  (setq fortran-common-start nil)
  (let ((found)(start))
       (if (and name (stringp name))
           (if (re-search-forward (format "^      +COMMON */ *%s */" name) nil t)
               (setq found name))
           (if (re-search-forward "^      +COMMON */ *\\([^ ]*\\) */" nil t)
               (setq found (buffer-substring (match-beginning 1) (match-end 1)))))
       (if found (save-excursion (beginning-of-line)
                 (setq fortran-common-start (point))
                 ;;
                 ;; End of common after all continuation lines
                 (forward-line) (while (looking-at "     [^ ]") (forward-line))
                 ;;
                 ;; Directly trailing `common-block-comment-lines'
                 ;; declarations are part of the definition (if any)
                 (while (and (or (looking-at "[Cc*!] *<%> *[^ \n]")
                                 (looking-at fortran-decline-regexp))
                             (setq start (point)) (not (eobp)))
                        (forward-line))
                 ;;
                 ;; Plus one trailing empty `common-block-comment-line'
                 (if (looking-at "[Cc*!] *<%> *$") (forward-line))
                 ;;
                 (setq fortran-common-end (point))
                 (goto-char fortran-common-start)
                 (setq start fortran-common-start) (forward-line -1)
                 ;;
                 ;; Directly leading declarations are part of the definition
                 (while (and (looking-at fortran-decline-regexp)
                             (setq start (point)) (not (bobp)))
                        (forward-line -1))
                 ;;
                 ;; Extend the start of common-block definition only if
                 ;; there is a leading 'common-block-comment-line'
                 (if (looking-at "[Cc*!] *<%>") (progn
                     ;;
                     ;; Take leading `common-block-comment-lines' (if any)
                     (while (and (looking-at "[Cc*!] *<%> *[ -]")
                                 (setq start (point)) (not (bobp)))
                            (forward-line -1))
                     ;;
                     ;; Plus one leading empty `common-block-comment-line'
                     (if (looking-at "[Cc*!] *<%> *$") (setq start (point)))
                     ;;
                     (setq fortran-common-start start)
                 ))
                 (eval found))
           (setq fortran-common-start nil) (setq fortran-common-end nil))
))
;;
(defvar fortran-common-master-file "~/files/Common.f"
  "*File name with master definitions of common blocks.")
;;
(defun fortran-common-text (name)
  "Get the master definition of common block NAME (or nil if none)."
  (let ((file fortran-common-master-file) (buffer "*Common-Blocks*")
        (fortran-common-start) (fortran-common-end))
       (save-window-excursion
         (if (get-buffer buffer) (set-buffer buffer)
             (unconditional-new-buffer buffer 'other)
             (if (file-readable-p file) (insert-file-contents file))
             (set-buffer-modified-p nil) (setq buffer-read-only t))
         (goto-char (point-min))
         (if (fortran-next-common name)
             (buffer-substring fortran-common-start fortran-common-end)
             nil)
)))
;;----------------------------------------------------------------------------
(defun fortran-tab-left-right () "tab-left-right for fortran-mode."
  (interactive)
  (if (fortran-comment-line-p) (tab-left-right)
      (if (eq last-command 'fortran-tab-left-right) (tab-left-right)
          (fortran-indent-line))))

;;;****************************************************************************
;;;***** Additional FORTRAN Abbreviations (with function definitions)     *****
;;;****************************************************************************
(define-abbrev fortran-mode-abbrev-table ";1"   "100 CONTINUE" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";2"   "200 CONTINUE" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";3"   "300 CONTINUE" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";4"   "400 CONTINUE" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";5"   "500 CONTINUE" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";6"   "600 CONTINUE" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";7"   "700 CONTINUE" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";8"   "800 CONTINUE" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";9"   "900 CONTINUE" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";0"   "999 CONTINUE" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";bc"  "" 'fortran-blank-common)
(define-abbrev fortran-mode-abbrev-table ";c"   "continue" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";ca"  "" 'insert-list-of-calls)
(define-abbrev fortran-mode-abbrev-table ";cb"  "" 'fortran-common-comment)
(define-abbrev fortran-mode-abbrev-table ";co"  "" 'fortran-blank-common)
(define-abbrev fortran-mode-abbrev-table ";cp"  "" 'fortran-copyright-line)
(define-abbrev fortran-mode-abbrev-table ";cx"  "character*(*)" nil)
;;???          fortran-mode-abbrev-table ";cx"  "complex" nil)
;;???          fortran-mode-abbrev-table ";c*"  "character*(*)" nil)
(define-abbrev fortran-mode-abbrev-table ";c0"  "character*" nil)
(define-abbrev fortran-mode-abbrev-table ";c1"  "character*1" nil)
(define-abbrev fortran-mode-abbrev-table ";c2"  "character*2" nil)
(define-abbrev fortran-mode-abbrev-table ";c3"  "character*3" nil)
(define-abbrev fortran-mode-abbrev-table ";c4"  "character*4" nil)
(define-abbrev fortran-mode-abbrev-table ";c5"  "character*5" nil)
(define-abbrev fortran-mode-abbrev-table ";c6"  "character*6" nil)
(define-abbrev fortran-mode-abbrev-table ";c7"  "character*7" nil)
(define-abbrev fortran-mode-abbrev-table ";c8"  "character*8" nil)
(define-abbrev fortran-mode-abbrev-table ";c9"  "character*9" nil)
(define-abbrev fortran-mode-abbrev-table ";c10" "character*10" nil)
(define-abbrev fortran-mode-abbrev-table ";c11" "character*11" nil)
(define-abbrev fortran-mode-abbrev-table ";c12" "character*12" nil)
(define-abbrev fortran-mode-abbrev-table ";c13" "character*13" nil)
(define-abbrev fortran-mode-abbrev-table ";c16" "character*16" nil)
(define-abbrev fortran-mode-abbrev-table ";c20" "character*20" nil)
(define-abbrev fortran-mode-abbrev-table ";c40" "character*40" nil)
(define-abbrev fortran-mode-abbrev-table ";c80" "character*80" nil)
(define-abbrev fortran-mode-abbrev-table ";da"  "data //" 'backward-3char-h)
(define-abbrev fortran-mode-abbrev-table ";e"   "else"    'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";ed"  "enddo"   'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";ei"  "else if" 'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";el"  "elseif"  'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";en"  "endif"   'fortran-indent-line)
(define-abbrev fortran-mode-abbrev-table ";in"  "implicit none" nil)
(define-abbrev fortran-mode-abbrev-table ";int" "intrinsic" nil)
(define-abbrev fortran-mode-abbrev-table ";iv"  "" 'fortran-input-comment)
(define-abbrev fortran-mode-abbrev-table ";lv"  "" 'fortran-local-comment)
(define-abbrev fortran-mode-abbrev-table ";ml"  "" 'fortran-modification-line)
(define-abbrev fortran-mode-abbrev-table ";ns"  "" 'fortran-namsub-parameter)
(define-abbrev fortran-mode-abbrev-table ";np"  "" 'fortran-namsub-parameter)
(define-abbrev fortran-mode-abbrev-table ";pn"  "" 'fortran-namsub-parameter)
(define-abbrev fortran-mode-abbrev-table ";sn"  "" 'fortran-namsub-parameter)
(define-abbrev fortran-mode-abbrev-table ";mn"  "" 'fortran-myname-parameter)
(define-abbrev fortran-mode-abbrev-table ";o"   "open" nil)
(define-abbrev fortran-mode-abbrev-table ";op"  "open" nil)
(define-abbrev fortran-mode-abbrev-table ";ov"  "" 'fortran-output-comment)
(define-abbrev fortran-mode-abbrev-table ";p"   "print" nil)
(define-abbrev fortran-mode-abbrev-table ";pa"  "parameter ()" 'backward-char-h)
(define-abbrev fortran-mode-abbrev-table ";px"  "print*" nil)
(define-abbrev fortran-mode-abbrev-table ";re"  "read" nil)
(define-abbrev fortran-mode-abbrev-table ";rx"  "read (*,*)" nil)
(define-abbrev fortran-mode-abbrev-table ";se"  "stop\n      end" nil)
(define-abbrev fortran-mode-abbrev-table ";tv"  "" 'fortran-inout-comment)
(define-abbrev fortran-mode-abbrev-table ";w"   "write(*,*)" nil)
(define-abbrev fortran-mode-abbrev-table ";wn"  "write(*,*) namsub" nil)
(define-abbrev fortran-mode-abbrev-table ";wnx" "write(*,*) namsub," 'fortran-write-args)
(define-abbrev fortran-mode-abbrev-table ";wx"  "write(*,*) " 'fortran-write-args)
;;
(defun def-fortran-abbrev (name expansion &optional hook)
      (define-abbrev fortran-mode-abbrev-table name expansion hook))
;;
(def-fortran-abbrev ";ci"   "character*(*), intent(in)   ::" nil)
(def-fortran-abbrev ";cio"  "character*(*), intent(inout)::" nil)
(def-fortran-abbrev ";cm"   "character*(*), intent(inout)::" nil)
(def-fortran-abbrev ";cxi"  "character*(*), intent(in)   ::" nil)
(def-fortran-abbrev ";cxio" "character*(*), intent(inout)::" nil)
(def-fortran-abbrev ";cxm"  "character*(*), intent(inout)::" nil)
(def-fortran-abbrev ";cxo"  "character*(*), intent(out)  ::" nil)
(def-fortran-abbrev ";cxp"  "character*?, parameter :: =" 'backward-2char-h)
(def-fortran-abbrev ";cxt"  "character*(*), intent(inout)::" nil)
;(def-fortran-abbrev ";c0p"  "character*?, parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c1p"  "character*1, parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c2p"  "character*2, parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c3p"  "character*3, parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c4p"  "character*4, parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c5p"  "character*5, parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c6p"  "character*6, parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c7p"  "character*7, parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c8p"  "character*8, parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c9p"  "character*9, parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c10p" "character*10,parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c11p" "character*11,parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c12p" "character*12,parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c13p" "character*13,parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c16p" "character*16,parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c20p" "character*20,parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c40p" "character*40,parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";c80p" "character*80,parameter :: =" 'backward-2char-h)
;(def-fortran-abbrev ";co"  "character(*), intent(out)  ::" nil)
(def-fortran-abbrev ";d"    "double precision" nil)
(def-fortran-abbrev ";di"   "double precision, intent(in)   ::" nil)
(def-fortran-abbrev ";dio"  "double precision, intent(inout)::" nil)
(def-fortran-abbrev ";dm"   "double precision, intent(inout)::" nil)
(def-fortran-abbrev ";do"   "double precision, intent(out)  ::" nil)
(def-fortran-abbrev ";dp"   "double precision, parameter :: =" 'backward-2char-h)
(def-fortran-abbrev ";dt"   "double precision, intent(inout)::" nil)
(def-fortran-abbrev ";fh"   "" 'fortran-function-template)
(def-fortran-abbrev ";i"    "integer" nil)
(def-fortran-abbrev ";if"   "" 'fortran-if-illegal-integer)
(def-fortran-abbrev ";ii"   "integer, intent(in)   ::" nil)
(def-fortran-abbrev ";iio"  "integer, intent(inout)::" nil)
(def-fortran-abbrev ";im"   "integer, intent(inout)::" nil)
(def-fortran-abbrev ";io"   "integer, intent(out)  ::" nil)
(def-fortran-abbrev ";ip"   "integer, parameter :: =" 'backward-2char-h)
(def-fortran-abbrev ";it"   "integer, intent(inout)::" nil)
(def-fortran-abbrev ";l"    "logical" nil)
(def-fortran-abbrev ";li"   "logical, intent(in)   ::" nil)
(def-fortran-abbrev ";lio"  "logical, intent(inout)::" nil)
(def-fortran-abbrev ";lm"   "logical, intent(inout)::" nil)
(def-fortran-abbrev ";lo"   "logical, intent(out)  ::" nil)
(def-fortran-abbrev ";lp"   "logical, parameter :: =" 'backward-2char-h)
(def-fortran-abbrev ";lt"   "logical, intent(inout)::" nil)
(def-fortran-abbrev ";mh"   "" 'fortran-module-template)
(def-fortran-abbrev ";pr"   "" 'fortran-pr-template)
(def-fortran-abbrev ";p3"   "" 'fortran-p3-template)
(def-fortran-abbrev ";p2"   "" 'fortran-p2-template)
(def-fortran-abbrev ";r"    "real" nil)
(def-fortran-abbrev ";ri"   "real, intent(in)   ::" nil)
(def-fortran-abbrev ";rio"  "real, intent(inout)::" nil)
(def-fortran-abbrev ";rm"   "real, intent(inout)::" nil)
(def-fortran-abbrev ";ro"   "real, intent(out)  ::" nil)
(def-fortran-abbrev ";rp"   "real, parameter    :: =" 'backward-2char-h)
(def-fortran-abbrev ";rt"   "real, intent(inout)::" nil)
(def-fortran-abbrev ";sh"   "" 'fortran-subroutine-template)
(def-fortran-abbrev ";t"    "type ()::" 'backward-2char-h)
(def-fortran-abbrev ";td"   "" 'fortran-typedef-template)
(def-fortran-abbrev ";ti"   "type (), intent(in)   ::" nil)
(def-fortran-abbrev ";tio"  "type (), intent(inout)::" nil)
(def-fortran-abbrev ";tm"   "type (), intent(inout)::" nil)
(def-fortran-abbrev ";to"   "type (), intent(out)  ::" nil)
(def-fortran-abbrev ";tt"   "type (), intent(inout)::" nil)
(def-fortran-abbrev ";u"    "use" nil)
(def-fortran-abbrev ";x"    "complex" nil)
(def-fortran-abbrev ";xi"   "complex, intent(in)   ::" nil)
(def-fortran-abbrev ";xio"  "complex, intent(inout)::" nil)
(def-fortran-abbrev ";xm"   "complex, intent(inout)::" nil)
(def-fortran-abbrev ";xo"   "complex, intent(out)  ::" nil)
(def-fortran-abbrev ";xt"   "complex, intent(inout)::" nil)
(def-fortran-abbrev ";xp"   "complex, parameter    :: =" 'backward-2char-h)
(def-fortran-abbrev ";z"    "double complex" nil)
(def-fortran-abbrev ";zi"   "double complex, intent(in)   ::" nil)
(def-fortran-abbrev ";zio"  "double complex, intent(inout)::" nil)
(def-fortran-abbrev ";zm"   "double complex, intent(inout)::" nil)
(def-fortran-abbrev ";zo"   "double complex, intent(out)  ::" nil)
(def-fortran-abbrev ";zp"   "double complex, parameter    :: =" 'backward-2char-h)
(def-fortran-abbrev ";zt"   "double complex, intent(inout)::" nil)
;;;
(defun backward-3char-h () "" (interactive) (backward-char 3))
(defun backward-2char-h () "" (interactive) (backward-char 2))
;;;
(defun fortran-blank-common () "Insert blank common."
  (interactive)
;  (save-excursion (beginning-of-line 0)
;       (if (looking-at "[Cc*!] <%>") nil (forward-line) (insert "C <%>\n")))
  (insert "INTEGER   LOT") (fortran-indent-line) (insert "\n      ")
  (insert "COMMON // LOT(1)") (fortran-indent-line))
;;;-----
(defun fortran-input-comment () "" (interactive)
      (insert comment-start) (insert " --> -") (backward-2char-h))
(defun fortran-output-comment () "" (interactive)
      (insert comment-start) (insert " <-- -") (backward-2char-h))
(defun fortran-inout-comment () "" (interactive)
      (insert comment-start) (insert " <-> -") (backward-2char-h))
(defun fortran-local-comment () "" (interactive)
      (insert comment-start) (insert " >-< -") (backward-2char-h))
(defun fortran-common-comment () "" (interactive)
      (insert comment-start) (insert " <%> -") (backward-2char-h))
;;;-----
(defun fortran-if-illegal-integer () "" (interactive)
   (let ((name (fortran-read-variable)) save)
        (insert (format "IF (%s /= " name)) (setq save (point))
        (insert (format ")
     &   CALL ERRCC1 (1,010 5003,NAMSUB,'%s',%s)" name name))
        (update-permas-format) (goto-char save)))
;;
(defun fortran-read-variable () ""
   (upcase (read-no-blanks-with-default "List of variables" "IATOM")))
;;
(defun fortran-read-arg () ""
   (upcase (read-no-blanks-input "List of variables: ")))
;;
(defun fortran-write-args () "" (interactive)
   (let ((name (fortran-read-arg)))
        (and name (string< " " name)
            (insert (format "' %s=',%s" name name)))))
;;
(defun fortran-subroutine-template () "Insert a SUBROUTINE template (header)."
  (interactive)
  (let ((name (upcase (read-no-blanks-input "Name of SUBROUTINE: " ""))))
        (fortran-sub-fun-template "SUBROUTINE" name)))
;;
(defun fortran-function-template () "Insert a FUNCITON tmplate (header)."
  (interactive)
  (let ((name (upcase (read-no-blanks-input "Name of FUNCTION: " ""))))
        (fortran-sub-fun-template "FUNCTION" name)))
;;
(defun fortran-module-template () "Insert a MODULE template (header)."
  (interactive)
  (let ((name (upcase (read-no-blanks-input "Name of MODULE: " ""))))
        (fortran-sub-fun-template "MODULE" name)))
;;
(defun fortran-pr-template () "Insert a PRocessor template (header)."
  (interactive)
  (let ((name (upcase (read-no-blanks-input "Name of PRocessor: " ""))))
        (fortran-sub-fun-template "PR" name)))
;;
(defun fortran-p3-template () "Insert a P3 template (header)."
  (interactive)
  (let ((name (upcase (read-no-blanks-input "Name of P3-Routine: " ""))))
        (fortran-sub-fun-template "P3" name)))
;;
(defun fortran-p2-template () "Insert a P2 template (header)."
  (interactive)
  (let ((name (upcase (read-no-blanks-input "Name of P2-Routine: " ""))))
        (fortran-sub-fun-template "P2" name)))
;;
(defvar fortran-header-string nil
 "*Text to be inserted as standard document string in sub-fun-template.")
;;
(defvar f7x-name-brackets t
 "*Non-nil supports END SUBROUTINE XYZ naming.")
;;
(defun fortran-sub-fun-template (type name) ""
  (fortran-sub-fun-header type name)
  (let ((pos (point)))
       (insert "\n") (fortran-copyright-line)
       (cond ((equal type "MODULE") (insert "! <%>\n"))
             ((equal type "PR") (fortran-pr-body type name))
             ((equal type "P3") (fortran-pr-body type name))
             ((equal type "P2") (fortran-pr-body type name))
             (     t    (insert (fortran-sub-fun-body name))))
       (if (and (equal major-mode 'f7x-mode) f7x-name-brackets)
           (let ((ftype "SUBROUTINE"))
                (if (or (equal type "FUNCTION")
                        (equal type "MODULE")) (setq ftype type))
                (insert (format "      END %s %s" ftype name)))
           (insert "      END"))
       (goto-char pos))
  (if (stringp fortran-header-string)
      (if (string< " " fortran-header-string) (insert fortran-header-string))
      (insert (concat "#" (upcase
              (file-name-sans-extension (buffer-name))) "#")))
  (message "Starting a short description..."))
;;
(defun fortran-sub-fun-header (type name) ""
  (let ((ftype "SUBROUTINE") (xtype "I") (cstart "C"))
       (if (equal type "FUNCTION") (setq ftype type))
       (if (equal type "MODULE") (setq ftype type))
       (if (equal type "PR") (setq xtype "X"))
       (if (equal major-mode 'f7x-mode) (setq cstart "!"))
       (beginning-of-line)
       (insert (fortran-sub-fun-cx-line name))
       (insert (format "      %s %s" ftype name))
       (or (equal type "MODULE")
           (if (equal type "PR") (insert " (NAMTBL,NAMBUK)") (insert " ()")))
       (if (equal type "PR") (insert "\n      USE BUKDEF"))
       (insert "\n      IMPLICIT NONE")
       (if (equal type "PR") (insert "
      CHARACTER*(*), INTENT(IN) :: NAMTBL,NAMBUK"))
       (insert (format "
%s==%s====================================================================
%s " cstart xtype cstart)))  ;;\nC\nC --> XYZARG - \nC\n")
)
;;;
(defun fortran-namsub-parameter ()
   (beginning-of-line)
   (insert (format "      CHARACTER*(*), PARAMETER :: NAMSUB='%s'"
                   (fortran-sub-fun-name)))
   (delete-horizontal-space)) ; remove trailing blanks
;;;
(defun fortran-myname-parameter ()
   (beginning-of-line)
   (insert (format "      CHARACTER*(*), PARAMETER :: MYNAME='%s'"
                   (fortran-sub-fun-name)))
   (delete-horizontal-space)) ; remove trailing blanks
;;;
(defun fortran-copyright-line () ""
  (interactive) (beginning-of-line) (insert (format
(if (equal major-mode 'f7x-mode)
"!-----------------------------------------------------------------------
! (C) Copyright %s, %s, %s   %s    %s        1\n"
"C-----------------------------------------------------------------------
C (C) Copyright %s, %s, %s   %s    %s        1\n")
      company-address-name company-town company-state
      user-two-letter-name (mon-yyyy)))
  (fortran-modification-line))
;;;---
(defun fortran-modification-line () ""
  (interactive) (beginning-of-line) (insert (format
(if (equal major-mode 'f7x-mode)
"!!mod--------------------------------------------%s-%s-%s\n"
"C!mod--------------------------------------------%s-%s-%s\n")
      user-two-letter-name (dd-mon-yyyy) (hh-mm-ss))))
;;;---
(defun fortran-sub-fun-cx-line (name) ""
(let ((namex (substring (concat name " ...............") 0 16)))
     (concat comment-start
            (format "*($) %s%s%s%s\n" namex namex namex namex))))
;;
(defun fortran-sub-fun-body (name) "" (format
"      CHARACTER*(*),PARAMETER :: NAMSUB='%s'\n! <%%>
      CALL TRACIN (NAMSUB)\n!----
      CALL ERRC (1,010 5000,NAMSUB)
!*10  (%%s): Routine not (fully) implemented
!----
  999 CALL TRACEX (NAMSUB)\n" name))

;;
(defun fortran-pr-body (type name) ""
  (insert (format "      CHARACTER*(*),PARAMETER :: NAMSUB='%s'\n" name))
  (if (equal type "PR") (insert "      TYPE (BUKDES) :: IGDES\n"))
  (if (equal type "PR") (insert "      INTEGER  IDBOOK,IDTBL,NOFTUP\n"))
  (if (equal type "P3") (insert "      INTEGER  I3REG,IFRMAT,IWIN(4),NOFROW,NOFCOL\n"))
  (if (equal type "P2") (insert "      INTEGER  I2REG,IFRMAT,IWIN(4),NOFROW,NOFCOL\n"))
  (insert (format "      LOGICAL  OK\n! <%%>
      CALL %sINIT (NAMSUB)\n!----\n!\n" type))
  (insert "!     > Access and test data objects\n")
  (insert "!      ------------------------------\n")
  (if (equal type "PR") (progn
      (insert "      CALL PRITBL (NAMTBL,1, I D T B L , N O F T U P )\n")
      (insert "      CALL PRMBUK (NAMBUK,1, I D B O O K , I G D E S )\n")
      (insert "!\n!     > Additional tests\n")
      (insert "      CALL PRCTBL (IDTBL,3,NOFCOL,-1)\n")
      (insert "      CALL PRCBUK (IDBOOK,NROWP,NCOLP,NGROWS,NGCOLS,IATOM,MFORM)\n")))
  (if (equal type "P3") (progn
      (insert "      CALL P3IMAT (IDBOOK, I 3 R E G , I F R M A T , N O F R O W ,\n")
      (insert "     &                                 N O F C O L , I W I N )\n")))
  (if (equal type "P2") (progn
      (insert "      CALL P2IMAT (I3REG, I 2 R E G , I F R M A T , N O F R O W ,\n")
      (insert "     &                                N O F C O L , I W I N )\n")))
  (insert (format "      CALL %sSTART ( O K )
      IF (.NOT.OK) GOTO 999
!
!
!     > Operation branch
!      ------------------
      CALL ERRCC (1,010 5001,NAMSUB,'Processor')
!*10  (%%s): %%s not (fully) implemented
      GOTO 999
!
!
!     > Errors
!      --------
  901 CALL %sERRx (... extra error messages \n!\n" type type type type))
       (insert (format "!----\n  999 CALL %sEXIT (NAMSUB)\n" type))
)

;;
(defun fortran-typedef-template () "Insert a type definition template."
  (interactive)
  (let*((name (upcase (read-no-blanks-input "TYPE name of TYPE_DEF: ")))
        (ln (length name)))
       (beginning-of-line)
       (insert         "! <%>\n")
       (insert (format "      CHARACTER*(*), PARAMETER :: %s_TIT = '%s Definition'\n" name name))
       (insert (format "      TYPE :: %-12s!       %s       %s\n" (concat name "_DEF")
                                              (make-string ln ?=) (make-string (+ ln 13) ?=)))
       (insert         "         SEQUENCE         !\n")
       (insert         "         INTEGER FIELD1   !") (setq pos (point))
                                                      (insert "Field Description\n")
       (insert         "         INTEGER FIELD2   !\n")
       (insert         "         INTEGER FIELD3   !\n")
       (insert (format "      END TYPE %s_DEF\n" name))
;?     (insert (format "      INTEGER      , PARAMETER :: %s_NOF = 3 ! Number of fields\n" name))
       (insert (format "      INTEGER      , PARAMETER :: %s_LEN = 3 ! Lenx for QMOVE etc.\n" name))
       (insert (format "      CHARACTER*(*), PARAMETER :: %s_NAM =   ! List of field titles\n" name))
       (insert (format "     &'|Field1|Field2|Field3|'\n"))
       (insert         "! <%>\n")
       (goto-char pos)))

;;;****************************************************************************
;;;***** FORTRAN Auto-Upcase / Context-Sensitive Caps-Lock                *****
;;;****************************************************************************
(defvar fortran-auto-upcase t
"*Setting this variable to non-nil will automatically upcase any character
typed within a Fortran buffer. Only characters on comment lines [cC!*] and
characters within quoted text areas will be inserted with unchanged case.
Resetting to nil suppresses upcasing (just like normal text-mode)")
;;;
(defun fortran-self-insert-command ()
"The self-inserted character will be upcased automatically, if point is
in a Fortran command line and the variable fortran-auto-upcase is non-nil !"
      (interactive)
      (call-interactively 'self-insert-command)
      (if fortran-auto-upcase
         (if (>= (preceding-char) ?a)
             (if (fortran-comment-or-text)
                 (fortran-comment-update)
                 (upcase-region (- (point) 1) (point))))))
;;;
(defun fortran-comment-update () "Update comment according to major-mode."
  (if (equal major-mode 'f7x-mode)
      (let ((original (point)))
           (backward-char)
           (if (looking-at "^c") (progn (delete-current-char 1) (insert-char ?! 1)))
           (goto-char original))
))
;;;
(defun fortran-comment-or-text ()
  "Test actual point to be within Fortran comment line (starting with [cC*!])
or within a quoted Fortran text area, returns t if so - otherwise nil !
Also special comment-lines, starting with \"C..-..\" (. for any character)
classify the next 8 character position as upcase name."
  (let ((original (point)) (ret nil) (first-char))
       (beginning-of-line)
       (setq first-char (following-char))
       (cond ((or (char-equal first-char ?c)
                  (char-equal first-char ?C)
                  (char-equal first-char ?!))
              (upcase-region (point) (+ (point) 1))
              (if (looking-at ".fmt")
                  (upcase-region (point) (+ (point) 4)))
              (if (>= (+ (point) 6) original) (setq ret t)
                  (if (< (+ (point) 14) original) (setq ret t);16??
                      (goto-char (+ (point) 3))
                      (if (char-equal (following-char) ?-)
                          (setq ret nil) (setq ret t)))))
             ((char-equal first-char ?*) (setq ret t))
             (t (setq pos1 (+ (point) 6)) (setq pos2 (- original 1))
                (while (< pos1 pos2)
                       (if (or (char-equal (char-after pos1) ?')
                               (char-equal (char-after pos1) ?\")
                               (char-equal (char-after pos1) ?!))
                           (setq ret (not ret)))
                       (setq pos1 (+ pos1 1)))))
        (goto-char original)
        (equal ret t)))
;;
(defun fortran-comment-line-p ()
  "Test actual point to be within Fortran comment line."
  (save-excursion (beginning-of-line) (looking-at comment-start-skip)))
;;
(defun fortran-newline-indent ()
  "Creates a new, automatically indented line (newline&fortran-indent-line)."
  (interactive) (newline) (fortran-indent-line))
;;;
;;
;; Extracting infos:
;;--------------------
(defun fortran-sub-fun-name ()
  "Get name of fortran subroutine/function around point."
  (save-excursion (beginning-of-line 1)
  (if (looking-at "[!C][*](") (forward-line 2) (forward-line 1))
  (if (re-search-backward
  "^      +\\(subroutine\\|.*function\\|module\\|program\\) +\\([a-z<_][][a-z0-9_<>$]*\\)" nil t)
; "^      +\\(subroutine\\|.*function\\|module\\|program\\) +\\([a-z<][^ ()\n]*\\)\\b" nil t)
      (match-string 2) (eval "???"))))
;;
(defun fortran-sub-fun-type ()
  "Get type of fortran subroutine/function around point."
  (save-excursion (beginning-of-line 1)
  (if (looking-at "[!C][*](") (forward-line 2) (forward-line 1))
  (if (re-search-backward
  "^      +.*\\(subroutine\\|function\\|module\\|program\\) " nil t)
      (match-string 1) (eval "???"))))
;;
(defun fortran-sub-fun-title ()
  "Get title line of fortran subroutine/function around point."
  (save-excursion (beginning-of-line 1) (if (looking-at "[!C][*](") (forward-line 1))
  (let ((end (end-of-fortran-header))) (beginning-of-fortran-subprogram)
       (if (re-search-forward "^[^ ] +\\(#[^#]+# *\\)?\\(.*\\)$" end t)
           (match-string 2) (eval "NoTitle")))))
;;
(defun fortran-sub-fun-synopsis ()
  "Get synopsis (extended title) of fortran subroutine/function around point."
  (save-excursion (beginning-of-line 1) (if (looking-at "[!C][*](") (forward-line 1))
  (let ((end (end-of-fortran-header))) (beginning-of-fortran-subprogram)
       (if (re-search-forward "^[^ ] +\\(#[^#]+# *\\)?" end t)
           (let ((start (point)))
                (re-search-forward "^ \\|^. [<>=-]\\|---------------------" end t)
                (beginning-of-line)
                (replace-regexp-in-string "\n. *\n" "\n"
                        (buffer-substring start (point))))
           (eval "???\n")))))
;;
(defun fortran-sub-fun-argstring ()
  "Get argument string of fortran subroutine/function around point."
  (save-excursion (beginning-of-line 1) (if (looking-at "[!C][*](") (forward-line 1))
  (let ((end (end-of-fortran-header))) (beginning-of-fortran-subprogram)
  (if (re-search-forward
  "^      +\\(subroutine\\|.*function\\|module\\|program\\) [^\n]*(\\([^()]*\\))" end t)
      (replace-regexp-in-string "[ &+*\n]" "" (match-string 2)) (eval "")))))
;;
(defun fortran-sub-fun-arglist ()
  "Get list of arguments for fortran subroutine/function around point.
Each element is a list in itself with (name iotype datatype text)."
  (save-excursion (beginning-of-line 1) (if (looking-at "[!C][*](") (forward-line 1))
  (let ((arglist) (end (end-of-fortran-header))) (beginning-of-fortran-subprogram)
  (if (re-search-forward "^[cC*!] [<%>-]+ " end t)
      (progn   (beginning-of-line)
               (while (looking-at "^[cC*!] [<%>-]+ \\([^ :-]+\\)[ :-]*")
                      (setq arglist (cons (fortran-arglist-element) arglist)))))
  (if arglist (reverse arglist)))))
;;
(defun fortran-arglist-element ()
   (if (looking-at "^[cC*!] \\([<%>-]+\\) \\([^ :-]+\\)[ :-]*\\(.*\\)$")
       (let ((iotype   (match-string 1))
             (name     (match-string 2))
             (datatype "Unknown") (end)
             (text     (match-string 3)))
            (if (string-match "[^ ;,]+[;,] *" text)
                (progn (setq end (match-end 0))
                  (setq datatype (substring text 0 (string-match "[;,]" text)))
                  (setq text (substring text end)))
                (setq datatype (capitalize (fortran-arglist-datatype name)))
                (setq datatype (replace-regexp-in-string " +" "" datatype))
                (if (string-match "type(" datatype) (progn
                    (setq datatype (upcase datatype))
                    (setq datatype (replace-regexp-in-string "TYPE" "@@@@" datatype))
                    (setq datatype (replace-regexp-in-string "@@@@" "Type" datatype))
            )))
            (beginning-of-line 2)
            (while (looking-at "^[cC*!] [<%>-]*   +\\([^ ].*\\)$")
                   (setq text (concat text "\n" (match-string 1)))
                   (beginning-of-line 2))
       (list name iotype datatype text))))
;;
(defun fortran-arglist-datatype (name) "Get data type of NAME."
  (save-excursion
  (let ((end (end-of-fortran-subprogram))) (beginning-of-fortran-subprogram)
       (if (re-search-forward (concat fortran-decline-regexp
                              ".*\\b" name "\\b *\\(\\(([0-9,*]+)\\)?\\)") end t)
           (concat (match-string 1) (match-string 2))
           (let ((char (string-to-char name)))
                (cond ((< char ?I) (eval "Real"))
                      ((< char ?O) (eval "Integer"))
                      ( t          (eval "Real"))))))))
;;
;;----------------------------------------------------------------------------
(defun fortran-to-tex ()
  "Convert the header of the current subroutine/function into a rudimentary
TeX-Description for the \\begin{call}{title}{name}{args} environment."
  (interactive)
  (save-window-excursion
  (let* ((name (fortran-sub-fun-name))
         (title (fortran-sub-fun-title))
         (synopsis (fortran-sub-fun-synopsis))
         (type (fortran-sub-fun-type))
         (argstring (fortran-sub-fun-argstring))
         (arglist (fortran-sub-fun-arglist))
         (maxlen (maxlength-tex-call-arg arglist)))
  (unconditional-new-buffer "*scratch*" t)
  (if (equal type "FUNCTION") (setq name (concat "\\U{" name "}")))
;;
  (insert (format "\\B{call}{%s}\n        {%s}" title name))
  (let ((start (point)) TeXstring)
       (insert-tex-call-argstring arglist)
       (setq TeXstring (buffer-substring start (point)))
       (setq TeXstring (replace-regexp-in-string "\\\\[UI]" "" TeXstring))
       (setq TeXstring (replace-regexp-in-string "[ {}&+*\n]" "" TeXstring))
       (if (equal TeXstring argstring) nil
           (message TeXstring) (message argstring)
           (beep) (message "FORTRAN- and TeX-Argumenstring do not match, please check !")
           (sleep-for 3) (beep))
  )
  (insert "\\B{argl}%")
  (insert (format "%s%s-%s\n  \\item[Synopsis] "
             (make-string (length name) ?-) user-two-letter-name (dd-mon-yyyy)))
  (insert (replace-regexp-in-string "\n[^ ] +" "\n                  " synopsis))
  (insert-tex-call-arg "-->" "Input"     arglist maxlen)
  (insert-tex-call-arg "<->" "Transient" arglist maxlen)
  (insert-tex-call-arg "<--" "Output"    arglist maxlen)
  (insert "%  \\item[Remark] Additional comments ...\n")
;;(insert (format "%s is a PTM processor (is parallelized automatically).\n" name))
  (insert "%  \\item[Errors]\n")
  (insert "%     \\B{errlist}\n")
  (insert "%       \\item[F] Error condition ...\n")
  (insert "%     \\E{errlist}\n")
  (insert "\\E{argl}\n\\E{call}\n\\bigskip\n")
  (goto-char (point-min))
  (loop-for-numeric-arglist (point-max))
  (goto-char (point-min))
;; Finally grab new TeX-Description into paste buffer
  (copy-region-as-kill (point-min) (point-max))
  (message (format "TeX-Description of %s copied to *scratch* and *paste*" name))
)))
;;
(defun insert-tex-call-arg (iotype title arglist maxlen)
  "Create an {argd}-Tex-list for all IOTYPE arguments in ARGLIST (if any)."
  (let (open arg (args arglist) (indent (make-string maxlen ? )))
       (while args (setq arg (car args)) (setq args (cdr args))
         (if (equal (nth 1 arg) iotype)
             (let ((name (car arg))
                   (text (concat (nth 2 arg) "; " (nth 3 arg))))
                (setq text (replace-regexp-in-string "\n" "@@           " text))
                (setq text (replace-regexp-in-string "@@" (concat "\n    " indent) text))
                (setq text (replace-regexp-in-string "r[*]([*])" "r$*(*)$" text))
                (setq text (replace-regexp-in-string "r[*]" "r$*$" text))
                (setq text (replace-regexp-in-string "[*]);" "$*$);" text))
                (setq text (replace-regexp-in-string "[*](;" "$*$(;" text))
                (if open nil (insert (format "  \\item[%s]\n" title))
                             (insert "     \\B{argd}\n") (setq open t))
                (insert (format
                (concat "       \\item[%-" (number-to-string (+ 1 maxlen)) "s %s\n")
                             (concat name "]") text)))))
       (if open (insert "     \\E{argd}\n"))))
;;
(defun insert-tex-call-argstring (arglist) "test"
  (insert "{")
  (let (comma arg (args arglist))
       (while args (setq arg (car args)) (setq args (cdr args))
         (let ((name (car arg)) (iotype (nth 1 arg)))
              (if comma (insert ", "))
              (cond ((equal iotype "<--") (insert (concat "\\U{" name "}")))
                    ((equal iotype "<->") (insert (concat "\\I{" name "}")))
                    (t                    (insert name))))
       (setq comma t)))
  (insert "}\n"))
;;
(defun maxlength-tex-call-arg (arglist)
  "Get length of longest argument name in ARGLIST."
  (let ((maxlen 3) (args arglist))
       (while args (setq arg (car args)) (setq args (cdr args))
                   (setq maxlen (max maxlen (length (car arg)))))
       (eval maxlen)))
;;
(defun loop-for-numeric-arglist (end)
  (while (re-search-forward "^           *\\(- +\\)?\\([<=>]* *[0-9.+-]+\\)" end t)
         (beginning-of-line) (insert "          \\B{arg$}\n")
         (beginning-of-line 1)
         (while (looking-at "              ")
                (if (looking-at "         *\\(- +\\)?\\([<=>]* *[0-9.+-]+\\)[ :-]*")
                    (replace-match "            \\\\item[\\2] ")
                    (insert "  "))
                (beginning-of-line 2))
         (insert "          \\E{arg$}\n")))
;;
;;----------------------------------------------------------------------------
(provide 'gold_f7x)
