;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Additional/modified functions for GNATS Problem-Reports !           ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_gnats")
(require 'gnats)
;;;!!!???(require 'gold_sendpr)
;;--------------
(defun pr-view-mode () "Interactive gnats-view-mode"
   (interactive) (gnats-view-mode))
;;
(setq gnats-edit-font-lock-keywords (list
  '("^>\\S-+:"                        . 'font-lock-keyword-face)
  '("^>Synopsis:\\s-*\\(.*\\)$"       1 'font-lock-variable-name-face)
  '("^>Number:\\s-*\\(.*\\)$"         1 'font-lock-reference-face)
  '("Fixed in Version *\\([0-9.]+\\)" 1 'font-lock-match-face)
  '("^>State: *\\(closed\\)"          1 'font-lock-match-face)
  '("^>State: *\\(open\\)"            1 'font-lock-mismatch-face)
))
;;--------------
;; Ensure that mapc is present (Emacs-20 from cl-extra, Built-in-Emacs-21)
(or (functionp 'mapc)
(defun mapc (cl-func cl-seq &rest cl-rest)
  "Like `mapcar', but does not accumulate values returned by the function."
  (if cl-rest
      (apply 'map nil cl-func cl-seq cl-rest)
    (mapcar cl-func cl-seq))
  cl-seq))
;;-----------------------------------------------------
;; setenv GNATSDB "gnats.local:1529:default:ast:"
(setq gnats-server "gnats.local")
(setq gnats-user (user-real-login-name))
(setq gnats-password ":")
;;
;; Take defaults from company.el (if not pre-defined)
(if (string-equal "" gnats-default-submitter)
    (setq gnats-default-submitter company-name))
(if (string-equal "" gnats-default-organization)
    (setq gnats-default-organization company-address-name))
;;
;; Allow Emacs-kill to silently kill a gnats-server process
(add-hook 'gnats-edit-mode-hook  'gnats-silent-kill)
(add-hook 'gnats-view-mode-hook  'gnats-silent-kill)
(add-hook 'gnats-query-mode-hook 'gnats-silent-kill)
(defun gnats-silent-kill ()
 (let ((process (get gnats-server-conn 'process)))
      (if (processp process) (process-kill-without-query process))
))
;;;;-------------
(defun my-gnats () "Test function" (interactive)
  (goto-char (point-min))
  (re-search-forward "Synopsis:[ \t]*" nil t)
)
;;----------------------------------------------------------------------------
;;
;; Extend standard query-pr with a usefull default string
;;--------------------------------------------------------
(fset 'gold-orig-query-pr (symbol-function 'query-pr))
(defun query-pr (query)
  "Create query buffer resulting from QUERY.
QUERY is a string representing a query in the gnatsd format.
This is just `gold-orig-pr-query' with a usefull default string."
  (interactive (list (read-string "Query: " (or query-pr-default
  (concat "Responsible=\"" (user-real-login-name)
                                 "\" & State!=\"closed\"")))))
  (gold-orig-query-pr query))
;;
(defvar query-pr-default nil
   "*Default string for calling query-pr.
If nil query-pr will build a default string based on the user's login name.")
;;
;;----------------------------------------------------------------------------
(defvar send-pr-template-directory "/ds-1/cusdata/templates/"
   "*Directory where `send-pr-template' looks for customer template-file.")
;;
(defun send-pr-template ()
  "Create and send a Problem Report for GNATS with an optional template file.
Prompts for a template-file from `send-pr-template-directory' (if readable).
If no template-file is given the standard `send-pr' is invoked."
  (interactive)
  (let ((tmp-dir send-pr-template-directory))
       (if (file-readable-p (directory-file-name tmp-dir))
          (let ((tmp-file (read-file-name
                           "PR template (default is standard `send-pr'): "
                           send-pr-template-directory " ")))
               (if (and (string-lessp " " tmp-file)
                        (file-readable-p tmp-file))
                   (send-pr-template-edit tmp-file) (send-pr)))
          (send-pr))
))
;;
(defun send-pr-template-edit (template-file)
  "Create a gnats Problem Report with a TEMPLATE-FILE and invoke send-pr."
  (let*((submitter (upcase (file-name-nondirectory template-file)))
        (buffer-name (concat "*" submitter "-pr*")) (case-fold-search t))
       (if (get-buffer buffer-name)
           (progn (switch-to-buffer buffer-name)
                  (if (or (not (buffer-modified-p))
                          (y-or-n-p "Erase previous problem report? "))
                      (progn (erase-buffer)
                             (insert-file-contents template-file))))
           (switch-to-buffer buffer-name)
           (insert-file-contents template-file))
        (setq subject (read-string ">Synopsis: "))
        (goto-char (point-min))
        (if (re-search-forward "^Subject:.*$" nil t)
            (replace-match (concat "Subject: " subject)))
        (if (re-search-forward "^>Synopsis:.*$" nil t)
            (replace-match (concat ">Synopsis:      " subject)))
        (if (re-search-forward "^>Description:[ \t]*$" nil t)
            (replace-match ">Description:   "))
        (send-pr-mode)
        (message (substitute-command-keys
                 "To send the problem report use: \\[mail-send]"))
))

;;gnats-3vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
;;gnats-3vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
;;gnats-3vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
;;=============================================================================
;; Changes to gnats.el:
;;=============================================================================
;(add-hook 'gnats-mode-hook 'gold-gnats-mode t)
;(defun gold-gnats-mode () "GOLD Additions/Modifications to send-pr-mode."
;  (setq context-update-function 'gnats-context-update)
;  (setq compile-buffer-function 'gnats:submit-pr-and-exit-recursion)
;  (local-set-key "\C-c\C-c"     'gnats:submit-pr-and-exit-recursion)
;  (local-set-key "\C-c\C-g"     'gnats:unlock-buffer)
;  (local-set-key "\M-\C-a"      'gnats:previous-field)
;  (local-set-key "\M-\C-e"      'gnats:next-field)
;  (message (concat "To re-submit the problem report use: "
;           (substitute-command-keys "\\[gnats:submit-pr-and-exit-recursion] ")
;           (substitute-command-keys " (\\[gnats:unlock-buffer] to unlock)"))))
;;;
;(defvar gnats-mail-signature-file "~/.signature"
;  "*Name of a mail signature file for gnats.")
;;;
;(defun gnats-context-update () "Context sensitive update of data around point"
;  (interactive)
;  (if (not (save-excursion (beginning-of-line 1) (looking-at ">[A-Z]")))
;      ;; Update day-month-year string in line (if an old date string is found)
;      (update-day-month-year)
;      ;;
;      (cond
;         ((save-excursion (beginning-of-line 1) (looking-at ">Category:"))
;          (gnats:category-change-from-to))
;         ;;
;         ((save-excursion (beginning-of-line 1) (looking-at ">Responsible:"))
;          (gnats:responsible-change-from-to) (end-of-line 0) (insert " ")
;          (message "Insert reason for changing >Responsible: field !"))
;         ;;
;         ((save-excursion (beginning-of-line 1) (looking-at ">State:"))
;          (gnats:state-change-from-to) (end-of-line 0) (insert " ")
;          (message "Insert reason for changing >State: field !"))
;         ;;
;         (t (gnats:change-field))
;)))
;;;
;(setq gnats:userid (user-login-name))
;(defun gnats::isme (addr)
;  (setq addr (mail-strip-quoted-names addr))
;  (or (string= addr (user-login-name))
;      (string= addr (concat (user-login-name) "@intes.de"))
;      (string= addr (concat (user-login-name) "@" (system-name)))))
;;;
;(defun gnats::mail-PR-changed-mail-setup (to subject cc buffer)
;  (let (mail-signature)
;  (or (gnats::vmish-mail
;       (format "notification of changes to PR %s" gnats:::buffer-pr)
;       nil to subject nil cc buffer)
;      (error "Submit aborted; PR is still locked."))))
;;;
;;; query-pr:
;;;-----------
;;; Default options to pass to query-pr.
;;; (setq gnats:query-pr-default-options (format "-r %s -P" (user-login-name)))
;(setq gnats:query-pr-default-options
;     (format "-r %s -q --class=sw-bug --state=\"open|feedback|analyzed\"" (user-login-name)))
;;; <e> - Invoke edit-pr for current PR (PR of current line)
;;; <v> - Invoke view-pr for current PR (PR of current line)
;;; <s> - Sort for PR-IDs/Categories
;;; <u> - Unlock current PR (PR of current line)
;;; <Ret> - Wie <v>
;;; <Context-Update> - Wie <e>
;;
;;;----------------------------------------------------------------------------
;(add-hook 'send-pr-mode-hook 'gold-send-pr-mode t)
;(defun gold-send-pr-mode () "GOLD Additions/Modifications to send-pr-mode."
;  (setq context-update-function 'gnats:change-field)
;  (setq compile-buffer-function 'send-pr:submit-pr);-with-mail-aliases)!!!!????
;  (local-set-key "\C-c\C-c"     'send-pr:submit-pr);-with-mail-aliases)!!!!????
;  (local-set-key "\M-\C-a"      'gnats:previous-field)
;  (local-set-key "\M-\C-e"      'gnats:next-field)
;  (run-hooks 'gold-send-pr-hook)
;)
;;
;(defvar send-pr-force-bcc   nil
;"*Non-nil forces `send-pr' to insert (user-login-name) to BCC: field.
;If `send-pr-force-bcc' is a string that string will be inserted instead.")
;;;
;(defvar send-pr-force-reply nil
;"*Non-nil forces `send-pr' to inserted (user-login-name) to Reply-To: field.
;If `send-pr-force-reply' is a string that string will be inserted instead.")
;;
;(defvar send-pr-default-category     nil "*Default for send-pr field `Category'")
;(defvar send-pr-default-class        nil "*Default for send-pr field `Class'")
;(defvar send-pr-default-confidential nil "*Default for send-pr field `Confidential'")
;(defvar send-pr-default-severity     nil "*Default for send-pr field `Severity'")
;(defvar send-pr-default-priority     nil "*Default for send-pr field `Prtiority'")
;(defvar send-pr-default-release      nil "*Default for send-pr field `Release'")
;(defvar send-pr-default-submitter    nil "*Default for send-pr field `Submitter-ID'")
;;;;
;(setq send-pr::fields (`(
;  ("Category" (("perm4") ("permas") ("permit") ("pre-post") ("felix")
;               ("emacs") ("gnats") ("parallel") ("pending"))
;       (, (or send-pr-default-category
;              (gnats::get-config "DEFAULT_CATEGORY") 1)) enum)
;  ("Class"    (("sw-bug") ("doc-bug") ("change-request") ("support"))
;       (, (or send-pr-default-class
;              (gnats::get-config "DEFAULT_CLASS") 3)) enum)
;  ("Confidential" (("yes") ("no"))
;       (, (or send-pr-default-confidential
;              (gnats::get-config "DEFAULT_CONFIDENTIAL") 1)) enum)
;  ("Severity" (("non-critical") ("serious") ("critical"))
;       (, (or send-pr-default-severity
;              (gnats::get-config "DEFAULT_SEVERITY") 1)) enum)
;  ("Priority" (("low") ("medium") ("high"))
;       (, (or send-pr-default-priority
;              (gnats::get-config "DEFAULT_PRIORITY") 1)) enum)
;  ("Release"  nil
;       (, (or send-pr-default-release
;              (gnats::get-config "DEFAULT_RELEASE") "PERMAS 7.00.000")) text)
;  ("Submitter-Id" nil
;       (, (or send-pr-default-submitter
;              (gnats::get-config "DEFAULT_SUBMITTER") "INTES")) text)
;  ("Synopsis" nil nil text
;       (lambda (a b c) (gnats::set-mail-field "Subject" c)))
;)))
;;----------------------------------------------------------------------------
(provide 'gold_gnats)
;;----------------------------------------------------------------------------
