;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Additional/modified functions and customizations for MAIL !         ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_mail")
;;----------------------------------------------------------------------------
(setq sendmail-program "/usr/lib/sendmail") ; Do not use stupid fakemail
(setq mail-self-blind t); Send myself a blind carbon copy of each outgoing mail
(setq mail-archive-file-name nil); but do not send it to a plain mail-archive !
(if mail-default-reply-to  ; Ignore t default setting (because of gnats).
    (or (stringp mail-default-reply-to) (setq mail-default-reply-to nil)))
;;----------------------------------------------------------------------------
(defun mime-install () "Explicitely install mime-features (may not be present)"
   (interactive)
   (setq mime/tmp-dir rmail-default-directory)
   (require 'mailcrypt)
   (require 'mime-setup)
   (setq mime-editor/split-message nil)
   (setq mime-editor/transfer-level 8)
   (setq signature-insert-at-eof t)
   (setq mail-yank-prefix "> "))
;;----------------------------------------------------------------------------
(add-hook 'mail-mode-hook 'add2-mail-mode)
(defun add2-mail-mode () "Additions to mail-mode"
   (setq local-abbrev-table 'mail-abbrev-table)
   (modify-syntax-entry ?\; "w"  mail-mode-syntax-table)  ; Abbrev starter
   (mail-abbrevs-setup)
   (define-key mail-mode-map "\C-n" 'mail-abbrev-next-line)
   (setq context-help-function    'show-mail-aliases)
   (setq compile-buffer-function  'mail-send-and-exit-nil)
;  (define-key mail-mode-map "\t" 'tab-or-mailname-complete)
   (make-local-variable 'comment-start) (setq comment-start "> ")
   (setq mail-signature t)
   (alt-umlaut-on))
;;
;;(add-hook 'mail-setup-hook 'mail-delete-signature-separator t)
(defun mail-delete-signature-separator ()
  "Delete the mail's standard signature-seperator (if any)."
  (save-excursion (goto-char (point-max))
                  (if (search-backward "\n\n-- \n" nil t)
                      (replace-match "" nil t))
                  (set-buffer-modified-p nil)))
;;
;(defun tab-or-mailname-complete ()
;"Dynamic mailname completion if point is on To: CC: or BCC: line
;-- just standard tab-function otherwise !"
;   (interactive)
;   (if (save-excursion (beginning-of-line 1) (looking-at "To:\\|CC:\\|BCC:"))
;       (progn (shellc-dynamic-complete-mailname)
;              (if (equal (char-before) ? ) (backward-char-h)))
;       (tab-left-right)))
;;
;(autoload 'shellc-dynamic-complete-mailname "shellc"
;          "Dynamically complete mail name at point." t)
;;----------------------------------------------------------------------------
(defun mail-send-and-exit-nil () (mail-send-and-exit nil))
;;;
(defun show-mail-aliases () "Show all mail-aliases in *Help* Buffer."
   (interactive)
   (with-output-to-temp-buffer "*Help*"
        (princ "Known mail-aliases (from \"~/.mailrc\" and includes)'):\n")
        (let ((list (sort (copy-alist mail-aliases) 'mail-car-Lessp)) entry)
             (while list (setq entry (car list))
                   (princ (format "%-20s - %s\n" (car entry) (cdr entry)))
                   (setq list (cdr list))))))
;;;
(defun mail-car-Lessp (a b) "Special sort predicate for show-mail-aliases"
      (if (string= (downcase (car a)) (downcase (car b)))
          (string< (car b) (car a))
          (string< (downcase (car a)) (downcase (car b)))))
;;
;;----------------------------------------------------------------------------
(define-abbrev-table 'mail-mode-abbrev-table nil)
(define-abbrev mail-mode-abbrev-table ";mfg" "" 'mail-greeting-mfg)
(define-abbrev mail-mode-abbrev-table ";fg"  "" 'mail-greeting-fg)
(define-abbrev mail-mode-abbrev-table ";hg"  "" 'mail-greeting-hg)
(define-abbrev mail-mode-abbrev-table ";mg"  "" 'mail-greeting-mg)
(define-abbrev mail-mode-abbrev-table ";sy"  "" 'mail-greeting-sy)
(define-abbrev mail-mode-abbrev-table ";br"  "" 'mail-greeting-br)
;;
(defun mail-greeting-mfg () "" (interactive)
      (mail-greeting-wrap "Sehr geehrter Herr" "Mit freundlichen Gruessen"))
(defun mail-greeting-fg () "" (interactive)
      (mail-greeting-wrap "Hallo Herr" "Freundliche Gruesse"))
(defun mail-greeting-mg () "" (interactive)
      (mail-greeting-wrap "Hallo" "Gruss"))
(defun mail-greeting-hg () "" (interactive)
      (mail-greeting-wrap "Lieber" "Herzliche Gruesse"))
;;
(defun mail-greeting-sy () "" (interactive)
      (mail-greeting-wrap "Dear Sirs !" "Sincerely yours"))
(defun mail-greeting-br () "" (interactive)
      (mail-greeting-wrap "Dear" "Best regards"))
;;
(defun mail-greeting-wrap (anrede gruss) "Insert greeting strings"
   (let ((pos (point)))
        (goto-char (point-min))
        (if (search-forward "--text follows this line--" nil t) (forward-line))
        (insert anrede) (end-of-line) (setq pos (point))
        (forward-line 1)
        (if (search-forward "-----------" nil t) (beginning-of-line 1))
        (insert (format "\n%s,\n%s\n\n" gruss user-full-name))
        (goto-char pos)))
;;----------------------------------------------------------------------------
;; Mail-Crypt:
;; ------------
(autoload 'mc-encrypt-message        "mailcrypt" nil t)
(autoload 'mc-sign-message           "mailcrypt" nil t)
(autoload 'mc-insert-public-key      "mailcrypt" nil t)
(autoload 'mc-rmail-decrypt-message  "mailcrypt" nil t)
(autoload 'mc-rmail-verify-signature "mailcrypt" nil t)
(autoload 'mc-snarf-keys             "mailcrypt" nil t)
;;----------------------------------------------------------------------------
(provide 'gold_mail)
