;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  MORE additional/different emacs functions/definitions for gold.el ! ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_more")
;;------------------------
(setq suggest-key-bindings nil) ; We do NOT want to see equivalent key-bindings !

;; Message of the day to buffer:
;;-------------------------------
(defun motd () "Load and display Message-Of-The-Day"
  (interactive) (find-file-read-only "/etc/motd")
  (rename-buffer "*MOTD*"))
(defun new-motd () "Load and display Message-Of-The-Day if not already read."
  (interactive)
  (if (file-newer-than-file-p "/etc/motd" "~/.motd")
      (progn (copy-file "/etc/motd" "~/.motd" t)
             (find-file-read-only "/etc/motd") (rename-buffer "*MOTD*")
             (message "NEW Message-Of-The-Day (made a copy to ~/.motd) !"))
      (message "Message-Of-The-Day unchanged (copy lies on ~/.motd) !")))

;;-----------------------------------------------------------------------------
;; Permas-Infos:
;;---------------
(defvar DEFPROG
  (if (or (equal (GETENV "DEFPROG") "permd")
          (equal (GETENV "DEFPROG") "perm5")
          (equal (GETENV "DEFPROG") "permnew")) "permnew" "permas")
  "*default progname for permnew/permas.")
;;
(defvar permas-files-dir
   (if (file-exists-p "/source")
       (if (equal DEFPROG "permnew")
           (if (file-readable-p "/source/permnew/FILES")
                "/source/permnew/FILES" "/source/permnew/files")
           (if (file-readable-p "/source/permas/FILES")
                "/source/permas/FILES" "/source/permas/files")))
   "*Name of directory which holds e.g. the PERMAS 'formats' file.")
;;
(defun perminfo-mode ()
  "Just normal text-mode with an own symbol for special PermInfo files."
  (interactive) (text-mode)
  (setq mode-name "PermInfo") (setq buffer-read-only t)
  (setq major-mode 'perminfo-mode) (run-hooks 'perminfo-mode-hook))
;;
(defun crontab-info () "Load info file for standard cron-jobs."
  (interactive) (find-file-read-only "/doc/share/files/crontab.info"))
;;
(defun permas-client () "Load info file for PERMAS Clients."
  (interactive) (find-file-read-only (read-file-name "Client: "
                                      "/ds-1/cusdata/customer/text/")))
;;
(defun edit-client () "Load info file for PERMAS Clients."
  (interactive) (find-file (read-file-name "Client: "
                           "/ds-1/cusdata/customer/source/")))
;;
(defun felix-rev () "Load info file for Felix Revisions."
  (interactive) (find-file-read-only "/doc/share/files/felix.rev"))
;;
(defun permas-config () "Load info file for PERMAS Clients."
  (interactive) (find-file-read-only "/doc/share/files/permas.config"))
;;
(defun permas-notes () "Load `global-note-file' for PERMAS Clients."
  (interactive) (find-file-read-only global-note-file))
;;
(defun permas-doc () "Load info file for PERMAS Documentation."
  (interactive) (find-file-read-only "/doc/share/files/permas.doc"))
;;
(defun permas-rev () "Load info file for PERMAS Revisions."
  (interactive) (find-file-read-only "/doc/share/files/permas.rev"))
;;
(defun permas-todo () "Load Infofile for PERMAS-TODO list."
  (interactive) (find-file-read-only "/doc/share/files/TODO-dev"))
;;
(defun prepost-rev () "Load Infofile for PrePost-Releases."
  (interactive) (find-file-read-only "/doc/share/files/prepost.rev"))
;;
(defun permdoc-v10 () "Load Infofile for Changes to /doc/tresor/v10/..."
  (interactive) (find-file-read-only "/doc/tresor/v10/ChangeLog"))
;
(defun permdoc-v11 () "Load Infofile for Changes to /doc/tresor/v11/..."
  (interactive) (find-file-read-only "/doc/tresor/v11/ChangeLog"))
;;
(defun permas-error () "Load gnats summary of \"permas\" category."
  (interactive) (if (find-file-read-only "/doc/share/files/permas_errors")
                    (message " ")))
;;
(defun permas-formats () "Load PERMAS formats file."
  (interactive) (find-file-read-only "/source/permas/FILES/formats"))
;;
(defun permnew-formats () "Load PERMAS formats file."
  (interactive) (find-file-read-only "/source/permnew/FILES/formats"))
;;
(defun permnew-entries () "Load PERMAS entries file."
  (interactive) (find-file-read-only "/source/permnew/qlinux/perm.ent1310"))
;;
(defun permas-entries () "Load PERMAS entries file."
  (interactive) (find-file-read-only "/source/permas/qlinux/perm.ent1300"))
;;
(defun perm4-rev () "Load Infofile for PERMAS Version 4 Revisions."
  (interactive) (find-file-read-only "/doc/share/files/perm4.rev"))
;;
(defun perm4-error () "Load gnats summary of \"perm4\" category."
  (interactive) (find-file-read-only "/doc/share/files/perm4_errors"))
;;
(defun resources () "Load resources.info file."
  (interactive) (find-file "/doc/share/files/resources.info"))
;;
(defun local-environment () "Load info file concerning 'local environment'."
  (interactive) (find-file "/doc/share/files/local_environment"))
;;
(defun f90-errors () "Load info file with global F90 error messages."
  (interactive) (find-file-read-only "/source/permnew/FILES/F90.errors"))
;;
(defun F90-Errors () "Load info file with global F90 error messages."
  (interactive) (find-file-read-only "/source/permnew/FILES/F90.errors"))
;;
(defun permnew-f90-errors () "Load info file with global F90 error messages."
  (interactive) (find-file-read-only "/source/permnew/FILES/F90.errors"))
;;
;;
;; Tools to deal with PARMACS
;;----------------------------
(defun parmacs-combine-trace ()
  "Combine strange splitted PARMACS trace back into readable lines."
  (interactive)
  (combine-parmacs-trace-lines)
  (while (re-search-backward "] *$" (- (point) 10))
         (combine-parmacs-trace-lines)))
;;
(defun combine-parmacs-trace-lines () " "
  (let ((end (re-search-forward "] *$" (+ (point) 150) t))
        (start (re-search-backward "^([ 0-9]*)" (- (point) 150) t))
        (lines 0))
       (if (and start end) (setq lines (count-lines start end)))
       (while (< 0 lines) (combine-lines) (setq lines (- lines 1)))
       (beginning-of-line 1)
  ))
;;-----------------------------------------------------------------------------
(defun count-string-matches (regexp string &optional start)
"Return the number of matches for REGEXP in STRING.  If optional
argument START is non-nil, count matches from that index in STRING."
  (let ((match-data (match-data)) (count 0))
    (unwind-protect (while (string-match regexp string start)
                           (setq count (1+ count) start (match-end 0)))
                    (store-match-data match-data)) count))
;;
(defun match-string (n &optional string)
  "Return match of `(match-beginning N)...(match-end N)' from STRING.
If STRING is not given, use the current buffer.  See `string-match'."
   (if (stringp string)
       (substring string (match-beginning n) (match-end n))
       (buffer-substring-no-properties (match-beginning n) (match-end n))))
;;
(defun match-string-tail (n &optional string)
  "Return tail of `(match-beginning N)...(match-end N)' from STRING.
If STRING is not given, use the current buffer.  See `string-match'."
   (if (stringp string)
       (substring string (match-end n))
       (buffer-substring-no-properties (match-end n))))
;;
(defun replace-regexp-in-region (regexp string start end)
  "Replace REGEXP by STRING in buffer region from START to END."
  (goto-char start)
  (while (re-search-forward regexp end t)
         (replace-match string t nil)))
; ? end->start since multi-repl. changes end recursively
;;
;;-----------------------------------------------------------------------------
(setq comment-style 'plain) ; do *not* use indent for comment-start
;;-----------------------------------------------------------------------------
(defun show-next-error ()
"Show next available error message for actual buffer (context-sensitive)."
  (interactive)
      (cond
       ((or (equal major-mode 'permas-pro-mode)
            (equal major-mode 'permas-res-mode)) (next-permas-error))
       ((equal major-mode 'shell-mode)    (next-shell-error))
       (t (call-interactively 'next-error))))
;;
(defun next-permas-error ()
   "Search and show next/previous PERMAS error in current buffer."
   (interactive)
   (if advance
      (or (re-search-forward  permas-error-regexp nil t)
          (re-search-forward  permas-warn-regexp nil t)
          (re-search-forward  permas-comment-regexp))
      (or (re-search-backward permas-error-regexp nil t)
          (re-search-backward permas-warn-regexp nil t)
          (re-search-backward permas-comment-regexp))))
;;
(defvar fortran-error-regexp "\\(/[^ :]+[.]f:[.0-9]+\\):"
  "Regular expression for fortran-compiler error messages.")
;;
(defvar compiler-error-regexp
  "\\([^ :\n]+\\(:  *\\|, line \\|(\\)[0-9]+\\)\\|\\([0-9]+ *of *[^ \n]+[.]\\)"
  "Regular expression for filename/linenumber in general compiler error.")
;;
(defvar permas-fatal-regexp "\\(^ ?\\*[FX]\\*\\)\\|\\(==\\*[FX]\\*\\)"
       "Regular expression for PERMAS error messages.")
;;
(defvar permas-error-regexp "\\(^ ?\\*[EFX]\\*\\)\\|\\(==\\*[EFX]\\*\\)"
       "Regular expression for PERMAS error messages.")
;;
(defvar permas-warn-regexp "\\(^ ?\\*[WEFX]\\*\\)\\|\\(==\\*[WEFX]\\*\\)"
       "Regular expression for PERMAS error messages.")
;;
(defvar permas-comment-regexp "\\(^ ?\\*[CWEFX]\\*\\)\\|\\(==\\*[CWEFX]\\*\\)"
       "Regular expression for PERMAS error messages.")
;;
(defvar shell-error-regexp (concat "\\(" permas-error-regexp "\\)\\|"
       "\\(" tex-error-regexp "\\)\\|" fortran-error-regexp "\\|" compiler-error-regexp)
       "Regular expression for error messages in shell buffers.")
;;
(defun find-permas-error ()
  "Tag search for permas error call (needs group and number as string)"
  (interactive)
  (let ((find-file-hooks (append find-file-hooks (list 'tags-find-file-hook)))
        (err-regexp (errnum-at-point)))
       (if err-regexp (tags-search err-regexp))
       (setq this-command 'tags-search-h)
       (setq last-tag-search "search-tags")))
;
(defun errnum-at-point ()
"Returns a regexp with the next PERMAS error/format number around point."
  (condition-case () (save-excursion
  (skip-chars-backward " 0-9/")
; (or (looking-at "[0-9]+/[0-9]+") (beginning-of-line 1))
  (skip-chars-forward " *CWEFX")
  (if (looking-at "[0-9]+/[0-9]+")
      (replace-regexp-in-string "/" " ?"
      (buffer-substring-no-properties (match-beginning 0) (match-end 0)))))))
;
(defun read-integer-at-point (text)
  (let ((enable-recursive-minibuffers t))
       (read-with-default text (integer-at-point))))
;
;;-----------------------------------------------------------------------------
(defun permas-help-mode ()
  "Just normal text-mode with a own symbol to enable special context-help."
  (interactive) (text-mode) (setq mode-name "PERMAS-HELP")
  (setq major-mode 'permas-help--mode) (run-hooks 'permas-help-mode-hook))
;;
(defun delete-permas-headlines ()
"Delete all PERMAS Headlines in current buffer, i.e. all lines starting
with \"1 PERMAS \" (the each match following blank line is also deleted).
The command starts at point (actual cursor) and searches forward up to EOF."
      (interactive) (while (re-search-forward "^1 PERMAS " nil t)
                    (delete-current-line 2)))
(defun compress-res ()
      "Compress PERMAS RES-Files in order to enlighten Dump-Reading !"
      (interactive)
      (goto-char (point-min)) (cut-trailing-blanks) (delete-permas-headlines)
      (goto-char (point-min))
      (while (re-search-forward "^ Status: " nil t)
             (previous-line 1) (delete-current-line 1) (next-line 1))
      (goto-char (point-min)) (replace-regexp "^ Status: " " Status :")
      (goto-char (point-min)) (while (re-search-forward "^ Parameters" nil t)
                                     (next-line 1) (delete-current-line 1))
      (goto-char (point-min)) (replace-regexp "^ Parameters"
                                              "  Parameters and Descriptors:")
      (goto-char (point-min))
      (while (re-search-forward "^ Level descriptor" nil t)
             (previous-line 2) (delete-current-line 4))
      (goto-char (point-min))
      (while (re-search-forward "^ Dump of level " nil t)
             (previous-line 2) (delete-current-line 4))
      (goto-char (point-min))
      (while (re-search-forward "^ Current level " nil t)
             (next-line 1) (if (eolp) (delete-current-line 1)))
      (goto-char (point-min)) (replace-regexp "^ List "
                                              "  List:")
      (goto-char (point-min))
      (while (re-search-forward "^  List:" nil t)
             (next-line 1) (if (eolp) (delete-current-line 1)))
      (goto-char (point-min)) (replace-regexp "^ Current level \\(.*\\)$"
                                              "  Current level \\1 and dump:"))
;;;*****
(gold-debug "set-hooks")
(add-hook 'text-mode-hook 'text-mode-ON)
(remove-hook 'text-mode-hook 'turn-on-auto-fill)
(defun text-mode-ON ()    "Additionals when changing to text-modes"
  (or x-terminal (local-set-key "\n" 'scroll-up-down)) ; Qume-"LFD/QUME"
  (local-set-key "\t" 'tab-left-right)
)
;;
;;;----------------------------------------------------------------------------
(add-hook 'lisp-mode-hook       'lisp-mode-ON)
(add-hook 'emacs-lisp-mode-hook 'lisp-mode-ON)
(defun lisp-mode-ON () "Additionals when changing to (emacs)-lisp-modes"
  (or x-terminal (local-set-key "\n" 'scroll-up-down)) ; Qume-"LFD/QUME"
  (local-set-key "\t"    'tab-left-right)
  (local-set-key "\177"  'delete-previous-char)
  (local-set-key "\M-#"  'lisp-spell-symbol)
  (substitute-key-definition 'ispell-word 'lisp-spell-symbol
                             (current-local-map) global-map)
  (setq context-help-function   'elisp-context-help)
  (setq compile-buffer-function 'elisp-compile-buffer)
)
(autoload 'lisp-spell-symbol "fuzzy-match"
                             "Spell check Lisp symbol preceding point." t)
(or xemacs
(define-key-after ispell-menu-map [ispell-symbol]
  '("Check Symbol" . lisp-spell-symbol) 'ispell-word)
)
;;
(defun elisp-compile-buffer () " " (byte-compile-file (buffer-file-name)))
(defun elisp-context-help () "Context-help for actual lisp symbol at point."
  (interactive)
  (let ((type      "FUNCTION/COMMAND")
        (keysymbol (or (variable-at-point) 0)))
       (if (not (equal keysymbol 0)) (setq type "VARIABLE")
           (setq keysymbol (function-called-at-point)))
       (if keysymbol (progn
          (if (equal type "VARIABLE") (describe-variable keysymbol)
                                      (describe-function keysymbol))
          (message (concat "Description of lisp " type
                           " \"" (symbol-name keysymbol) "\"")))
          (message (concat "Lisp symbol '" (word-at-point)
                           "' unknown (invalid or local symbol) !")))))
;;;----------------------------------------------------------------------------
;;
;No hook for fundamental-mode: redefine default-tabulator-function !
(setq indent-line-function 'tab-left-right)
;;----------------------------------------------------------------------------
(defun month-year () "Return actual date in form Jan-1991"
   (let ((string (current-time-string)))
        (concat (substring string 4 7)  "-"
                (substring string 20))))
;;
(defun month () "Return actual month in form Jan"
   (substring (current-time-string) 4 7))
;;
(defun year () "Return actual year in form 1991"
   (substring (current-time-string) 20))
;;
(defun day-month-year () "Return current date in form 06-Jan-1991"
   (let ((string (current-time-string)))
        (concat (if (string= (substring string 8 9) " ")
                    (concat "0" (substring string 9 10))
                    (substring string 8 10)) "-"
                (substring string 4 7)  "-"
                (substring string 20))))
;;
(defun hh-mm-ss () "Return current time in form 12:04:57"
   (substring (current-time-string) 11 19))
;;
(defun dd-mon-yyyy () "Return current date in form 06-Jan-1991"
   (let ((string (current-time-string)))
        (concat (if (string= (substring string 8 9) " ")
                    (concat "0" (substring string 9 10))
                    (substring string 8 10)) "-"
                (substring string 4 7)  "-"
                (substring string 20))))
;;
(defun dd-mon-yy () "Return actual date in form 17-Jan-98"
   (let ((string (current-time-string)))
        (concat (if (string= (substring string 8 9) " ")
                    (concat "0" (substring string 9 10))
                    (substring string 8 10)) "-"
                (substring string 4 7)  "-"
                (substring string 22))))
;;
(defun mon-yy () "Return actual date in form Jan-98"
   (let ((string (current-time-string)))
        (concat (substring string 4 7)  "-"
                (substring string 22))))
;;
(defun mon-yyyy () "Return actual date in form Jan-2001"
   (let ((string (current-time-string)))
        (concat (substring string 4 7)  "-"
                (substring string 20))))
;;
(defun day-month-year-time ()
   "Return actual date & time in form 06-Jan-1991 14:12"
   (concat (day-month-year) " "
           (substring (current-time-string) 11 16)))
;;
(defun update-day-month-year ()
"Update day-month-year string in line (if an old date string is found)
respectively just insert actual day-month-year string otherwise."
  (interactive)
  (let ((save (point)) eol)
     (end-of-line) (setq eol (point)) (beginning-of-line)
     (if (re-search-forward "[ 0-9][0-9]-[A-Z][a-z][a-z]-[1-2][0-9][0-9][0-9]" eol t)
         (replace-match (day-month-year))
     (if (re-search-forward "[ 0-9][0-9]-[A-Z][a-z][a-z]-[0-9][0-9]" eol t)
         (replace-match (dd-mon-yy))
     (if (re-search-forward "[A-Z][a-z][a-z]-[1-2][0-9][0-9][0-9]" eol t)
         (replace-match (month-year))
     (if (re-search-forward "[A-Z][a-z][a-z]-[0-9][0-9]" eol t)
         (replace-match (mon-yy))
         (goto-char save) (insert (dd-mon-yy))
     ))))
     (if (re-search-forward "[ 0-2][0-9]:[0-5][0-9]:[0-5][0-9]" eol t)
         (replace-match (hh-mm-ss)))
     (goto-char save)))
;;----------------------------------------------------------------------------
;;
(defun permas-logtime () "Get PERMAS LogTime from start to end of region,
respectively from start of PERMAS run to current position (if no region)."
  (interactive)
  (let ((bound (point)) time0 time1 timex (time 0)) (save-excursion
       (if mark-active (progn (setq bound (region-end))
                              (goto-char (region-beginning)))
           (or (search-backward "P E R M A S" nil t) (goto-char (point-min))))
       (beginning-of-line)
       (setq time0 (get-next-logtime)) (setq time1 time0)
       (while (setq timex (get-next-logtime bound))
              (if (< timex time1) ; CA-Hack is not fully save, but works in most cases!
                  (or (castatus-logtime-exception) (setq time (+ time 86400))))
              (setq time1 timex))
       (if time1 (progn
           (setq time (+ time time1 (- time0)))
           (kill-new (format "%i =%s" time (sec-to-hhhmmss time)))
           (message "LogTime = %i [sec] = %s [hh:mm:ss]" time (sec-to-hhhmmss time)))
           (beep) (message "No valid LogTime found !"))
)))
;;
(defun castatus-logtime-exception ()  "non-nil if current line is a CA-Status-LOG-line,
but preceeding line with time isn't or vice versa."
 (if (equal major-mode 'permas-pro-mode) (save-excursion (beginning-of-line 1)
  (let ((caloglin ".* [|│║][ 0-9]*:\\|        ?[+]--------"))
  (if (looking-at caloglin) ; Special treatment for CA-Status-lines (V10)
      (progn (beginning-of-line 0) (not (looking-at caloglin)))
      (progn (beginning-of-line 0) (looking-at caloglin)))))))
;;
(defun get-next-logtime (&optional bound) "Get next PERMAS LogTime in seconds."
 (let (logtime)
   (while (and (re-search-forward "^  ?\\([0-2][0-9][0-5][0-9][0-5][0-9]\\) " bound t)
               (setq logtime (match-string 1))
               ; Keep while running if current line is a PSUM-Table-Entry
               (looking-at "NewEntrie\\|.*[()]") ; Stop if standard log-line
          ))
   (if logtime (logtime-to-sec logtime))))
;;
(defun logtime-to-sec (ctime) "Convert LogTime string to integer seconds."
  (let (hh mm ss)
       (setq hh (string-to-number (substring ctime 0 2)))
       (setq mm (string-to-number (substring ctime 2 4)))
       (setq ss (string-to-number (substring ctime 4 6)))
       (+ ss (* 60 (+ mm (* 60 hh))))))
;;
(defun sec-to-hhhmmss (sec) "Convert integer seconds to hhh:mm:ss string."
  (let (hh mm ss)
       (setq hh (floor (/ sec 3600)))
       (setq mm (floor (/ (mod sec 3600) 60)))
       (setq ss (mod sec 60))
       (format "%3.1i:%2.2i:%2.2i" hh mm ss)))
;;----------------------------------------------------------------------------

;; Customizing revert-buffer
;;---------------------------
(setq revert-without-query '(".*"))
(defvar revert-hook-opoint 0 "Original position preserved by revert-buffer")
(add-hook 'before-revert-hook 'gold-before-revert-hook t)
(add-hook 'after-revert-hook  'gold-after-revert-hook  t)
(defun gold-before-revert-hook () (setq revert-hook-opoint (point)))
(defun gold-after-revert-hook  () (goto-char (min revert-hook-opoint (point-max))))

;;----------------------------------------------------------------------------
(defun unconditional-new-buffer (buffer-name &optional display)
  "Create a new buffer unconditianally (i.e. kill old contents) and make it the
current buffer (show it if display is non-nil only)."
  (if (get-buffer buffer-name) (kill-buffer buffer-name))
  (generate-new-buffer buffer-name)
  (if display (if (equal display 'other)
                  (switch-to-buffer-other-window buffer-name)
                  (switch-to-buffer buffer-name))
              (set-buffer       buffer-name)))
;;----------------------------------------------------------------------------
(defvar temp-directory "/tmp/"
"*Name of temporary directory for holding shadow files during Lisp-Functions,
must have read/write/execute rights for world (or at least for current user).")
;
(defun shell-cmd-region (command start end &optional batch)
"Invoke a shell COMMAND which normally expects a file name as final argument
for the specified region START to END (creates a temporary dummy-file).
Execute in batch mode if optional fourth argument BATCH is non-nil."
  (save-excursion (message (concat "Invoking " command))
   (let ((dummy-file (file-name-for-region start end))
         (dummy-buffer (get-buffer-create "*DUMMY-process-output*")))
        (write-region start end (concat temp-directory dummy-file) nil 'silent)
        (set-buffer dummy-buffer) (setq default-directory temp-directory)
    (if batch
        (progn (call-process shell-file-name nil 0 nil "-fc" (concat command
                               " \"" dummy-file "\" ; rm \"" dummy-file "\""))
               (message (concat "Executing batch \"" command
                                 "\" for \"" dummy-file "\"")))
        (let ((executing-macro t)); Silent (push-mark): No Mark-Set-Message !
             (shell-command (concat command " \"" dummy-file "\"") t))
        (if (< (point-min) (point-max))
            (message (buffer-substring (point-min) (- (point-max) 1)))
            (message (concat command " done !")))
        (delete-file dummy-file))
    (kill-buffer dummy-buffer))))
;
(defalias 'gl 'glimpse)
(defun glimpse (string) "test glimpse"
   (interactive "sRE-Search: ")
   (let ((buffer (get-buffer-create "*glimpse*"))         ;; -b
         (command "glimpse -H /source/permnew/FILES -n -i -e "))
        (set-buffer buffer) (erase-buffer)
        (call-process shell-file-name nil buffer t "-fc"
                     (concat command "'" string "'"))
        (goto-char (point-min))
        (while (search-forward "/source/permnew/" nil t)
               (replace-match "" nil t))
        (goto-char (point-min))
        (insert (format "/source/permnew/: %d lines matching \"%s\"\n"
                (count-lines (point-min) (point-max)) string))
        (switch-to-buffer-other-window buffer)))
;
(defun file-name-for-buffer (pre-string)
      "Returns a valid file name for actual buffer with given PRE-STRING."
      (if (string-match "[* \"'/@]" (buffer-name))
          (concat (make-temp-name (concat pre-string "'")) "'")
          (concat pre-string "'" (buffer-name) "'")))
;;
(defun file-name-for-region (start end)
      "Returns a valid file name for actual region."
  (concat  (if (string-match "[ \"'/@]" (buffer-name))
                  (make-temp-name "Emacs_") (buffer-name))
      (if (region-equal-buffer start end) nil (concat
          "(" (number-to-string (count-lines (point-min) start))
          "-" (number-to-string (count-lines (point-min) end)) ")"))
))
;;
(defun region-equal-buffer (start end)
      "Tests wether START END matches whole buffer."
      (and (eq (point-min) start) (eq (point-max) end)))
;;----------------------------------------------------------------------------
(gold-debug "narrow")
(defun narrow () "Narrow displayed and adressable text to current region."
      (interactive) (narrow-to-region (region-beginning) (point))
      (message
"Adressable text narrowed to current region:  Return with Gold-<  (widen)."))
;;----------------------------------------------------------------------------
; Some dummy variable which are intended to be used with insert-variable !
(defvar USER-1 nil "*Dummy variable for users local macro definitions.")
(defvar USER-2 nil "*Dummy variable for users local macro definitions.")
(defvar USER-3 nil "*Dummy variable for users local macro definitions.")
(defvar USER-4 nil "*Dummy variable for users local macro definitions.")
(defvar USER-5 nil "*Dummy variable for users local macro definitions.")
(defvar USER-6 nil "*Dummy variable for users local macro definitions.")
(defvar USER-7 nil "*Dummy variable for users local macro definitions.")
(defvar USER-8 nil "*Dummy variable for users local macro definitions.")
(defvar USER-9 nil "*Dummy variable for users local macro definitions.")
(defvar USER-0 nil "*Dummy variable for users local macro definitions.")
;
(defvar insert-variable-width "1"
"*Minimal width of field (and optional pre-format character \"-\" or \"0\") used
for insert-variable.  If the variable exceeds the given range then always the
full contents is printed.  Mark that insert-variable-width should be a string !
E.g.
\"12\" - Print variables RIGHT-ADJUSTED in a field of at least 12 characters.
\"-5\" - Print variables LEFT-ADJUSTED in a field of at least 5 characters.
\"06\" - Right-adjusted with at least 6 characters, numbers with LEADING ZEROes."
)
(defun insert-variable-1 (var)
"Insert contents of a user variable at actual cursor position and increments the
variable by +1    --> See variable insert-variable-width also !"
  (interactive "vVariable to be inserted (and incremented by +1): ")
  (insert-variable var)
  (if (integerp (symbol-value var))
      (set var (+ (symbol-value var) 1))
      (message (concat (prin1-to-string var)
               " is not a integer variable (not incremented) !"))))
(defun insert-variable (var)
"Insert contents of a user variable at actual cursor position
--> See variable insert-variable-width (and function insert-variable-1) also !"
  (interactive "vSpecify variable to be inserted: ")
  (let ((symbol (symbol-value var))
        (varnam (prin1-to-string var)))
        (cond ((integerp symbol)         (insert-variable-princ "d" symbol))
              ((char-or-string-p symbol) (insert-variable-princ "s" symbol))
              ((eq symbol nil)(message (concat "Value of " varnam " is nil !")))
              (t (message (concat varnam " is not a printable symbol !"))))))
(defun insert-variable-princ (form symbol) "Dummy function for insert-variable"
      (princ (format (concat "%" insert-variable-width form) symbol)
             (current-buffer)))
;;----------------------------------------------------------------------------
(setq require-final-newline t)     ;; 'yes will ask user before inserting !
(setq-default require-final-newline t); Makes this the default for buffer-local
(defun test-require-final-newline () (interactive)
 (or require-final-newline (message ">>> require-final-newline is nil ! <<<")))
;;--------------------------------------------------------------------
(setq auto-save-interval 1000)
(setq window-min-height 2)
;;--------------------------------------------------------------------
(gold-debug "user-names")
(defvar user-login-name (user-login-name)
  "The name under which the user is logged in.")
;;
(defvar user-two-letter-name
   (let ((first-blank (string-match " " (user-full-name))))
       (if first-blank (concat (substring (user-full-name) 0 1)
       (substring (user-full-name) (+ first-blank 1) (+ first-blank 2))) "??"))
   "*User's name abbreviated by two (upper-case) characters.")
;;
(defvar user-first-name
   (let ((first-blank (string-match " " (user-full-name))))
        (if first-blank (capitalize (substring (user-full-name) 0 first-blank))
                        (concat "(" (user-login-name) ")" )))
   "*Users first name as a string.")
;;
(defvar user-extern-name
   (let ((first-blank (string-match " " (user-full-name))))
       (if first-blank (concat (substring (user-full-name) 0 1) ". "
                   (capitalize (substring (user-full-name) (+ first-blank 1))))
          (concat "(" (user-login-name) ")" )))
   "*User's name used for extern sender/adresses in form H. Mueller !")
;
(defvar system-full-name (system-name) "Long name of system.")
(defvar system-short-name
   (substring system-full-name 0 (string-match "[.]" system-full-name))
   "Short name of system.")
(or (and (boundp 'system-configuration) system-configuration)
    (setq system-configuration system-full-name))  ;; For old emacs
;;
(setq-default mode-line-format
    '(" " user-login-name "@" system-short-name "> " mode-line-variable
      "%1*%1* %b "     ;;mode-line-format and buffer-name !
      "%[(" mode-name mode-line-process minor-mode-alist "%n)%]-"
   ;;; current-column ;; display current column/-number is slow
      (line-number-mode "%l/")
      (-3 . "%p") "--" global-mode-string "-%-"))
;;
;; Display date&time in mode-line, update every 1 minutes
(setq display-time-24hr-format t) ; Use 23:00 instead of 11:00pm
(or xemacs (display-time))
;;
;; We do not want slow line-numbering !
(line-number-mode 0)

;; Make unread mails beep on mode-line occurance:
(add-hook 'display-time-hook 'beep-on-new-mail)
(defvar beep-on-new-mail t "*Non-nil makes unread mails beep.")
(defun  beep-on-new-mail () "Make unread mails beep on mode-line occurance."
  (if (and beep-on-new-mail (string-match " Mail" display-time-string))
      (if new-mail-beeped nil (beep) (beep) (beep) (setq new-mail-beeped t))
      (setq new-mail-beeped nil)))
(defvar new-mail-beeped nil "Dummy to avoid multiple new-mail beeps.")
;;
;;----------------------------------------------------------------------------
(gold-debug "note")
;;
(defvar personal-note-file "~/.mail/Notes"
   "*File name for personal notes written by the `note' command'.")
;;
(defvar global-note-file "/ds-1/cusdata/customer/Notes"
   "*File name for global notes written by the `note' command'.")
;;
(defun note (contact text)
  "Write a 1-line note and append it (together with date & time) to the
`personal-note-file' plus to the `global-note-file' if GLOBAL is non-nil."
  (interactive "sContact: \nsText: ")
  (set-buffer (generate-new-buffer "*note-buffer*"))
  (princ (format "%s %s - %-10s: %s\n"
                 (day-month-year-time) user-two-letter-name contact text)
         (current-buffer))
  (write-region (point-min) (point-max) personal-note-file t nil)
  (if (y-or-n-p (format "Written to %s, append to %s also "
                         personal-note-file global-note-file))
      (write-region (point-min) (point-max) global-note-file t nil))
  (kill-buffer "*note-buffer*"))

;;----------------------------------------------------------------------------
(gold-debug "keypad")
;;18>(require 'keypad)
(defvar edt-last-deleted-lines ""
       "Last text deleted by an GOLD-emulation line-delete command.")
(defvar edt-last-deleted-words ""
       "Last text deleted by an GOLD-emulation word-delete command.")
(defvar edt-last-deleted-chars ""
       "Last text deleted by an GOLD-emulation character-delete command.")

;;****************************************************************************
;;*****  Test function for parentheses-levels (hot stuff for TeX'er)  ********
;;****************************************************************************
(gold-debug "parentheses")
(defun test-buffer-parentheses ()
  "Check current buffer for paragraphs containing mismatched parentheses
(and mismatched $'s for TeX files or mismatched text quotes for Lisp etc.).
Paragraphs are separated by blank lines or by any line containing nothing
but ; % * c or C within first column (rest must be blank). So if this algorithm
separates two one logical block which need a blank line in between then insert
(e.g. for TeX) a ' %' line instead (blank start with trailing comment).
As each erroneous paragraph is found, the location is displayed for a few
seconds and a mark is pushed at its beginning (so each such paragraphs can be
jumped to using GOLD-. afterwards).
Remark: TeX \\begin...\\end brackets are (actually) not taken into account
        and \verb... contents and \\$ or \\{ constructs are significant code.
        Also comment regions with EOL as comment delimiter (like in TeX) are
        taken as significant code !"
  (interactive)
  (let ((opoint (point)) (ok t)) (goto-char (point-max))
    (unwind-protect
        (while (not (bobp))
          (message
"Scanning buffer for mismatched parentheses (paragraphwise from bottom to top)")
          (let ((end (point))
                (start (point-min)))
            (if (re-search-backward "\n[;%*cC]? *\n" nil 'move)
                (progn (setq start (point))
                       (goto-char (match-end 0))))
            (or (test-region-parentheses (point) end)
                (progn (push-mark (point))
                (message "Mismatched parentheses in paragraph (starting here)")
                (sit-for 2)(setq ok nil)))
            (goto-char start)))
      (goto-char opoint))
  (if ok (message "Done")
     (message
"Mismatches found (old cursor pos):  Use repeatedly GOLD-. for start location !")
)))

(defun test-region-parentheses (start end)
  (condition-case ()
      (save-excursion
        (save-restriction
          (narrow-to-region start end)
          (goto-char start)
          (forward-sexp (- end start))
       t))
    (error nil)))

(defun jump-mark () "Jump to next mark position within mark-ring."
      (interactive) (set-mark-command t))

(defun show-close-bracket ()
"Show matching close-bracket of actual text level (level of point-position)"
      (interactive)
      (let ((blink-matching-paren-distance (buffer-size))
            (blink-matching-paren t)(save-point (point)))
       (forward-char 1)
       (if (= 0 (car (parse-partial-sexp (point) (point-max) -1)))
(message "No closing bracket found:  Cursor is at top text level !")
           (backward-char 1) (sit-for 1))
       (goto-char save-point)))

(defun show-open-bracket () ""
"Show matching opening-bracket of actual text level (level of point-position)"
      (interactive)
      (let ((blink-matching-paren-distance (buffer-size))
            (blink-matching-paren t)(save-point (point)))
           (parse-partial-sexp (point) (point-max) -1)
           (if (string= (blink-matching-open) "Mismatched parentheses")
               (message "No opening bracket found:  Cursor is at top text level !"))
           (goto-char save-point)))

;;****************************************************************************
;;*****  Common utilities needed for Help functions (and others) !   *********
;;****************************************************************************
(defun buffer-linestring ()
"Returns a copy of the actual line in actual buffer as a string.
Leaves point at beginning of line."
      (buffer-substring-no-properties (progn (end-of-line 1) (point))
                                      (progn (beginning-of-line 1) (point))))
;
(defun word-at-point () "Returns the word of actual statement at point."
   (save-excursion
   (let ((pos1)(pos2))
        (re-search-backward "\\b[^ ]" nil t) (setq pos1 (point))
        (forward-char 1)
        (re-search-forward "\\b" nil t) (setq pos2 (point))
        (if (< pos1 pos2) (buffer-substring-no-properties pos1 pos2) (eval "???")))))
;;
(defun gold-function-at-point () "Returns a usefull function-name at point."
   (save-excursion
   (let ((pos1)(pos2))
        (if (and (re-search-backward "[^a-z0-9_-]\\|^" nil t)
                 (re-search-forward  "[a-z]" nil t))
            (setq pos1 (- (point) 1)))
        (while (looking-at "[a-z0-9_-]") (forward-char))
        (if (re-search-backward  "[^_-]" nil t)
            (setq pos2 (+ (point) 1)))
        (if (and pos1 pos2 (< pos1 pos2))
            (buffer-substring-no-properties pos1 pos2) (eval "???")))))
;
(defun keyword-at-point (&optional separators)
"Returns the keyword (starting with $ or A-z) of actual statement at point.
Searches from beginning of line if SEPARATORS is nil (or no string).
Searches back to sol or first character from within SEPARATORS otherwise."
  (condition-case () (save-excursion
   (let ((pos1)(pos2)(char))
        (if (and separators (stringp separators)) (progn
            (re-search-backward (concat "^\\|[" separators "]") nil t)
            (setq pos2 (point)) (skip-chars-forward (concat " " separators)))
            (beginning-of-line) (setq pos2 (point)) (skip-chars-forward " |│║"))
        (setq pos1 (point)) (setq char (following-char))
        (if (or (> char 64) (= char 36))
            (progn(skip-chars-forward "$0-9A-Za-z") (setq pos2 (point))
                  (if (< pos1 pos2) (buffer-substring-no-properties pos1 pos2) (eval "???")))
            (goto-char pos2) (if (bobp) "???"
            (previous-line 1) (end-of-line) (keyword-at-point separators)))))))
;
(defun integer-at-point ()
"Returns a string with the next available integer number around point."
  (condition-case () (save-excursion
   (let ((pos1)(pos2))
        (forward-char 1) (forward-word -1) (setq pos1 (point))
        (skip-chars-forward "0-9")         (setq pos2 (point))
        (if (< pos1 pos2) (buffer-substring-no-properties pos1 pos2) (eval "0"))))))
;
(defun ihex-at-point ()
"Returns a string with the next available integer/hex number around point."
  (condition-case () (save-excursion
   (let ((pos1)(pos2)(base "10#"))
        (forward-char 1) (forward-word -1)
        (if (looking-at "[$]") (forward-char 1)) (setq pos1 (point))
        (if (not (looking-back "[$]"))
            (skip-chars-forward "0-9")
            (skip-chars-forward "0-9a-fA-F") (setq base "16#"))
        (setq pos2 (point))
        (if (< pos1 pos2)
            (concat base (buffer-substring-no-properties pos1 pos2))
            (eval "0"))))))
;;
(defun string-to-length (string length &optional adjust)
      "Force STRING to have given LENGTH (adjust to \"L\" or \"R\")."
      (if (equal adjust "R")
          (substring (concat (make-string length ? ) string) (* -1 length))
          (substring (concat string (make-string length ? )) 0 length)))
(defun change-char-in-string (string pos char) "Change char at given position."
      (concat (substring string 0 pos) char (substring string (+ pos 1))))

;;****************************************************************************
;;***** Searching and Replace function (including advance/Backup) ! **********
;;****************************************************************************
(gold-debug "q-replace")
;;V18>(load "q-replace" nil t)
(defun search-permas-error () "" (interactive) (beep)
(message "Removed function: Use show-next-error (vt200-F11) instead !"))
;;;
(setq search-exit-char 13)
(setq search-last-string  "C*(")
(setq search-last-regexp  "C.(")
(defvar last-search-mode  "?" "Type of last processed searching")
(defvar last-isearch-start -1 "Start position of last `isearch-back-forward'.")
;;
(defun isearch-back-forward ()
  "Do incremental search forward/backward depending on actual ADVANCE/BACKUP mode.
As you type characters, they are added to the search string and are found.
DEL - to cancel characters from end of search string.
RET - to exit, leaving point at location found.
C-w - to yank word from buffer onto end of search string and search for it.
C-y - to yank rest of line onto end of search string, etc.
C-q - to quote control character to search for it.
C-g - cancels search (move point to start or last position successfully found).
<9> - to repeat the search (forward/backward depends on ADVANCE/BACKUP).
All other key-functions terminate the search and are then executed normally !"
  (interactive)
  (if (and (eq last-command 'isearch-other-meta-char)
           (eq last-isearch-start (point))) (search-word-back-forward)
  (setq last-isearch-start (point))
  (setq saved-mark-active mark-active)   ; Save current value of mark-active
  (if advance (call-interactively 'isearch-forward)   ;; Do we need (pop-mark)
              (call-interactively 'isearch-backward)) ; (pop-mark) ; any more ?
  (setq last-search-mode "isearch")))
;;
(add-hook 'isearch-mode-end-hook 'reset-saved-mark-active)
(defun reset-saved-mark-active () (setq mark-active saved-mark-active))

(defun re-search-back-forward ()
  "Do regular expression search forward/backward depending on ADVANCE/BACKUP mode.
Set point to the end/beginning of the occurrence found, so the next search
works properly."
  (interactive) (setq search-last-regexp (read-with-default
                      "RE-Search" search-last-regexp))
  (if advance (re-search-forward  search-last-regexp)
              (re-search-backward search-last-regexp))
  (setq last-search-mode "re-search"))

(defun repeat-search ()
   "Repeat last Search forward/backward depending on ADVANCE/BACKUP mode."
       (interactive)
       (cond ((equal last-search-mode "isearch")
              (if advance (isearch-repeat-forward)
                          (isearch-repeat-backward)))
             ((equal last-search-mode "re-search")
              (message (concat "Repeated RE-search: " search-last-regexp))
              (if advance (re-search-forward  search-last-regexp)
                          (re-search-backward search-last-regexp))
              (message " "))
             ((equal last-search-mode "string-search")
              (message "Repeated search: %s" search-last-string)
              (if advance (search-forward  search-last-string)
                          (search-backward search-last-string)))
             (t (message "No repeatable search defined !"))))

(defun read-with-default (prompt default)
  "Interactively read an input string with PROMPT and DEFAULT (both strings)."
  (let ((input (read-input
               (if default (format (concat prompt " (default %s): ") default)
               (concat prompt ":")) "" )))
      (if (equal input "") default input)))

(defun read-no-blanks-with-default (prompt default)
  "Interactively read an input string with PROMPT and DEFAULT (both strings)."
  (let ((input (read-no-blanks-input
               (if default (format (concat prompt " (default %s): ") default)
               (concat prompt ": ")) "" )))
      (if (equal input "") default input)))

(defun all-occur () "ALL-occur with default from last regexp-search."
   (interactive)
   (let ((original-pos (point)) (enable-recursive-minibuffers t))
        (setq search-last-regexp (read-with-default
              "List lines matching regexp" search-last-regexp))
        (goto-char (point-min)) (occur search-last-regexp)
        (goto-char original-pos) (setq last-search-mode "re-search")))

(defun search-word-back-forward ()
"Do word-under-point search forward/backward depending on ADVANCE/BACKUP mode.
Set point to the end/beginning of the occurrence found, so the next search
works properly."
  (interactive)
  (if advance (call-interactively 'search-word-forward)
              (call-interactively 'search-word-backward))
  (setq last-search-mode "string-search"))

(defun search-word-forward (num)
  "Search forward for next occurrence of the word under point.
With prefix argument, search for NUMth occurrence."
  (interactive "p")
  (let ((target-string (buffer-substring-no-properties
                        (re-search-backward "\\<")
                        (re-search-forward "\\>" nil t 2))))
    (setq search-last-string target-string)
    (message "Search for \`%s\'" target-string)
    (search-forward target-string nil nil num)))

(defun search-word-backward (num)
  "Search backward for previous occurrence of the word under point.
With prefix argument, search for NUMth occurrence."
  (interactive "p")
  (let ((target-string (buffer-substring-no-properties
                        (re-search-backward "\\<")
                        (re-search-forward "\\>" nil t 2))))
    (setq search-last-string target-string)
    (message "Search for \`%s\'" target-string)
    (search-backward target-string nil nil (1+ num))))

;;----------------------------------------------------------------------------
(defvar protect-tabs nil "*Non-nil protects tabs on 'cut-trailing-blanks'")
(defun cut-trailing-blanks ()
"Removes all trailing blanks and expands all TAB characters into spaces !
Protects TAB characters if 'protect-tabs' is non-nil."
      (interactive)
      (advance-direction)
      (if protect-tabs nil
         (message "Replacing all tab characters by spaces (preserving columns) !")
         (untabify (point-min) (point-max)))
      (let ((line-number (or (equal major-mode 'fortran-mode)
                             (equal major-mode 'f77-mode)
                             (equal major-mode 'f7x-mode)
                             (equal major-mode 'f7x2f77-mode))))
           (if line-number
              (progn (goto-char (point-min)) (end-of-line)
                     (setq line-number (and (= (point) (+ (point-min) 80))
                                           (char-equal (preceding-char) ?0)))))
           (goto-char (point-min))
           (if line-number
              (progn (message "Eliminating all trailing line numbers !")
                     (while (re-search-forward " *[0-9]..[0-9]...0$" nil t)
                            (replace-match "" nil nil)))
              (message "Eliminating all trailing blanks !")
              (while (re-search-forward " +$" nil t)
                     (replace-match "" nil nil))))
      (goto-char (point-min))
      (message "Done"))

;;****************************************************************************
;;***** Cutting and Pasting ! ************************************************
;;****************************************************************************
(gold-debug "cut/paste")
(defvar fix-advance-copycut t
"*Standard direction (EOL/SOL) of COPY and CUT without preceding MARK command:
  nil   - COPY/CUT End-of-Line in ADVANCE and Start-of-Line in BACKUP mode.
non-nil - COPY/CUT always End-of-Line (independent of ADVANCE/BACKUP).")

(defvar rectangular-paste-buffer nil
"Non-nil specifies the active Paste-Buffer (from last copy/cut-paste command)
 to be a rectangle rather than a region.")

(if xemacs (defvar mark-active nil
"Non-nil means the mark and region are currently active in this buffer.
 Automatically local in all buffers."
))

(defun mark-paste ()
  "Mark region of text for COPY/CUT to Paste-Buffer.
The first `mark-paste' marks text for a new (normal) COPY/CUT. A directly
following second call of `mark-paste' will mark text for appending COPY/CUT."
  (interactive)
  (if (eq last-command 'mark-paste)
      (mark-paste-append) (mark-paste-normal)))

(defun mark-paste-normal ()
  "Mark region of text for normal COPY/CUT to Paste-Buffer."
  (interactive)
  (if (and mark-active (equal mark-active "column") rm-mark-active)
      (rm-deactivate-mark)) ;; Deactivate rectangular-mark explicitely
  (call-interactively 'set-mark-command)
  (if mark-active (setq mark-active "normal"))
  (if (equal major-mode 'calendar-mode)(call-interactively 'calendar-set-mark))
  (message "MARKed position for NORMAL cut/copy to Paste-Buffer !"))

(defun mark-paste-append ()
  "Mark region of text for appending COPY/CUT to actual Paste-Buffer."
  (interactive)
  (if (and mark-active (equal mark-active "column") rm-mark-active)
      (rm-deactivate-mark)) ;; Deactivate rectangular-mark explicitely
  (if rectangular-paste-buffer (error
"Mark-Append IMPOSSIBLE:  Paste-Buffer contains a rectangle (SET NEW MARK) !")
  (call-interactively 'set-mark-command)
  (if mark-active (setq mark-active "append"))
  (message "MARKed position for APPENDing cut/copy to Paste-Buffer !")))

(defun mark-paste-column ()
  "Mark a text rectangle for COLUMN-wise COPY/CUT to Paste-Buffer."
  (interactive)
;;(call-interactively 'set-mark-command)
  (call-interactively 'rm-set-mark)
  (if mark-active (setq mark-active "column"))
  (message "MARKed position for COLUMN-wise cut/copy to Paste-Buffer !"))

(defun copy-to-paste ()
"Copy EOL/SOL or a marked region respectively rectangle into the Paste-Buffer.
Depending on the preceding MARK command the following function will be invoked:
no mark     - Copy Start-of-Line (Backup-Mode) or End-of-line (Advance-Mode)
              into the Paste-Buffer (see also variable fix-advance-copycut).
MARK        - Copy the text region from last MARK up to actual cursor position
              into the Paste-Buffer (overwrite old contents).
MARK APPEND - Copy text region from MARK APPEND up to actual cursor position
              at the end of the actual Paste-Buffer (after acual contents).
MARK COL    - Copy the rectangle from MARK COL up to actual cursor position
              column-wise into the Paste-Buffer (overwrite old contents)."
  (interactive)
  (cond ((not mark-active)
         (let ((pos (point)))
          (if (or advance fix-advance-copycut) (end-of-line 1)
                                               (beginning-of-line 1))
          (call-interactively 'set-mark-command) (goto-char pos)
          (call-interactively 'copy-region-as-kill))
         (if (or advance fix-advance-copycut)
                (message "COPY End-of-Line to Paste-Buffer !")
                (message "COPY Start-of-Line to Paste-Buffer !"))
         (setq rectangular-paste-buffer nil))
        ;;
        ((equal mark-active "append")
         (if rectangular-paste-buffer (error
"Append-Copy IMPOSSIBLE:  Paste-Buffer contains a rectangle (SET NEW MARK) !")
         (append-next-kill)
         (call-interactively 'copy-region-as-kill)
         (setq rectangular-paste-buffer nil)
         (message "APPEND COPY of marked region to Paste-Buffer !")))
        ;;
        ((equal mark-active "column")
         (call-interactively 'copy-rectangle)
         (goto-char (region-beginning))
         (setq rectangular-paste-buffer t)
         (rm-deactivate-mark)
         (message "COLUM-wise COPY of marked rectangle to Paste-Buffer !"))
        ;;
        (t
         (call-interactively 'kill-ring-save)
         (setq rectangular-paste-buffer nil)
         (message "NORMAL COPY of marked region to Paste-Buffer !"))
  )
  (setq mark-active nil)        ; Unhighlighting region
  (setq this-command 'nothing)) ; Suppress appended kill-region(s)

(defun cut-to-paste ()
"Cut EOL/SOL or a marked region respectively rectangle into the Paste-Buffer.
Depending on the preceding MARK command the following function will be invoked:
no mark     - Cut Start-of-Line (Backup-Mode) or End-of-line (Advance-Mode)
              into the Paste-Buffer (see also variable fix-advance-copycut).
MARK        - Cut the text region from last MARK up to actual cursor position
              into the Paste-Buffer (overwrite old contents).
MARK APPEND - Cut text region from MARK APPEND up to actual cursor position
              at the end of the actual Paste-Buffer (after acual contents).
MARK COL    - Cut the rectangle from MARK COL up to actual cursor position
              column-wise into the Paste-Buffer (overwrite old contents)."
  (interactive)
  (cond ((not mark-active)
         (call-interactively 'set-mark-command)
         (if (or advance fix-advance-copycut) (end-of-line 1)
                                              (beginning-of-line 1))
         (call-interactively 'kill-region)
         (if (or advance fix-advance-copycut)
                (message "CUT End-of-Line to Paste-Buffer !")
                (message "CUT Start-of-Line to Paste-Buffer !"))
         (setq rectangular-paste-buffer nil))
        ;;
        ((equal mark-active "append")
         (if rectangular-paste-buffer (error
"Append-Cut IMPOSSIBLE:  Paste-Buffer contains a rectangle (SET NEW MARK) !")
         (append-next-kill)
         (call-interactively 'kill-region)
         (setq rectangular-paste-buffer nil)
         (message "APPEND CUT of marked region to Paste-Buffer !")))
        ;;
        ((equal mark-active "column")
         (call-interactively 'kill-rectangle)
         (goto-char (region-beginning))
         (setq rectangular-paste-buffer t)
         (rm-deactivate-mark)
         (message "COLUM-wise CUT of marked rectangle to Paste-Buffer !"))
        ;;
        (t
         (call-interactively 'kill-region)
         (setq rectangular-paste-buffer nil)
         (message "NORMAL CUT of marked region to Paste-Buffer !"))
        ;;
  )
  (setq mark-active nil)        ; Unhighlighting region
  (setq this-command 'nothing)) ; Suppress appended kill-region(s)

(defun insert-paste (arg)
"Insert contents of the Paste-Buffer (unchanged) at actual cursor position.
Depending on preceding MARK/COPY/CUT the following function will be invoked:\n
no mark     - Insert a Start-of-Line or End-of-line from Paste-Buffer.
MARK        - Insert a region of text from the Paste-Buffer at actual position.
MARK APPEND - Insert a collected region of text from the Paste-Buffer.
MARK COL    - Insert a rectangle from the Paste-Buffer column-wise."
      (interactive "p")
      (let ((pos (point)))
       (cond (rectangular-paste-buffer
                (call-interactively 'yank-rectangle)
                (message "COLUM-wise PASTE of rectangle from Paste-Buffer !"))
             (t (call-interactively 'yank)
                (message "NORMAL PASTE from Paste-Buffer !")))
      (goto-char pos)))

(defun append-paste-to-buffer (buffer)
"Append contents of the Paste-Buffer (unchanged) to the specified buffer.
Depending on preceding MARK/COPY/CUT the following function will be invoked:\n
no mark     - Append a Start-of-Line or End-of-line from Paste-Buffer.
MARK        - Append a region of text from the Paste-Buffer at actual position.
MARK APPEND - Append a collected region of text from the Paste-Buffer.
MARK COL    - Append a rectangle from the Paste-Buffer column-wise."
    (interactive "BAppend Paste-Buffer to buffer: ")
    (set-buffer (get-buffer-create buffer))
    (switch-to-buffer-other-window (get-buffer-create buffer))
    (goto-char (point-max))
    (call-interactively 'insert-paste))

(defun append-paste-to-file (file)
"Append contents of the Paste-Buffer (unchanged) to the specified file.
Depending on preceding MARK/COPY/CUT the following function will be invoked:\n
no mark     - Append a Start-of-Line or End-of-line from Paste-Buffer.
MARK        - Append a region of text from the Paste-Buffer at actual position.
MARK APPEND - Append a collected region of text from the Paste-Buffer.
MARK COL    - Append a rectangle from the Paste-Buffer column-wise."
    (interactive "FAppend Paste-Buffer to file: ")
    (set-buffer (generate-new-buffer "*GOLD-Paste-Buffer*"))
    (call-interactively 'insert-paste)
    (write-region (point-min) (point-max) file t nil)
    (kill-buffer "*GOLD-Paste-Buffer*"))

(defun copy-rectangle (start end)
      "Copy rectangle with corners at point and mark; save as last killed one.
Calling from program, supply two args START and END, buffer positions.
But in programs you might prefer to use delete-extract-rectangle."
      (interactive "r")
      (setq killed-rectangle (extract-rectangle start end)))

;;----------------------
(defun kill-a-buffer () "Show all buffers and interactively kill one of them."
      (interactive)
      (list-buffers) (call-interactively 'kill-buffer))

;;****************************************************************************
;; Additional (i)Spell-Check features
;;****************************************************************************
;;(autoload 'auto-ispell-mode "min-ispl"  "Toggles auto-ispell-mode." t)
;;
;;(defun ispell-minor-mode (&optional arg) "Toggles auto-ispell-mode.
;;With prefix arg, turn auto-ispell-mode on iff arg is positive.
;;In auto-ispell-mode, inserting a space or a new line applies ispell-word
;;on the previous word. Pressing TAB activates ispell-complete-word."
;;  (interactive "P")
;;  (if (auto-ispell-mode arg)
;;      (define-key auto-ispell-keymap "\t" 'ispell-complete-word)))
;;
(setq ispell-dictionary "american")

(defun translate-word ()
   "Translating word at cursor position:  English <-> German."
   (interactive)
   (let ((actual-buffer (current-buffer)) (language "english")
         (word (word-at-point))(line " "))
        (if (and (boundp 'ispell:dictionary) ispell:dictionary)
            (setq language ispell:dictionary))
        (translate-dictionary) (goto-char (point-min))
   (cond ((and (equal language "german")
               (re-search-forward (concat " " word "$") nil t))
               (setq line (buffer-linestring))
               (message (concat "German->English:  \"" word "\" -> \""
                 (substring line 0 (string-match " " line)) "\"")))
         ((and (equal language "english")
               (re-search-forward (concat "^" word " ") nil t))
               (setq line (buffer-linestring))
               (message (concat "English->German:  \"" word "\" -> \""
                 (substring line (+ (string-match " " line) 1)) "\"")))
         (t (beep) (message (concat "No translation for \"" word
                   "\" in language \"" language "\" (search failed) !"))))
     (set-buffer actual-buffer)))
(defun translate-dictionary ()
      "Switch to buffer with translate dictionary (create at first call)"
      (if (get-buffer "*TransDict*") (set-buffer "*TransDict*")
          (message "Loading *TransDict*")
          (unconditional-new-buffer "*TransDict*" nil) (advance-direction)
          (insert-file "/doc/share/dict/book.vok")
          (message "Loading *TransDict* Done")
          (set-buffer-modified-p nil)
          (setq buffer-read-only t)))

;;****************************************************************************
;;***** Copying & (Un)Deleting Lines, Words and Characters ! *****************
;;****************************************************************************
(gold-debug "copy/del")
(defun glue-cursor ()
"Cursor will hold the selected line/word during following cursor movements
(transpose line/word), terminated by any non-cursor key sequence."
(interactive) (message "Cursor glues actual line/word !"))

(defun transpose-next/previous-lines (num)
  "Exchange current line and next/previous line(s) depending on actual
ADVANCE/BACKUP mode, leaving point in \"current-line\"."
  (interactive "p") (delete-current-line 1)
  (if advance (next-line num) (previous-line num))
  (undelete-lines) (previous-line 1))

(defun combine-lines () "Append next line to end of current line."
  (interactive) (end-of-line 1) (delete-current-char 1)
  (if (or (equal major-mode 'fortran-mode)
          (equal major-mode 'f77-mode) (equal major-mode 'f7x-mode)
          (equal major-mode 'f7x2f77-mode)) (delete-current-char 6))
  (just-one-space))

(defun multiply-current-line (num)
"Duplicate current line one or more times (according to prefix variable num).
The whole line will be copied (starting with column 1 independent of current
point/cursor position) including the newline character at the end of line."
  (interactive "p")
  (beginning-of-line)
  (let ((beg (point)))
    (forward-line 1) (if (not (eq (preceding-char) ?\n))(insert "\n"))
    (setq end (point))
    (while (< 0 num)
           (insert (buffer-substring beg end))
           (setq num (- num 1)))
    (forward-line -1)))

(defun delete-double-lines ()
   "Delete all double lines directly following its master-line."
   (interactive)
   (perform-replace "^\\(.*\\)\\(\n\\1\\)*$" "\\1" nil t nil))

(defun delete-current-line (num)
      "Delete one or specified number of lines. The whole line(s) will be
deleted (starting with column 1 independent of current point/cursor position).
This includes the newline character at the end of each line.
The line(s) are saved for the EDT undelete-lines command."
  (interactive "p")
  (beginning-of-line)
  (let ((beg (point)))
    (forward-line num)
    (if (not (eq (preceding-char) ?\n))
        (insert "\n"))
    (setq edt-last-deleted-lines
          (buffer-substring beg (point)))
    (delete-region beg (point))))

(defun delete-to-sol (num)
  "Delete text from start of line up to point.
With argument, delete up to Nth start-line past point.
Text is saved for the EDT undelete-lines command."
  (interactive "p")
  (let ((beg (point)))
    (forward-char 1)
    (beginning-of-line num)
    (setq edt-last-deleted-lines
          (buffer-substring beg (point)))
    (delete-region beg (point))))

(defun delete-to-eol (num)
  "Delete text up to end of line.
With argument, delete up to Nth line-end past point.
They are saved for the EDT undelete-lines command."
  (interactive "p")
  (let ((beg (point)))
    (end-of-line num)
    (setq edt-last-deleted-lines
          (buffer-substring beg (point)))
    (delete-region beg (point))))

(defun delete-current-word (num)
  "Delete one or specified number of words after point.
They are saved for the EDT undelete-words command."
  (interactive "p")
  (let ((beg (point)))
    (forward-word num)
    (setq edt-last-deleted-words
          (buffer-substring beg (point)))
    (delete-region beg (point))))

(defun delete-previous-word (num)
  "Delete one or specified number of words before point.
They are saved for the EDT undelete-words command."
  (interactive "p")
  (let ((beg (point)))
    (forward-word (- num))
    (setq edt-last-deleted-words
          (buffer-substring (point) beg))
    (delete-region beg (point))))

(defun delete-current-char (num)
  "Delete one or specified number of characters after point.
They are saved for the EDT undelete-chars command."
  (interactive "p")
  (setq edt-last-deleted-chars
        (buffer-substring (point) (min (point-max) (+ (point) num))))
  (delete-region (point) (min (point-max) (+ (point) num))))

(defun delete-previous-char (num)
  "Delete one or specified number of characters before point.
They are saved for the EDT undelete-chars command."
  (interactive "p")
  (setq edt-last-deleted-chars
        (buffer-substring (max (point-min) (- (point) num)) (point)))
  (delete-region (max (point-min) (- (point) num)) (point)))

(defun undelete-lines ()
  "Yank lines deleted by last GOLD-emulation line-deletion command."
  (interactive)
  (beginning-of-line)
  (insert edt-last-deleted-lines))

(defun undelete-words ()
  "Yank words deleted by last EDT word-deletion command."
  (interactive)
  (insert edt-last-deleted-words))

(defun undelete-chars ()
  "Yank characters deleted by last EDT character-deletion command."
  (interactive)
  (insert edt-last-deleted-chars))

;;****************************************************************************
;;***** Moving and Scrolling ! ***********************************************
;;****************************************************************************
(gold-debug "move/scroll")
(setq scroll-step 1)  ; Line-by-line scrolling when point moves out-of-window
(setq-default truncate-lines t)
(add-hook 'post-command-hook 'horizontal-adjust t)
;;;
(defvar horizontal-adjust-minscroll 10
  "*Minimal number of columns to scroll horizontally on horizontal-adjust.")
;;;
(defun horizontal-adjust ()
"This function checks if the actual point is within the horizontal visibility
of its window area, i.e. if point is beyond the left respectively right border.
If so then the window is scrolled left respectively right horizontally with a
minimum of 'horizontal-adjust-minscroll' columns (but at least so that actual
point position is visible).
Remark: Does nothing if truncate-lines or horizontal-adjust-minscroll is nil !"
  (if (and truncate-lines horizontal-adjust-minscroll)
      (let ((column (- (current-column) (max (window-hscroll) 1))))
           (if (and (< column 0) (> (window-hscroll) 0))
               (scroll-right (max (- column) horizontal-adjust-minscroll))
               (if (>= column (- (window-width) 2))
                   (scroll-left (max (- column (window-width) -3)
                                     horizontal-adjust-minscroll))))
))
;(if (pos-visible-in-window-p (point-max))
;    (let ((savex (point))) (goto-char (point-max)) (recenter -1)
;                           (goto-char savex)))
)
;;
(defun toggle-truncate () "Toggle the truncate-lines variable"
  (interactive)
  (setq truncate-lines (not truncate-lines))
  (if truncate-lines (message "Truncation turned ON (in current buffer)")
                     (message "Truncation turned OFF (in current buffer)"))
  (recenter nil))
;;;; Thanks to Peter Fraenkel

;; Set dynamically growing/shrinking minibuffer
(if (< emacs-major-version 22) (progn
    (resize-minibuffer-mode 1)
    (setq resize-minibuffer-mode t)
    (add-hook 'minibuffer-setup-hook 'truncate-lines-in-minibuffer)
    (defun truncate-lines-in-minibuffer () (setq truncate-lines nil))))

(defvar fix-advance-word t
"*Standard direction for function word-back-forward:
  nil   - Move forward/backward to next word depending on ADVANCE/BACKUP mode.
non-nil - Move always forward to next word (independent of ADVANCE/BACKUP).")

(defun word-back-forward (num)
  "Move forward to next word-beginning respectively backward to next word-end,
depending on the actual ADVANCE/BACKUP mode (see variable fix-advance-word).
Allows a prefix for the Nth following word-beginning respectively -ending."
  (interactive "p")
  (if (or advance fix-advance-word)
         (progn (forward-word (1+ num)) (forward-word -1))
         (forward-word (- (1+ num)))    (forward-word 1)))

(defun scroll-half-down () "Scroll the display down a half-window-full."
  (interactive) (set-temporary-goal-column)
  (scroll-down (/ (window-height) 2))
  (move-to-column (or goal-column temporary-goal-column)))
;
(defun scroll-half-up () "Scroll the display up a half-window-full."
  (interactive) (set-temporary-goal-column)
  (scroll-up (/ (window-height) 2))
  (move-to-column (or goal-column temporary-goal-column)))
;
(defun goto-percent (perc)
  "Move point to ARG percentage of the buffer."
  (interactive "NGoto-percentage: ")
  (if (or (> perc 100) (< perc 0))
      (error "Percentage %d out of range 0 < percent < 100" perc)
      (let ((cent (/ (point-max) 100)))
           (goto-char (* perc cent)))))

(defun beginning-of-window ()
      "Home cursor to top of window."
      (interactive) (move-to-window-line 0))

(defun line-to-bottom-of-window ()
      "Move the current line to the top of the window."
      (interactive) (recenter -1))

(defun line-to-top-of-window ()
      "Move the current line to the top of the window."
      (interactive) (recenter 0))

(defun split-screen (nofwin)
"Splits whole screen into NOFWIN windows (NOFWIN is prefix-arg with default 3).
Any window will display the actually selected buffer with top window selected."
   (interactive "p") (delete-other-windows)
   (let ((nx nofwin)) (if (< nx 2) (setq nx 3))
        (while (< 1 nx) (split-window-vertically
                              (/ (* (- nx 1) (+ (window-height) 1)) nx))
                        (setq nx (- nx 1)))))

;;****************************************************************************
;;***** Other Functions ! ****************************************************
;;****************************************************************************
(gold-debug "more")
;;;
;;; Additional hooks to be run after special interactive commands:
;;;----------------------------------------------------------------
(add-hook 'post-command-hook 'more-post-command-hooks)
(defvar self-insert-hook   nil "Hook to be run after self-insert-command.")
(defvar mode-change-hook   nil "Hook to be run after major-mode changes.")
(defvar buffer-change-hook nil "Hook to be run after selected buffer changes.")
;;;
(defun more-post-command-hooks () "Run additional command-specific hooks:
Runs 'self-insert-hook' after each self-insert-command.
Runs 'mode-change-hook' if actual major-mode changes interactively.
Runs 'buffer-change-hook' if interactively selected buffer changes."
  ;;
  (if (equal this-command 'self-insert-command)
      ;;
      ;; Self-insert does not change mode and/or buffer
      (run-hooks 'self-insert-hook)
      ;;
      ;; Test on mode-changing first (before mode-changing)
      (if mode-change-hook   ; Do nothing if no mode-change-hook active
         (if (equal major-mode last-major-mode) nil
             (run-hooks 'mode-change-hook)
             (setq last-major-mode major-mode)))
      (if buffer-change-hook ; Do nothing if no buffer-change-hook active
          (if (equal last-buffer-name (buffer-name)) nil
              (run-hooks 'buffer-change-hook)
              (setq last-buffer-name (buffer-name))))
))
(defvar last-buffer-name nil "Last 'buffer-name' for more-post-commands-hooks")
(defvar last-major-mode  nil "Last 'major-mode' for more-post-commands-hooks")
;;;------
(add-hook 'find-file-hooks  'advance-direction t)
;;;------
;;
(defun application-keys-on (&optional silent)
  "Switch on Application Keypad and Cursor Keys of vt100/220-Terminal."
  (interactive)
  (let ((do (or (equal TERM "su") (string-match "vt[123][024][05]" TERM))))
       (if do (send-string-to-terminal "\e[?1h\e="))
       (or silent (if do (message "Application Keypad & Cursor active !")
                         (beep) (message
            "Application Keypad&Cursor not supported on active terminal !")))))
;
;;---------------------------------------
(defun set-default-colors ()
  "Set 'good-looking' default colors for background, foreground, mouse etc."
  (interactive)
  (if x-terminal
     (progn (set-background-color "white")
            (set-foreground-color "black")
            (set-cursor-color     "red")
            (set-mouse-color      "red")
            (set-face-background  'modeline "blue")
            (set-face-foreground  'modeline "white"))
     (beep) (message
     "Setting colors is not supported on active terminal !")))
;;
(if x-terminal (progn (set-face-background  'modeline "blue")
                      (set-face-foreground  'modeline "white")))
;;---------------------------------------
(defun what-position () "Shows the current cursor position in buffer.
Prints actual (and maximal) line number as well as column number."
      (interactive)
      (setq inverse-video t)
      (momentary-string-display (concat
       "         1         2         3         4         5"
       "         6         7         8         9        10"
       "        11        12        13        14        15"
       "        16        17        18        19        20"
       "        21        22        23        24        25\n"
       "12345678901234567890123456789012345678901234567890"
       "12345678901234567890123456789012345678901234567890"
       "12345678901234567890123456789012345678901234567890"
       "12345678901234567890123456789012345678901234567890"
       "1234567890123456789012345678901234567890123456789012345\n")
       (save-excursion (beginning-of-line) (point)) nil
        (concat "Line " (int-to-string (buffer-line))
                      " (of " (int-to-string (max-line))
                 "), Column " (int-to-string (+(current-column)1))
                (if (equal major-mode 'doc-mode)
                    (concat " >>> Page " (page-number) " of document"))))
      (setq inverse-video nil))
;;;
(defun buffer-line ()
"Return the vertical position of point in the selected buffer.
Counts each text line only once, even if it warps (starts with 1)."
      (+ (count-lines (point-min) (point))
         (if (= (current-column) 0) 1 0)))

(defun window-line ()
"Return the vertical position of point in the selected window.
Counts each text line only once, even if it warps (starts with 1)."
      (+ (count-lines (window-start) (point))
         (if (= (current-column) 0) 1 0)))

(defun max-line ()
"Return the maximum line number in buffer.
Counts each text line only once, even if it warps (starts with 1)."
      (count-lines (point-min) (point-max)))

(defun region-lines ()
"Return the number of lines in actually selected region.
Counts each text line only once, even if it warps (starts with 1)."
      (count-lines (region-beginning) (region-end)))
;;-------------
(defun keep-dir-filter (process out)
  (if (and (not dir) (stringp out) (string-lessp "/ " out)
           (not (string-match "[<#%>!]" out))) ;; illegal char. in dir-name
      (progn (setq dir (substring out 0 -1)) (cd dir))))

(defun UNDO () "Just calls major-mode-dependent undo function." (interactive)
  (cond ((equal major-mode 'calc-mode) (call-interactively 'calc-undo))
        (t (call-interactively 'undo))))

(defun redraw-display-h ()
  "Just redraw-display (and application-keys on)."
  (interactive)
  (redraw-display) (application-keys-on 'silent)
  (if (equal major-mode 'shell-mode) (call-interactively 'dirs)))
;
(defun switch-buffer ()
      "Show all buffers and interactively switch to other buffer."
      (interactive) (list-buffers)
      (call-interactively 'switch-to-buffer-other-window))
;;
(defun next-buffer ()
  "Put current buffer to end of buffer list and switch to next file buffer."
  (interactive)
  (let ((old (buffer-name))
        (new nil))
       (bury-buffer old)
       (switch-to-buffer (other-buffer old))
       (setq new (buffer-name))
       (if (not (equal new old)) ; Avoid recursion if only 1 buffer
           (if (equal "*" (substring new 0 1)) (next-buffer)))))
;;
(defun main-buffer () "Make current buffer fill the whole window frame or
switch to next buffer if current buffer is the only one visible."
   (interactive)
   (delete-other-windows)
;; (if (one-window-p t 1) (next-buffer) (delete-other-windows))
)

(defun scratch-buffer () "Switch to *scratch* buffer (re-create if not existing)."
 (interactive) (switch-to-buffer "*scratch*"))

;;-----------------------
(defun archie-start (host name)
  "Open a network login connection to archie host named HOST (a string).
The telnet communication with HOST is recorded in a buffer *archie-NAME*.
Normally input is edited in Emacs and sent a line at a time."
  (if (not (boundp 'telnet)) (load "telnet")) (require 'shell)
  (let ((NAME (concat "archie-" name)))
      (message (concat "Starting telnet session for \"" NAME
                       "\" (use \"archie\" for login)"))
      (switch-to-buffer (make-comint NAME "telnet"))
      (set-process-filter (get-process NAME) 'telnet-initial-filter)
      ;; Don't send the `open' cmd till telnet is ready for it.
      (accept-process-output (get-process NAME))
      (erase-buffer)
      (send-string  (concat "*" NAME "*") (concat "open " host "\n"))
      (telnet-mode)
      (setq comint-input-sender 'telnet-simple-send)
      (setq telnet-count telnet-initial-count)))
;
(defun archie () "Telnet session for default archie (Darmstadt) !"
  (interactive) (archie-start "archie.th-darmstadt.de" "Darmstadt"))
(defun archie-Darmstadt () "Telnet session for Darmstadt archie !"
  (interactive) (archie-start "archie.th-darmstadt.de" "Darmstadt"))
(defun archie-London () "Telnet session for London archie !"
  (interactive) (archie-start "146.169.2.10" "London"))
(defun archie-Helsinki () "Telnet session for Helsinki archie !"
  (interactive) (archie-start "128.240.6.100" "Helsinki"))
(defun archie-NewYork () "Telnet session for New-York archie !"
  (interactive) (archie-start "147.225.1.10" "NewYork"))
(defun archie-Montreal () "Telnet session for Canadian archie !"
  (interactive) (archie-start "132.206.2.3" "Montreal"))
;;
;; [Remote] Login-Shells:
;;------------------------
(defun login (command &optional buffer-name)
  "Remote login shell"
  (interactive "sMachine: ")
  (if buffer-name (rlogin command (concat "*" buffer-name "*"))
                  (rlogin command (concat "*" command "*")))
  (shell-mode))
;;
(defun bertold () (interactive) (login "bertold" ))
(defun dec-1   () (interactive) (login "dec-1" ))
(defun dec-2   () (interactive) (login "dec-2" ))
(defun dec-3   () (interactive) (login "dec-3" ))
(defun ebi     () (interactive) (login "ebi"   ))
(defun hans    () (interactive) (login "hans"  ))
(defun hardy   () (interactive) (login "hardy" ))
(defun hp-1    () (interactive) (login "hp-1"  ))
(defun hp-2    () (interactive) (login "hp-2"  ))
(defun hp-3    () (interactive) (login "hp-3"  ))
(defun hp-4    () (interactive) (login "hp-4"  ))
(defun hp-5    () (interactive) (login "hp-5"  ))
(defun ibm-1   () (interactive) (login "ibm-1" ))
(defun ibm-2   () (interactive) (login "ibm-2" ))
(defun ibm-3   () (interactive) (login "ibm-3" ))
(defun markus  () (interactive) (login "markus"))
(defun mathias () (interactive) (login "mathias"))
(defun netserv () (interactive) (login "netserv"))
(defun rainer  () (interactive) (login "rainer"))
(defun rolf    () (interactive) (login "rolf"  ))
(defun sgi-1   () (interactive) (login "sgi-1" ))
(defun sgi-2   () (interactive) (login "sgi-2" ))
(defun sgi-3   () (interactive) (login "sgi-3" ))
(defun sgi-4   () (interactive) (login "sgi-4" ))
(defun sgi-5   () (interactive) (login "sgi-5" ))
(defun sgi-l   () (interactive) (login "sgi-l" ))
(defun sun-1   () (interactive) (login "sun-1" ))
(defun sun-2   () (interactive) (login "sun-2" ))
(defun sun-3   () (interactive) (login "sun-3" ))
(defun www     () (interactive) (login "www" ))
;;-----------------------------------------------------------------------------

;; Developers function:
;; ---------------------
(defun Lisp-byte-compile (file)
 "Byte compiling lisp source file on disk (default is file of current buffer)."
      (interactive "fLisp byte compiling of file: ")
      (byte-compile-file file))

;;-----------------------------------------------------------------------------
(defvar fix-advance-caseflip t
"*Standard direction for function case-flip-left-right:
  nil   - Case-flip forward/backward depending on ADVANCE/BACKUP mode.
non-nil - Case-flip always forward (independent of ADVANCE/BACKUP).")

(defun case-flip-left-right (num)
  "Change the case of the character under the cursor and move forward/backward
depending on the actual ADVANCE/BACKUP mode.
Accepts a prefix argument of the number of characters to be changed."
  (interactive "p")
  (if (or advance fix-advance-caseflip)
     (while (> num 0) (funcall (if (<= ?a (following-char))
                       'upcase-region 'downcase-region)
                      (point) (1+ (point))) (forward-char 1)
                      (setq num (1- num)))
     (while (> num 0) (funcall (if (<= ?a (following-char))
                        'upcase-region 'downcase-region)
                             (point) (1+ (point))) (backward-char 1)
                      (setq num (1- num)))))

(defun new-line (num)
   "Create a new line. With arg, create that many new lines."
   (interactive "p") (end-of-line 1) (newline num))

(defun reset () ;;??"Keyboard-quit (escape from command line)
"Redraw sreen plus additional directory resynchrinization if in shell-mode."
       (interactive)
       (if (equal major-mode 'shell-mode) (call-interactively 'dirs))
       (redraw-display) (setq rectangular-paste-buffer nil)
       (keyboard-quit))

(defvar dir-title-file ".dir_title" "*Name of directory title-file(s).")
(defun edit-dir-title-file ()
  "Edit the 'dir-title-file' of the actual default-directory."
  (interactive)
  (let*((new (not (file-exists-p dir-title-file))))
       (find-file dir-title-file)
       (if new (insert "----> ")
               (goto-char (point-min)) (forward-char 6))))

(defun indent-or-fill-region ()
  "Fill region in text modes, indent region in programming language modes."
  (interactive)
  (if (string= paragraph-start "^$\\|^")
      (indent-region (point) (mark) nil)
    (fill-region (point) (mark))))

;;****************************************************************************
;;***** GOLD-Tabulator-Handling **********************************************
;;****************************************************************************
(gold-debug "tabulator")
(setq-default indent-tabs-mode nil)
(defvar fix-advance-tab t
"*Standard direction of tabulator movements through tab-left-right:
  nil   - Direction depends on general ADVANCE/BACKUP mode.
non-nil - Fixed to advance direction (independent of ADVANCE/BACKUP).")

(defun tab-left-right ()
"Move Cursor back or forward to the next defined tab stop column depending on
the actual ADVANCE/BACKUP mode (see also variable fix-advance-tab).
The variable tab-stop-list is a list of columns at which there are tab stops.
Use GOLD-TAB to edit them interactively."
  (interactive)
  (if abbrev-mode (expand-abbrev))
  (if (or advance fix-advance-tab)
     (let ((tabs tab-stop-list))
          (while (and tabs (>= (current-column) (car tabs)))
                 (setq tabs (cdr tabs)))
          (if tabs
             (move-or-extend (- (car tabs) (current-column)))
             (if (= ?\n (following-char)) (forward-char 1) (end-of-line 1))))
     (let ((tabs (reverse tab-stop-list)))
          (while (and tabs (<= (current-column) (car tabs)))
                 (setq tabs (cdr tabs)))
          (if tabs
             (backward-char (- (current-column) (car tabs)))
             (if (= ?\n (preceding-char)) (backward-char 1) (beginning-of-line 1))
     ))))

(defun move-or-extend (num)
      "Move cursor one column to the right, insert blanks at line-end."
      (interactive "p")
      (while (> num 0)
             (if (or (= ?\n (following-char)) (= 0 (following-char)))
                 (insert ? )
                 (forward-char 1))
             (setq num (- num 1))))

(defun context-indent () "Update indent of actual context (mode-dependent)."
   (interactive)
   (cond ((equal major-mode 'fortran-mode) (fortran-indent-subprogram))
         (t (tab-left-right))
   ))
;;------------------------------------------------------------------------
(setq tab-stop-list ;; Internal column counter starts with 0
  '(6 9 12 15 18 21 24 27 30 33 36 39 44 49 54 59 64 69 70 71))

(define-key edit-tab-stops-map "\C-m" 'edit-tab-stops-note-changes)
(define-key edit-tab-stops-map "\eOM" 'edit-tab-stops-note-changes)
(define-key edit-tab-stops-map "\eOn" 'leave-edit-tab-stops)
(define-key edit-tab-stops-map "\t"   'tab-edit-tab-stops)

(defun tab-stops-line ()
  "Create a one line string showing the current tab stops (represented by ':')."
  (let ((tabs tab-stop-list))
    (while tabs
      (indent-to (car tabs) 0)
      (insert ?:)
      (setq tabs (cdr tabs)))))

(defun columns-line ()
  "Create two numbered line strings showing the columns."
  (let ((count 10))
    (insert ?\n)
    (while (> count 0)
      (insert "1234567890")
      (setq count (1- count)))
    (insert ?\n)
    (while (< count 10)
      (setq count (1+ count))
      (insert "         ")
      (insert (+ count ?0)))))

(gold-debug "edit-tab")
(defun edit-tab-stops ()
  "Edit the tab stops used by tab-to-tab-stop.
Creates a buffer *Tab Stops* containing text describing the tab stops.
A colon indicates a column where there is a tab stop.
You can add or remove colons and type RETURN to make changes take effect."
  (interactive)
  (setq edit-tab-stops-buffer (current-buffer))
  (switch-to-buffer-other-window (get-buffer-create "*Tab Stops*"))
  (shrink-window 4)
  (use-local-map edit-tab-stops-map)
  (make-local-variable 'indent-tabs-mode)
  (setq indent-tabs-mode nil)
  (overwrite-mode 1) (setq truncate-lines t)
  (erase-buffer)     (tab-stops-line)
  (columns-line)
(insert "\n\n           Type RETURN to install changes  (RESETscreen to quit)")
(insert   "\n          ------------------------------------------------------")
  (insert   "\nRemember: Some File-Modes will overwrite the TAB-function !")
  (goto-char (point-min)))

(defun leave-edit-tab-stops ()
  "Exit from edited tab stops without changes."
  (interactive)
    (let (tabs) (save-excursion
        (goto-char 1) (end-of-line)
        (while (search-backward ":" nil t)
          (setq tabs (cons (current-column) tabs))))
      (bury-buffer (prog1 (current-buffer)
                          (switch-to-buffer edit-tab-stops-buffer))))
  (delete-window) (message "Quit"))

(defun edit-tab-stops-note-changes ()
  "Put edited tab stops into effect."
  (interactive)
    (let (tabs) (save-excursion
        (goto-char 1) (end-of-line)
        (while (search-backward ":" nil t)
          (setq tabs (cons (current-column) tabs))))
      (bury-buffer (prog1 (current-buffer)
                          (switch-to-buffer edit-tab-stops-buffer)))
      (setq tab-stop-list tabs))
  (delete-window) (message "New tab stops installed"))

(defun tab-edit-tab-stops ()
      "Tabulator within edit-tab-stops."
      (interactive)
      (if (or advance fix-advance-tab)
         (progn (forward-char 1) (search-forward  ":" nil t) (backward-char 1))
         (backward-char 1) (search-backward ":" nil t) (forward-char  1)))


;;****************************************************************************
;;***** Use ".bak" (".bak123") for Backup-Files, set APOLLO-ACLs properly ! **
;;****************************************************************************
(gold-debug "backup")
; Try copying for backup files if renaming would change user or group:
(setq backup-by-copying-when-mismatch t)

;;
(defun save-buffer (&optional args)
  "Save current buffer in visited file if modified.  Versions described below.
By default, makes the previous version into a backup file
 if previously requested or if this is the first save.
With 1 or 3 \\[universal-argument]'s, marks this version
 to become a backup when the next save is done.
With 2 or 3 \\[universal-argument]'s,
 unconditionally makes the previous version into a backup file.
With argument of 0, never makes the previous version into a backup file.

If a file's name is FOO, the names of its numbered backup versions are
 FOO.~i~ for various integers i.  A non-numbered backup file is called FOO~.
Numeric backups (rather than FOO~) will be made if value of
 `version-control' is not the atom `never' and either there are already
 numeric versions of the file being backed up, or `version-control' is
 non-nil.
We don't want excessive versions piling up, so there are variables
 `kept-old-versions', which tells Emacs how many oldest versions to keep,
 and `kept-new-versions', which tells how many newest versions to keep.
 Defaults are 2 old versions and 2 new.
`dired-kept-versions' controls dired's clean-directory (.) command.
If `delete-old-versions' is nil, system will query user
 before trimming versions.  Otherwise it does it silently."
  (interactive "p")
  (let ((modp (buffer-modified-p))
        (large (> (buffer-size) 50000))
        (make-backup-files (or (and make-backup-files (not (eq args 0)))
                               (memq args '(16 64)))))
    (if modp (if (or force-backup (memq args '(16 64)))
                 (setq buffer-backed-up nil)))
    (if (and modp large) (message "Saving file %s..." (buffer-file-name)))
    (basic-save-buffer)
    (and modp (memq args '(4 64)) (setq buffer-backed-up nil))))

;; Backup-files:
;;---------------
(defvar backup-file-dir (getenv "BACKUPDIR")
 "*Directory where backup-files should go to (default is current directory).")
(if backup-file-dir
   (or (file-directory-p backup-file-dir) (setq backup-file-dir nil)))
;;
(defvar force-backup t
  "*Non-nil forces `save-buffer' to write a backup (not only at first save).")
;;
(defun make-backup-file-name (file)
  "Create the non-numeric backup file name for FILE.
 This is a separate function so you can redefine it for customization."
   (if (and backup-file-dir
           (file-writable-p (directory-file-name backup-file-dir)))
      (expand-file-name (concat backup-file-dir
                               (file-name-nondirectory file) ".bak"))
      (concat file ".bak")))
;;
(defun backup-file-name-p (file)
  "Return non-nil if FILE is a backup file name (numeric or not).
This is a separate function so you can redefine it for customization.
You may need to redefine file-name-sans-versions as well."
  (string-match ".bak$" file))
;;
;; For numbered backup (version-control t):
(setq kept-old-versions 1)  ; Number of oldest versions to keep
(setq kept-new-versions 3)  ; Number of newest versions to keep
(setq delete-old-versions t); Delete excess backup versions silently (NEW)
(setq trim-versions-without-asking t) ; Delete ......................(OLD)
;;
(defun find-backup-file-name (fn)
  "Find a file name for a backup file, and suggestions for deletions.
Value is a list whose car is the name for the backup file
 and whose cdr is a list of old versions to consider deleting now."
  (if (eq version-control 'never)
      (list (make-backup-file-name fn))
    (let* ((base-versions (concat (file-name-nondirectory fn) ".bak"))
           (bv-length (length base-versions))
           (possibilities (append
                           (file-name-all-completions base-versions
                            (file-name-directory fn))
                          (if backup-file-dir
                           (file-name-all-completions base-versions
                            backup-file-dir))))
           (versions (sort (mapcar
                            (function backup-extract-version)
                            possibilities)
                           '<))
           (high-water-mark (apply 'max 0 versions))
           (deserve-versions-p
            (or version-control
                (> high-water-mark 0)))
           (number-to-delete (- (length versions)
                                kept-old-versions kept-new-versions -1)))
      (if (not deserve-versions-p)
          (list (make-backup-file-name fn))
        (setq fn (make-backup-file-name fn))
        (cons (concat fn (int-to-string (1+ high-water-mark)))
              (if (and (> number-to-delete 0)
                       ;; Delete nothing if there is overflow
                       ;; in the number of versions to keep.
                       (>= (+ kept-new-versions kept-old-versions -1) 0))
                  (mapcar (function (lambda (n)
                                            (concat fn (int-to-string n))))
                          (let ((v (nthcdr kept-old-versions versions)))
                            (rplacd (nthcdr (1- number-to-delete) v) ())
                            v))))))))

(defun backup-extract-version (fn)
  (if (and (string-match "[0-9]+$" fn bv-length)
           (= (match-beginning 0) bv-length))
      (string-to-number (substring fn bv-length))
      0))

;; Auto-save files:
;;------------------
(defvar auto-save-dir backup-file-dir
 "*Directory where auto-save-files should go to (default is current directoy).")
;;
(if auto-save-dir (setq auto-save-list-file-name
   (concat (expand-file-name auto-save-dir) "AUTOSAVE.list")))
;;
(defun make-auto-save-file-name ()
  "Return file name to use for auto-saves of current buffer.
Does not consider `auto-save-visited-file-name' as that variable is checked
before calling this function.  You can redefine this for customization.
See also `auto-save-file-name-p'."
  (if buffer-file-name
     (format "%s#%s#" (or auto-save-dir
                         (file-name-directory buffer-file-name))
                      (file-name-nondirectory buffer-file-name))
    ;; For non-file bfr, use bfr name and Emacs pid.
    (expand-file-name (concat auto-save-dir
                      (format "#%s#%s#" (buffer-name) (make-temp-name ""))))))
;;
(defun auto-save-file-name-p (filename)
  "Return non-nil if FILENAME can be yielded by `make-auto-save-file-name'.
FILENAME should lack slashes.  You can redefine this for customization."
  (string-match "^#.*#$" filename))

;; Access log protocols (doc):
;;----------------------------
(gold-debug "log-hooks")
(add-hook 'write-file-hooks 'access-log-w t)
(add-hook 'find-file-hooks  'access-log-r t)
;;
(defun access-log-w () "write-file-hook to log modification access."
  (access-log "files/.log/w-access"))
;;
(defun access-log-r () "find-file-hook to log read access."
  (access-log "files/.log/r-access"))
;;
(defun access-log (logfile) " "
  (let ((file buffer-file-name) (path "") rest log) (setq rest file)
 ;;??          (string-match "^\\(/[^@/]+/\\)[^:/]+/" file)
       (while (and rest (string-match "^\\(/?[^/]*/\\)" rest))
              (setq path (concat path (match-string 0 rest)))
              (setq log (concat path logfile))
              (if (file-executable-p log)
                  (let ((user (concat user-login-name "@" system-name)))
                       (call-process log nil 0 nil user file)
                       (setq rest nil))
                  (setq rest (substring rest (match-end 0)))
))))

;; Always accept (y or n): No need for long (yes or no) !
;;--------------------------------------------------------
(fset 'yes-or-no-p (symbol-function 'y-or-n-p))

;; Convert XXX-to-unix
;;---------------------
(defun save-buffer-literally ()
   "Save current buffer  with no conversion of any kind.
Format conversion and character code conversion are both disabled,
and multibyte characters are disabled in the resulting buffer.
I.e. usefull for converting DOS/MAC-Files to ASCII unix."
   (interactive)
   (make-local-variable 'coding-system-for-write)
   (setq coding-system-for-write 'raw-text-unix)
   (set-buffer-modified-p t) (save-buffer))

;;----------------------------------------------------------------------------
(provide 'gold_more)

