;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  GOLD PRINT MENU (usually the first file for company-modifications)  ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_print")
;;
(defun prt-region (start end)
  "Print REGION contents as with an INTES print command."
  (interactive "r")
  (let ((type "REGION") (of " of "))
       (if (region-equal-buffer start end) (message
        "Illegal region (whole buffer selected) -> print request terminated !")
       (prt-menu))))
;;
(defun prt-buffer ()
  "Print BUFFER contents as with an INTES print command."
  (interactive)
  (let ((type "BUFFER") (start (point-min)) (end (point-max)) (of "    "))
       (prt-menu)))
;
(defun prt-menu () "Auxiliary function for prt-region/buffer !"
  (let ((lines (count-lines start end)))
       (save-window-excursion (prt-help) (prt-execute))
       (kill-buffer "*Print-Menu*")))
;;
(setq print-executable (find-executable-file "print"))
(setq a2ps-executable (find-executable-file "a2ps"))
(setq lp-executable (find-executable-file "lp"))
;;;
(defvar default-print-command (or a2ps-executable lp-executable "lpr")
  "*Default print command used if the INTES print commands are not available.")

;;;
(defun prt-help () "Auxiliary function for prt-region/buffer !"
  (let ((pag52 (format "%5d" (/ (+ lines 51) 52)))
        (pag66 (format "%5d" (/ (+ lines 65) 66)))
        (pa105 (format "%5d" (/ (+ lines 104) 105)))
        (pa115 (format "%5d" (/ (+ lines 114) 115)))
        (pa132 (format "%5d" (/ (+ lines 131) 132))))
       (with-output-to-temp-buffer "*Print-Menu*"  ; Not *Help* since then
        (princ (concat "Print "                    ; no help could be printed !
                        type (substring (concat of " '" (buffer-name)
                                   "'                          ") 0 26)
                                     " Columns/Line  Lines/Page   Pages
  4  (print -4) - A4 binding (4 pages/sheet) 80 cut    2*66    ->" pa132"
  5  (print -5) - A5 binding (4 pages/sheet) 80 cut    2*66    ->" pa132"
  N  (print -n) - Normal width (80 columns)  80 cut      66    ->" pag66"
  W  (print -w) - Wide lines (120 columns)  120 cut     105    ->" pa105"
  X  (print -x) - Extra wide (132 columns)  132 wrap    115    ->" pa115"
  L  (print -q) - Landscape printing (Quer) 132 wrap     52    ->" pag52"
  P  (laser)    - Print " type " as an Adobe PostScript file (Non-ASCII)."
 )))))
;;
(defun prt-execute () "Auxiliary function for prt-region/buffer !"
  (if print-executable
  (let ((key (read-key-sequence
             (concat "Print " type " (" (int-to-string lines)
                     " lines) -> Select option:"))))
  (cond               ((equal key "") (message " "))
                      ((equal key "4")  (shell-cmd-region "print -4n" start end t))
                      ((equal key "5")  (shell-cmd-region "print -5n" start end t))
  ((or (equal key "n") (equal key "N")) (shell-cmd-region "print -n" start end t))
  ((or (equal key "w") (equal key "W")) (shell-cmd-region "print -w" start end t))
  ((or (equal key "x") (equal key "X")) (shell-cmd-region "print -x" start end t))
  ((or (equal key "l") (equal key "L")
       (equal key "q") (equal key "Q")) (shell-cmd-region "print -q" start end t))
  ((or (equal key "p") (equal key "P")) (shell-cmd-region "print -g ps" start end t))
  (t (message "Illegal selection -> print request terminated !"))))
  (shell-cmd-region default-print-command start end)))
;;----------------------------------------------------------------------------
(provide 'gold_print)
