;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Additional/modified functions and customizations for RMAIL !        ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_rmail")
;;----------------------------------------------------------------------------
(require 'rmail)
;(load "rmailgen" t)
;;----------------------------------------------------------------------------
(if (load "etach" t t) (progn
;;
    (global-set-key "\C-c\C-f" 'find-file-at-point)
    (global-set-key "\C-c\C-u" 'browse-url-at-point)
    (define-key rmail-mode-map "F" 'mime-forward)
    (setq etach-prompt-me-for-file-names t)
    (setq etach-restore-buffer-after-detach t)
    (or (functionp 'base64-decode-region)
        (setq etach-use-mimencode t))
;;  etach needs to see content-type headers (hidden in emacs-21)
    (setq rmail-ignored-headers (replace-regexp-in-string
         "\\\\|\\^content-[^:]+:" "" rmail-ignored-headers))
))
;;----------------------------------------------------------------------------
(defun detach-extract ()
  "Detach & extract the encoded attachments from the present email message."
  (interactive)
  (let ((etach-restore-buffer-after-detach nil)) (detach)))
;;----------------------------------------------------------------------------
(add-hook 'rmail-mode-hook 'add2rmail-mode)
(defun add2rmail-mode () "Additions to rmail-mode"
   (define-key rmail-mode-map "o"    'Rmail-output-to-rmail-file)
   (define-key rmail-mode-map "R"    'Rmail-reply-to-sender-only)
   (define-key rmail-mode-map "\e\C-a" 'rmail-previous-submail)
   (define-key rmail-mode-map "\e\C-e" 'rmail-next-submail)
   (alt-umlaut-on))
;;----------------------------------------------------------------------------
(defvar rmail-pop-host "mail.local"    "*Host name of POP3 server.")
(defvar rmail-pop-user user-login-name "*User name for POP3 server.")
;;
(defun rmail-from-pop (&optional pop-host pop-user pop-pass)
  "Read mail from pop server into current rmail file."
 (interactive)
 (let* ((save-host  (getenv "MAILHOST"))
        (save-inbox rmail-primary-inbox-list)
        (save-pass rmail-pop-password)
    (host (or pop-host rmail-pop-host (getenv "MAILHOST") (read-string "Host: ")))
    (user (or pop-user rmail-pop-user user-login-name (read-string "User: ")))
    (pass (or pop-pass rmail-pop-password (read-passwd "Password: ")))
    (rmail-pop-password-required (not (null pass))))
 ;;
 (setq rmail-primary-inbox-list (list (concat "po:" user)))
 (if pass (rmail-set-pop-password pass))
 (setenv "MAILHOST" host) (rmail)
 (setq rmail-primary-inbox-list save-inbox)
 (if pass (setq rmail-pop-password save-pass))
 (setenv "MAILHOST" save-host)))
;;----------------------------------------------------------------------------
(defun rmail-previous-submail () "Move to start of current submail"
  (interactive) (forward-line -2)
  (if (re-search-backward "^subject:\\|^--part-boundary=" nil "t")
      (progn (beginning-of-line)
             (if (looking-at "^subject:") (forward-line -2))
             (line-to-top-of-window)
             (if (looking-at "^--part-boundary=") (forward-line 1))
)))
;;
(defun rmail-next-submail () "Move to start of next submail"
  (interactive) (forward-line +3)
  (if (re-search-forward "^subject:\\|^--part-boundary=" nil "t")
      (progn (beginning-of-line)
             (if (looking-at "^subject:") (forward-line -2))
             (line-to-top-of-window)
             (if (looking-at "^--part-boundary=") (forward-line 1))
)))
;
;;----------------------------------------------------------------------------
(setq rmailgen-default-directory "~/.mail/")
(setq rmail-default-directory    "~/.mail/")    ; Dirname must end in slash
(setq rmail-file-name            "~/.mail/INBOX")  ; New mails are placed here
(or (file-exists-p  (directory-file-name rmail-default-directory))
    (make-directory rmail-default-directory))
;;----------------------------------------------------------------------------
(setq rmailgen-gzip-file-name t)
;;----------------------------------------------------------------------------
;e23:(load "/doc/share/files/rmail-alist" t t t)
;e23:(setq rmail-output-file-alist (append (list
;e23:  '("^\\(To\\|From\\):[ \t]*.*gnats@.*intes" . "~/.mail/intes/gnats.gz")
;e23:  '("^\\(To\\|From\\):[ \t]*.*bugs@.*intes"  . "~/.mail/intes/gnats.gz")
;e23:  '("^>Submitter-ID: *\\(.*\\)" . (concat "~/.mail/customer/"
;e23:                                       (downcase (match-string 1)) ".gz"))
;e23:  '("^\\(To\\|From\\):[ \t]*\\([^ ]*\\)@.*intes" . (concat "~/.mail/intes/"
;e23:                                                  (match-string 2) ".gz"))
;e23:  ) rmail-output-file-alist))
;;----------------------------------------------------------------------------
(defun Rmail-output-to-rmail-file ()
  "Just like `rmail-output-to-rmail-file' but ask for additional
rmail-savings based on the contents of `rmail-default-rmail-file'
from 1st call to `rmail-output-to-rmail-file'"
  (interactive)
  (call-interactively 'rmail-output-to-rmail-file)
  (if (file-readable-p "~cusdata/customer/mail")
      (let ((rmail-default-rmail-file rmail-default-rmail-file))
           (if (string-match "gnats/customer/mail/" rmail-default-rmail-file)
               (let ((file (concat rmail-default-directory "customer/"
                     (substring rmail-default-rmail-file (match-end 0)))))
                    (if (y-or-n-p (concat "Save to " file " also "))
                        (rmail-output-to-rmail-file file)))
               (if (string-match "/customer/" rmail-default-rmail-file)
                   (let ((file (concat "~cusdata/customer/mail/"
                         (substring rmail-default-rmail-file (match-end 0)))))
                        (if (y-or-n-p (concat "Save to " file " also "))
                            (rmail-output-to-rmail-file file)))

)))))
;;
(defun Rmail-reply-to-sender-only ()
  "Like `rmail-reply' but ignore CC:others, i.e. reply just to sender"
  (interactive) (rmail-reply 'just-sender))
;;
;;----------------------------------------------------------------------------
(defun rmail-mime-decode () ""
   (interactive) (require 'mime)
   (let ((start (point-min)) (end (point-max))
         (boundary "^--[ -=_]*part")
;;Content-Type: multipart/mixed; boundary="---- =_NextPart_000_01BC8886
         (buffer "*mime*") (encoding "quoted-printable"))
        (if (re-search-backward boundary nil t)
            (setq start (match-beginning 0)))
        (goto-char (match-end 0))
        (if (re-search-forward boundary nil t)
            (setq end (- (match-beginning 0) 2)))
        (goto-char start)
        (if (re-search-forward "^content-type.*=\"\\([^ \n]+\\)\"" end t)
            (setq buffer (match-string 1)))
        (if (re-search-forward "^content-transfer-encoding: *\\([^ \n]+\\)" end t)
            (setq encoding (match-string 1)))
        (if (re-search-forward "^$" end t)
            (setq start (+ (match-end 0) 1)))
        (copy-region-as-kill start end)
        (append-paste-to-buffer buffer)
        (mime-decode-region encoding (point-min) (point-max))
))
;;----------------------------------------------------------------------------
(defun rmail2mbox () "Convert current RMAIL-File into a standard Mail file"
   (interactive)
   (let ((rmail-file (buffer-file-name)) (modified (buffer-modified-p))
         (mbox-file "~/Mail/RMAIL") name)
        (setq name (file-name-nondirectory rmail-file))
        (setq name (replace-regexp-in-string "\.gz" "" name))
        (setq mbox-file (concat mbox-file "_" name))
        (unrmail rmail-file mbox-file)
        (set-buffer-modified-p modified)))
;;----------------------------------------------------------------------------
(provide 'gold_rmail)
