;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Additional/modified SEND-PR functions for GOLD-GNATS !              ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_sendpr")
(require 'gnats)
;;----------------------------------------------------------------------------
(defun send-pr-mode () "GOLD Additions/Modifications to send-pr-mode."
  (interactive) (send-pr:send-pr-mode)
  (setq mode-name "send-pr") (setq major-mode 'send-pr-mode)
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(gnats-edit-font-lock-keywords t t))
  (setq context-update-function 'gnats:change-field)
  (setq compile-buffer-function 'mail-send)
  (local-set-key "\C-c\C-c"     'mail-send)
  (local-set-key "\M-\C-a"      'gnats:previous-field)
  (local-set-key "\M-\C-e"      'gnats:next-field)
  (run-hooks 'send-pr-mode-hook)
)
(defvar send-pr-mode-hook (copy-sequence text-mode-hook))

;;----------------------------------------------------------------------------
;; Extract from old send-pr (with some adaptions) !!!!!!!!!!!!!!!!!!!!!!!!!!!!
;;----------------------------------------------------------------------------
(autoload 'server-buffer-done "server")
(defvar server-buffer-clients nil)
(defvar mail-self-blind nil)
(defvar mail-default-reply-to nil)

(defvar gnats:root "/opt/gnats/gnats-db"
  "*The top of the tree containing the GNATS database.")

(defconst send-pr::fields
  (` (("Category" (("permas") ("felix") ("perm4") ("permit")
                              ("pre-post") ("gnats") ("emacs"))
                   (, nil) enum) ;; Default: (, 0) enum)
      ("Class"    (("sw-bug") ("doc-bug") ("change-request") ("support"))
                   (, nil) enum) ;; Default: (, 0) enum)
      ("Confidential" (("yes") ("no"))
                   (, nil) enum) ;; Default: (, 1) enum)
      ("Severity" (("non-critical") ("serious") ("critical"))
                   (, nil) enum) ;; Default: (, 1) enum)
      ("Priority" (("low") ("medium") ("high"))
                   (, nil) enum) ;; Default: (, 1) enum)
      ("Release"  (("PERMAS 8.00"))
                   (, 0) text)
      ("Submitter-Id" (("INTES"))
                   (, 0) text)
      ("Synopsis" nil nil text
       (lambda (a b c) (gnats::set-mail-field "Subject" c)))))
  "AList, keyed on the name of the field, of:
1) The field name.
2) The list of completions.  This can be a list, a function to call, or nil.
3) The default value.
4) The type of the field.
5) A sub-function to call when changed.")

(defvar gnats::fields nil)

(defmacro gnats::push (i l)
  (` (setq (, l) (cons (,@ (list i l))))))

(defconst send-pr::pr-buffer-name "*send-pr*"
  "Name of the temporary buffer, where the problem report gets composed.")

(defconst send-pr::err-buffer-name "*send-pr-error*"
  "Name of the temporary buffer, where send-pr error messages appear.")

(defvar send-pr:::err-buffer nil
  "The error buffer used by the current PR buffer.")

(defvar send-pr:::spawn-to-send nil
  "Whether or not send-pr-mode should spawn a send-pr process to send the PR.")

(defconst gnats::indent 16 "Indent for formatting the value.")

;;;;---------------------------------------------------------------------------
;;;; send-pr:send-pr-mode mode
;;;;---------------------------------------------------------------------------
(defvar send-pr-mode-map
  (let ((map (make-sparse-keymap)))
    (define-key map "\C-c\C-c" 'mail-send)
    (define-key map "\C-c\C-f" 'gnats:change-field)
    (define-key map "\M-n" 'gnats:next-field)
    (define-key map "\M-p" 'gnats:previous-field)
    (define-key map "\C-\M-f" 'gnats:forward-field)
    (define-key map "\C-\M-b" 'gnats:backward-field)
    map)
  "Keymap for send-pr mode.")

(defconst gnats::keyword "^>\\([-a-zA-Z]+\\):")
(defconst gnats::before-keyword "[ \t\n\f]*[\n\f]+>\\([-a-zA-Z]+\\):")
(defconst gnats::after-keyword "^>\\([-a-zA-Z]+\\):[ \t\n\f]+")

(defun send-pr:send-pr-mode ()
  "Major mode for submitting problem reports.
For information about the form see gnats(1) and send-pr(1).
Special commands: \\{send-pr-mode-map}
Turning on send-pr-mode calls the value of the variable send-pr-mode-hook,
if it is not nil."
  (interactive)
  (gnats::patch-exec-path)
  (put 'send-pr:send-pr-mode 'mode-class 'special)
  (kill-all-local-variables)
  (setq major-mode 'send-pr:send-pr-mode)
  (setq mode-name "send-pr")
  (use-local-map send-pr-mode-map)
  (set-syntax-table text-mode-syntax-table)
  (setq local-abbrev-table text-mode-abbrev-table)
  (setq buffer-offer-save t)
  (make-local-variable 'send-pr:::site)
  (make-local-variable 'send-pr:::categories)
  (make-local-variable 'send-pr:::err-buffer)
  (make-local-variable 'send-pr:::spawn-to-send)
  (make-local-variable 'send-pr:::sent)
  (setq send-pr:::sent nil)
  (make-local-variable 'paragraph-separate)
  (setq paragraph-separate (concat (default-value 'paragraph-separate)
                                   "\\|" gnats::keyword "[ \t\n\f]*$"))
  (make-local-variable 'paragraph-start)
  (setq paragraph-start (concat (default-value 'paragraph-start)
                                "\\|" gnats::keyword))
  (run-hooks 'send-pr-mode-hook)
  t)

;;;;---------------------------------------------------------------------------
;;;; Functions to read and replace field values.
;;;;---------------------------------------------------------------------------

(defun gnats::position-on-field (field &optional quiet)
  (goto-char (point-min))
  (if (not (re-search-forward (concat "^>" field ":") nil t))
      (if quiet
          nil
        (error "Field `>%s:' not found." field))
    (re-search-forward "[ \t\n\f]*")
    (if (looking-at gnats::keyword)
        (backward-char 1))
    t))

(defun gnats::mail-position-on-field (field)
  (let (end
        (case-fold-search t))
    (goto-char (point-min))
    (re-search-forward "^$")
    (setq end (match-beginning 0))
    (goto-char (point-min))
    (if (not (re-search-forward (concat "^" field ":") end 'go-to-end))
        (insert field ": \n")
      (re-search-forward "[ \t\n\f]*"))
    (skip-chars-backward "\n")
    t))

(defun gnats::field-contents (field &optional elem move)
  (let (pos)
    (unwind-protect
        (save-excursion
          (if (not (gnats::position-on-field field t))
              nil
            (setq pos (point-marker))
            (if (or (looking-at "<.*>$") (eolp))
                t
              (looking-at ".*$")        ; to set match-{beginning,end}
              (gnats::nth-word
               (buffer-substring (match-beginning 0) (match-end 0))
               elem))))
      (and move pos (goto-char pos)))))

(defun gnats::functionp (thing)
  (or (and (symbolp thing) (fboundp thing))
      (and (listp thing) (eq (car thing) 'lambda))))

(defun gnats::field-values (field)
  "Return the possible (known) values for field FIELD."
  (let* ((fields (if (eq major-mode 'gnats:gnats-mode) gnats::fields
                   send-pr::fields))
         (thing (elt (assoc field fields) 1)))
    (cond ((gnats::functionp thing) (funcall thing))
          ((listp thing) thing)
          (t (error "ACK")))))

(defun gnats::field-default (field)
  "Return the default value for field FIELD."
  (let* ((fields (if (eq major-mode 'gnats:gnats-mode) gnats::fields
                   send-pr::fields))
         (thing (elt (assoc field fields) 2)))
    (cond ((stringp thing) thing)
          ((null thing) "")
          ((numberp thing) (car (elt (gnats::field-values field) thing)))
          ((gnats::functionp thing)
           (funcall thing (gnats::field-contents field)))
          ((eq thing t) (gnats::field-contents field))
          (t (error "ACK")))))

(defun gnats::field-type (field)
  "Return the type of field FIELD."
  (let* ((fields (if (eq major-mode 'gnats:gnats-mode) gnats::fields
                   send-pr::fields))
         (thing (elt (assoc field fields) 3)))
    thing))

(defun gnats::field-action (field)
  "Return the extra handling function for field FIELD."
  (let* ((fields (if (eq major-mode 'gnats:gnats-mode) gnats::fields
                   send-pr::fields))
         (thing (elt (assoc field fields) 4)))
    (cond ((null thing) 'ignore)
          ((gnats::functionp thing) thing)
          (t (error "ACK")))))

;;;;---------------------------------------------------------------------------
;;;; Point movement functions
;;;;---------------------------------------------------------------------------

(or (fboundp 'defsubst) (fset 'defsubst 'defun))

(defun send-pr::maybe-change-field (field)
  (setq field (car field))
  (let ((thing (gnats::field-contents field)))
    (and thing (eq t thing)
         (not (eq 'multi-text (gnats::field-type field)))
         (gnats:change-field field))))

(defun gnats:change-field (&optional field default)
  "Change the value of the field containing the cursor.  With arg, ask the
user for the field to change.  From a program, the function takes optional
arguments of the field to change and the default value to use."
  (interactive)
  (or field current-prefix-arg (setq field (gnats::current-field)))
  (or field
      (setq field
            (completing-read "Field: "
                             (if (eq major-mode 'gnats:gnats-mode)
                                 gnats::fields
                               send-pr::fields)
                             nil t)))
  (gnats::position-on-field field)
  (sit-for 0)
  (let* ((old (gnats::field-contents field))
         new)
    (if (null old)
        (error "ACK")
      (if (or (interactive-p) t)
          (let ((prompt (concat ">" field ": "))
                (domain (gnats::field-values field))
                (type (gnats::field-type field)))
            (or default (setq default (gnats::field-default field)))
            (setq new
                  (if (eq type 'enum)
                      (completing-read prompt domain nil t default)
                    (read-string prompt default 1))))
        (setq new default))
      (gnats::set-field field new)
      (funcall (gnats::field-action field) field old new)
      new)))

(defun gnats::set-field (field value)
  (save-excursion
    (gnats::position-on-field field)
    (delete-horizontal-space)
    (looking-at ".*$")
    (replace-match
     (concat (make-string (- gnats::indent (length field) 2) ?\40 ) value) t)))

(defun gnats::set-mail-field (field value)
  (save-excursion
    (gnats::mail-position-on-field field)
    (delete-horizontal-space)
    (looking-at ".*$")
    (replace-match (concat " " value) t)))

(defun gnats::before-keyword (&optional where)
  "Returns t if point is in some white space before a keyword.
If where is nil, then point is not changed; if where is t then point is moved
to the beginning of the keyword, otherwise it is moved to the beginning
of the white space it was in."
  ;;
  (if (looking-at gnats::before-keyword)
      (prog1 t
        (cond  ((eq where t)
                (re-search-forward "^>") (backward-char))
               ((not (eq where nil))
                (re-search-backward "[^ \t\n\f]") (forward-char))))
       nil))

(defun gnats::after-keyword (&optional where)
  "Returns t if point is in some white space after a keyword.
If where is nil, then point is not changed; if where is t then point is moved
to the beginning of the keyword, otherwise it is moved to the end of the white
space it was in."
  ;;
  (if (gnats::looking-after gnats::after-keyword)
      (prog1 t
        (cond  ((eq where t)
                (re-search-backward "^>"))
               ((not (eq where nil))
                (re-search-forward "[^ \t\n\f]") (backward-char))))
       nil))

(defun gnats::in-keyword (&optional where)
  "Returns t if point is within a keyword.
If where is nil, then point is not changed; if where is t then point is moved
to the beginning of the keyword."
  ;;
  (let ((old-point (point-marker)))
    (beginning-of-line)
    (cond ((and (looking-at gnats::keyword)
               (< old-point (match-end 0)))
           (prog1 t
             (if (eq where t)
                 t
               (goto-char old-point))))
          (t (goto-char old-point)
             nil))))

(defun gnats::forward-bofield ()
  "Moves point to the beginning of a field. Assumes that point is in the
keyword."
  ;;
  (if (re-search-forward "[ \t\n\f]+[^ \t\n\f]" (point-max) '-)
      (backward-char)
    t))

(defun gnats::backward-eofield ()
  "Moves point to the end of a field. Assumes point is in the keyword."
  ;;
  (if (re-search-backward "[^ \t\n\f][ \t\n\f]+" (point-min) '-)
      (forward-char)
    t))

(defun gnats::forward-eofield ()
  "Moves point to the end of a field. Assumes that point is in the field."
  ;;
  ;; look for the next field
  (if (re-search-forward gnats::keyword (point-max) '-)
      (progn (beginning-of-line) (gnats::backward-eofield))
  (re-search-backward "[^ \t\n\f][ \t\n\f]*" (point-min) '-)
  (forward-char)))

(defun gnats::backward-bofield ()
  "Moves point to the beginning of a field. Assumes that point is in the
field."
  ;;
  ;;look for previous field
  (if (re-search-backward gnats::keyword (point-min) '-)
      (gnats::forward-bofield)
    t))


(defun gnats:forward-field ()
  "Move point forward to the end of the field or to the beginning of the next
field."
  ;;
  (interactive)
  (if (or (gnats::before-keyword t) (gnats::in-keyword t)
          (gnats::after-keyword t))
        (gnats::forward-bofield)
    (gnats::forward-eofield)))

(defun gnats:backward-field ()
  "Move point backward to the beginning/end of a field."
  ;;
  (interactive)
  (backward-char)
  (if (or (gnats::before-keyword t) (gnats::in-keyword t)
          (gnats::after-keyword t))
      (gnats::backward-eofield)
    (gnats::backward-bofield)))

(defun gnats:next-field ()
  "Move point to the beginning of the next field."
  ;;
  (interactive)
  (if (or (gnats::before-keyword t) (gnats::in-keyword t)
          (gnats::after-keyword t))
      (gnats::forward-bofield)
    (if (re-search-forward gnats::keyword (point-max) '-)
        (gnats::forward-bofield)
      t)))

(defun gnats:previous-field ()
  "Move point to the beginning of the previous field."
  ;;
  (interactive)
  (backward-char)
  (if (or (gnats::after-keyword t) (gnats::in-keyword t)
          (gnats::before-keyword t))
      (progn (re-search-backward gnats::keyword (point-min) '-)
             (gnats::forward-bofield))
    (gnats::backward-bofield)))

(defun gnats:beginning-of-field ()
  "Move point to the beginning of the current field."
  (interactive)
  (cond ((gnats::in-keyword t)
         (gnats::forward-bofield))
        ((gnats::after-keyword 0))
        (t
         (gnats::backward-bofield))))

(defun gnats::current-field ()
  (save-excursion
    (if (cond ((or (gnats::in-keyword t) (gnats::after-keyword t))
               (looking-at gnats::keyword))
              ((re-search-backward gnats::keyword nil t)))
        (buffer-substring (match-beginning 1) (match-end 1))
      nil)))

;;;;---------------------------------------------------------------------------
;;;; Support functions
;;;;---------------------------------------------------------------------------

(defun gnats::looking-after (regex)
  "Returns t if point is after regex."
  ;;
  (let* ((old-point (point))
         (start (if (eobp)
                   old-point
                 (forward-char) (point))))
    (cond ((re-search-backward regex (point-min) t)
           (goto-char old-point)
           (cond ((eq (match-end 0) start)
                  t))))))

(defun gnats::nth-word (string &optional elem)
  "Returns the elem-th word of the string.
If elem is nil, then the first wort is returned, if elem is 0 then
the whole string is returned."
   ;;
  (if (integerp elem)
      (cond ((eq elem 0) string)
            ((eq elem 1) (gnats::first-word string))
            ((equal string "") "")
            ((>= elem 2)
             (let ((i 0) (value ""))
               (setq string             ; strip leading blanks
                     (substring string (or (string-match "[^ \t]" string) 0)))
               (while (< i elem)
                 (setq value
                       (substring string 0
                                  (string-match "[ \t]*$\\|[ \t]+" string)))
                 (setq string
                       (substring string (match-end 0)))
                 (setq i (+ i 1)))
               value)))
    (gnats::first-word string)))

(defun gnats::first-word (string)
  (setq string
        (substring string (or (string-match "[^ \t]" string) 0)))
  (substring string 0 (string-match "[ \t]*$\\|[ \t]+" string)))

;;;;---------------------------------------------------------------------------

(defun gnats::patch-exec-path ()
  ;;
  "Replaces `//' by `/' in `exec-path'."
  ;;
  ;(make-local-variable 'exec-path)
  (let ((err-buffer (get-buffer-create " *gnats::patch-exec-path*"))
        (ret))
    (setq exec-path (save-excursion (set-buffer err-buffer)
                                    (prin1 exec-path err-buffer)
                                    (goto-char (point-min))
                                    (while (search-forward "//" nil t)
                                      (replace-match "/" nil t))
                                    (goto-char (point-min))
                                    (setq ret (read err-buffer))
                                    (kill-buffer err-buffer)
                                    ret
                                    ))))

(defun gnats::get-value-from-shell (&rest command)
  "Execute shell command to get a list of valid values for `variable'."
  ;;
  (let ((err-buffer (get-buffer-create " *gnats::get-value-from-shell*")))
    (save-excursion
      (set-buffer err-buffer)
      (unwind-protect
          (condition-case var
              (progn
                (apply 'call-process
                       (car command) nil err-buffer nil (cdr command))
                (goto-char (point-min))
                (if (looking-at "[-a-z]+: ")
                    (error (buffer-substring (point-min) (point-max))))
                (read err-buffer))
            (error nil))
        (kill-buffer err-buffer)))))

(or (fboundp 'setenv)
    (defun setenv (variable &optional value)
      "Set the value of the environment variable named VARIABLE to VALUE.
VARIABLE should be a string.  VALUE is optional; if not provided or is
`nil', the environment variable VARIABLE will be removed.
This function works by modifying `process-environment'."
      (interactive "sSet environment variable: \nsSet %s to value: ")
      (if (string-match "=" variable)
          (error "Environment variable name `%s' contains `='" variable)
        (let ((pattern (concat "\\`" (regexp-quote (concat variable "="))))
              (case-fold-search nil)
              (scan process-environment))
          (while scan
            (cond
             ((string-match pattern (car scan))
              (if (eq nil value)
                  (setq process-environment (delq (car scan)
                                                  process-environment))
                (setcar scan (concat variable "=" value)))
              (setq scan nil))
             ((null (setq scan (cdr scan)))
              (setq process-environment
                    (cons (concat variable "=" value)
                          process-environment)))))))))

;;;; end of send-pr.el
;;----------------------------------------------------------------------------
(provide 'gold_sendpr)
