;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Additional/different tag functions for gold.el !                    ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(require 'etags)
(if (string-lessp emacs-version "21.0") ; overwrite incomplete etags-function
(defun tags-reset-tags-tables ()
  "Reset tags state to cancel effect of any previous \\[visit-tags-table] or \\[find-tag]."
  (interactive)
  ;; Clear out the markers we are throwing away.
  (let ((i 0))
    (while (< i find-tag-marker-ring-length)
      (if (aref (cddr tags-location-ring) i)
          (set-marker (aref (cddr tags-location-ring) i) nil))
      (if (aref (cddr find-tag-marker-ring) i)
          (set-marker (aref (cddr find-tag-marker-ring) i) nil))
      (setq i (1+ i))))
  (setq tags-file-name nil
        tags-location-ring (make-ring find-tag-marker-ring-length)
        find-tag-marker-ring (make-ring find-tag-marker-ring-length)
        tags-table-list nil
        tags-table-computed-list nil
        tags-table-computed-list-for nil
        tags-table-list-pointer nil
        tags-table-list-started-at nil
        tags-table-set-list nil))
)
;;
(gold-debug "gold_tags")
;;------------------------
(defvar permas-source-dir "/source/" "*Main directory for PERMAS source files.")
;;
(defun emacs-tags () "Set 'tags-file-name' to an Emacs-Lisp tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
   "/home/ast/files/EMACS.tags")))
;;
(defun gnats-tags () "Set 'tags-file-name' to an GNATS tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  "~gnats/GTAGS")))
;;
(defun permas-tags () "Set 'tags-file-name' to PERMAS tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permas/FILES/PERMAS"))))
;;
(defun permnew-tags () "Set 'tags-file-name' to PERMNEW tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permnew/FILES/PERMAS"))))
;;
(defun vpm-tags () "Set 'tags-file-name' to PERMAS-for-VisPer."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "vpm/FILES/VPM"))))
;;
(defun vgm-tags () "Set 'tags-file-name' to VisPer-Interface-Maintenance."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "vgm/FILES/VGM"))))
;;
(defun vgdev-tags () "Set 'tags-file-name' to VisPer-Interface-Development."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "vgdev/FILES/VGDEV"))))
;;
(defun visper-tags () "Set 'tags-file-name' to VisPer-Maintenance."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "vgm/FILES/VGM")
  (concat permas-source-dir "vpm/FILES/VPM"))))
; (concat permas-source-dir "permas/FILES/PERMAS"))))
;;
(defun visnew-tags () "Set 'tags-file-name' to VisPer-Development."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "vgdev/FILES/VGDEV")
  (concat permas-source-dir "permnew/FILES/PERMAS"))))
;;
(defun felix-tags () "Set 'tags-file-name' to FELIX tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "felix/FILES/FELIX")
  (concat permas-source-dir "permas/FILES/PERMAS"))))
;;
;; Set default TAGS table according to $DEFPROG environment variable
(let ((DEFPROG (GETENV "DEFPROG")))
     (cond ((equal DEFPROG "felix")       (felix-tags))
           ((equal DEFPROG "permas")      (permas-tags))
           ((equal DEFPROG "permnew")     (permnew-tags))
           ((equal DEFPROG "visper")      (visper-tags))
           ((equal DEFPROG "visnew")      (visnew-tags))
           ((equal DEFPROG "vgdev")       (vgdev-tags))
           ((equal DEFPROG "vgm")         (vgm-tags))
           ((equal DEFPROG "vpm")         (vpm-tags))
           (t                             (permas-tags))))
;;
;; Reread a TAGS table without querying, if it has changed.
(setq tags-revert-without-query t)
;(setq tags-add-tables nil) ; Do not concatenate tags-files ?

(defvar tags-read-only t
   "A non-nil value forces tags functions to load their files read-only !")
;;
(defvar last-tag-search  "?" "Type of last processed tag-searching")
(defun tags-find-file-hook () "Dummy function for setting tags-read-only !"
      (setq buffer-read-only tags-read-only))
;;
(defun find-tag-h ()
"Do regular `find-tag{-other-window}' and set the last-tag-search
to \"find-tag\" so repeat-tag-search can be used afterwards."
  (interactive)
  (if (or (eq last-command 'find-tag-h)
          (eq last-command 'tags-search-h)) (repeat-tag-search)
  (let ((find-file-hooks (append find-file-hooks (list 'tags-find-file-hook))))
       (call-interactively 'find-tag-other-window)
       (setq this-command 'find-tag-h)
       (setq last-tag-search "find-tag"))))
;;
(defun tags-search-h ()
"Do regular tags-search and set the last-tag-search
to \"tags-search\" so repeat-tag-search can be used afterwards."
  (interactive)
  (let ((find-file-hooks (append find-file-hooks (list 'tags-find-file-hook)))
        (err-regexp (errnum-at-point)))
       (if err-regexp (tags-search err-regexp)
                      (call-interactively 'tags-search))
       (setq this-command 'tags-search-h)
       (setq last-tag-search "search-tags")))
;;
(defun repeat-tag-search ()
   "Repeat last TAG Search forward (find-tag or search-tags)."
  (interactive)
  (let ((find-file-hooks (append find-file-hooks (list 'tags-find-file-hook))))
       (cond ((equal last-tag-search "find-tag")
              (message (concat "Repeated find-tag: " last-tag))
              (find-tag nil t) (message " ")) ;; Use same window afain !
             ((equal last-tag-search "search-tags")
              (message "Repeated tags-search !") ;;;!!!
              (call-interactively 'tags-loop-continue) (message " "))
             (t (message "No repeatable tag search defined !")))))
;;
;; Return a default tag to search for, based on the text at point.
(defun find-tag-default () (gold-function-at-point))

; Extend tag-symbol-match-p by special #member# tag
(fset 'orig-tag-symbol-match-p (symbol-function 'tag-symbol-match-p))
;
(defun tag-symbol-match-p (tag)
  (or (orig-tag-symbol-match-p tag)
      (tag-member-match-p tag)))
;
(defun tag-member-match-p (tag)  ;; Test for #tag# member tag
  (and (eq (char-after (point)) ?#)
       (eq (char-after (- (point) (length tag) 1)) ?#)))

;;----------------------------------------------------------------------------
(provide 'gold_tags)
