;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Special timecard support and (re)configurations !                   ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;;
(gold-debug "gold_timecard")
;;;
;;; Installing Timecard features:
;;;-------------------------------
(define-prefix-command       'timecard-map)
(substitute-key-definition   'self-insert-command
                             'timecard-checkout-key-account
                              timecard-map global-map)
(define-key timecard-map "b" 'timecard-insert-border)
(define-key timecard-map "d" 'timecard-sum-day)
(define-key timecard-map "f" 'timecard-find-timecard)
(define-key timecard-map "i" 'timecard-checkin)
(define-key timecard-map "m" 'timecard-month-summary)
(define-key timecard-map "o" 'timecard-checkout-account)
(define-key timecard-map "s" 'timecard-sum-week)
(define-key timecard-map "w" 'timecard-sum-week)
(define-key timecard-map "h" 'timecard-account-help)
(define-key timecard-map [kp-1]  'timecard-checkout-account)
;;;-----------
;;;
(defvar timecard-file (or (getenv "TIMECARD") (expand-file-name "~/.timecard"))
                      "* Name of the timecard-file")
(setq timecard-european-date    t )
(setq timecard-numeric-date    nil)
(setq timecard-rate-str       "rate")
(setq timecard-grandtotal-str "=> TOTAL")
(setq timecard-grandtotal-str "ARBEITSZEIT:")
(setq timecard-billed-str     "Arbeitszeit -")   ; "- billed -"
(setq timecard-unbilled-str   "Privat      -")   ; "- unbilled -"
(setq timecard-subtotals-str  "Gesamtstunden")   ; "- total hrs -"
(setq timecard-date/hours-str "DATE   HOURS   ") ;
(setq timecard-date/hours-str "Datum  Arbeit  ") ;
;;
;(if x-terminal (setq timecard-define-menus t))    ; only if you want menus

; (defconst timecard-longtime-regex "[ 0-9][0-9]+:[0-5][0-9]"
;   "Regular expression for a long timecard time entry in form HHHHHH:MM")
;;;-------------------------------------------------------------------------
(load "timecard-mode" nil t) ; Load original with error enabled, no message
;;;-------------------------------------------------------------------------
(add-hook 'timecard-mode-hook 'timecard-mode-ON)
(defun timecard-mode-ON () " "
  (define-key timecard-mode-map "\t"       'tab-left-right)
  (define-key timecard-mode-map "\C-M"     'newline)
  ;;
  (make-local-variable 'protect-tabs)
  (setq protect-tabs t)
  (setq tab-width (if timecard-numeric-date 10 11))
  (setq tab-stop-list (list tab-width  (* 2 tab-width)
                       (* 3 tab-width) (* 4 tab-width)
                       (* 5 tab-width) (* 6 tab-width)))
  ;;
  (goto-char (point-max))
  (recenter -2)
)

;; Change original functions:
;;----------------------------
(defun timecard-insert-border ()
  "Inserts a delimiter into the buffer to separates weeks"
  (interactive) (beginning-of-line) (insert "\n" (make-string 77 ?-) "\n\n"))
;------------------------------------------------------------------------
(defvar timecard-autosave-timecard t
 "*non-nil tries to save timecard (without query) whenever timecard-file is
modified via one of the GOLD timecard functions.")
(defvar timecard-read-only nil "*non-nil marks timecard file red-only.")
;------------------------------------------------------------------------
(defvar timecard-key-account-alist '(
  (?4 . "*PERM4*")
  (?5 . "*PERM*")
  (?6 . "*PERM*")
  (?7 . "*PERM*")
  (?e . "*ESPRIT*")
  (?h . "*Holiday*")
  (?j . "*Privat*Sport* Jogging")
  (?l . "*Lunch*")
  (?m . "*MAINT*")
  (?p . "*PROJECT*")
  (?s . "*SUPPORT*")
  (?t . "*TOOLS*")
) "List of key characters associated with standard account texts.")
;;
(defvar timecard-account-alist '(
  ("*PERM4*"   . ("Development of PERMAS version 4"
     ("*CA*"    . "Contact Analysis")
     ("*RA*"    . "Random response")
    ))
  ("*PIT4*"    . ("Development of PERMIT"
     ("*PAT*"   . "PATRAN interface")
     ("*SUP*"   . "I-DEAS/Supertab interface")
    ))
  ("*PERM*"    . ("Development of PERMAS"
     ("*CA*"    . "Contact Analysis")
     ("*RA*"    . "Random response")
    ))
  ("*MAINT*"   . ("Maintenance"
     ("*QA*"    . "Quality Assurance")
     ("*PERM4*")
     ("*INSTALL*")
    ))
  ("*SUPPORT*" . ("Support"
     ("*TRUMPF*")
     ("*GECI*")
    ))
  ("*PROJECT*" . ("Engineering projects"
     ("*GILLET*")
     ("*PORSCHE*")
    ))
  ("*SALES*"   . ("Sales activities"
     ("*CAT*"   . "Presentation on CAT Stuttgart")
     ("*NEWS*")
    ))
  ("*ESPRIT*"  .  "Esprit's PERMPAR project")
  ("*BMFT*"    .  "BMFT's PARFEM project")
  ("*ESSI*"    .  "ESSI project")
  ("*TOOLS*"   . ("Development and maintenance of TOOLS"
     ("*Unix*"  . "General Unix scripts")
     ("*Emacs*" . "Developments, maintenance and support for GNU-Emacs")
     ("*Gnu*"   . "Non-Emacs GNU tools (gcc, gnats or the like)")
     ("*TeX*"   . "TeX/LaTeX")
    ))
  ("*INTES*"   . ("All company stuff not in any other account"
     ("*Office*" )
     ("*Coffee*" )
    ))
  ("*Holiday*" . ("Your personal holidays (take 8 hours a day)"
     ("*Ski*"    )
     ("*Summer*" )
    ))
  ("*Ill*"     .  "Abscence caused by illness (take 8 hours a day)")
  ("*Lunch*"   .  "Eating pauses (good appetite)")
  ("*Privat*"  . ("Private times inside the company"
     ("*Party*" . "Vine, wives and singing")
     ("*Sport*" . "Jogging, tennis etc.")
     ("*Games*" . "Relaxing with flight simulator, bz, tetris or the like.")
    ))
)"Alist of standard accounts to be used in 'timecard-account-completion' with
cons cells:
             (\"*account*\" .  \"docstring\")
 and/or:     (\"*account*\" . (\"docstring\"))
 and/or:     (\"*account*\" . (\"docstring for main account\"
                            (\"*sub-account*\" . \"docstring for sub-account\")
                            (\"*sub-account*\" .  nil)
                            (\"*sub-account*\")))")
;;
;;;===========================================================================
;(add-hook 'kill-emacs-hook 'timecard-save-timecard-file)
(defun timecard-save-timecard-file ()
  (let ((buffer (get-file-buffer timecard-file)))
       (if buffer (save-excursion (set-buffer buffer)
                  (if (buffer-modified-p) (save-buffer))
                  (if timecard-read-only (setq buffer-read-only t))
))))

(defun timecard-checkin-and-kill ()
  "Timecard chekin&save provided for script calls with 'emacs -f ...'"
  (interactive)
  (if (file-locked-p timecard-file) nil
      (timecard-checkin nil)        ;;; (timecard-checkin-once)
      (timecard-save-timecard-file))
  (kill-emacs))

(defun timecard-checkout-and-kill ()
  "Timecard chekout&save provided for script calls with 'emacs -f ...'"
  (interactive)
  (if (file-locked-p timecard-file) nil
      (timecard-checkout nil)
      (if (and timecard-account-default
              (string-lessp "*" timecard-account-default ))
          (insert timecard-account-default)
          (insert "Logout*"))
      (timecard-save-timecard-file))
  (kill-emacs))

;;;---------------------------------------------------------------------------
(defvar timecard-account-default "*" "*Default string for account completion.")
(defvar timecard-account-must-match t
  "Non-nil forces 'timecard-account-completion' user to take an account out of
'timecard-account-alist'")

;;;###autoload
(defun timecard-account-help (account &optional account-alist)
  "Show a help page for the given timecard ACCOUNT. If the ACCOUNT is found in
optional ACCOUNT-ALIST (default 'timecard-account-alist') then the alist's
corresponding docstring (and sub-account alist, if any) is utilized for the
help page text, see documentation of 'timecard-account-alist'.
On interactive calls 'timecard-account-help' asks the user for one ACCOUNT
out of 'timecard-account-alist' before displaying help."
  (interactive "P")
  (or account-alist (setq account-alist timecard-account-alist))
  (or account (setq account (timecard-account-completion account-alist)))
  ;;
  (let ((account-help (cdr (assoc account account-alist))))
    (with-output-to-temp-buffer "*Help*"
     ;;
     ;; Print 2 header lines with ACCOUNT name and account-help-text (if any)
     (princ (format "The keyword %s is %s 'timecard-account-alist' !\n\n"
             account (if account-help "a timecard account stored in"
                                      "NOT a timecard account out of")))
     (princ (format "%s is used for: %s\n"
             account (if account-help (if (listp account-help)
                                          (car account-help) account-help)
                                      "Not documented")))
     (princ (make-string (length account) ?-))
     ;;
     ;; Print a list of sub-accounts from (cdr account-help) alist, if any
     (if (listp account-help)
         (while (setq account-help (cdr account-help))
           (let ((sub-account (car account-help)))
                (princ (format "\n%s%-12s: %s" account
                          (substring (car sub-account) 1)
                          (or (cdr sub-account) "Not documented")))
        )))
     ;; Print a dummy sub-account as last help page line
     (princ (format "\n%s%-12s: Additional sub-account texts" account "..."))
     ;;
     (print-help-return-message))))

;;###autoload
(defun timecard-context-help ()
  "Show a context-sensitive help for point position in timecard-mode."
  (interactive)
  (cond
   ;; 'timecard-context-help' typed twice at same position => describe-mode
   ((eq last-command this-command) (describe-mode))
   ;;
   ;; Checked out entry lines: Show a help page for the account string
   ((save-excursion (beginning-of-line 1) (looking-at timecard-entry-regex))
    (timecard-account-help
     (format "*%s*" (if (< (match-beginning 4) (match-end 4))
                           (buffer-substring (match-beginning 4) (match-end 4))
                           (eval "???")))))
   ;;
   ;; Date/hours-line: Account help if on account (describe-mode otherwise)
   ((save-excursion (beginning-of-line 1) (looking-at timecard-date/hours-str))
    (let ((pos (match-end 0))) (save-excursion
     (if (and (re-search-backward "\\b" nil t) (<= pos (match-beginning 0)))
         (timecard-account-help
          (format "*%s*" (progn (setq pos (match-beginning 0))
                                (forward-char 1)(re-search-forward "\\b" nil t)
                                (buffer-substring pos (match-beginning 0)))))
         (describe-mode)))))
    ;;
    ;; Any other position: Just describe timecard-mode
    (t (describe-mode))))

;;###autoload
(defun timecard-checkout-account (account)
  "Just normal 'timecard-checkout' with an additional ACCOUNT insertion.

If ACCOUNT is non-nil it must be an account string starting with *: \"*text...\"

If ACCOUNT arg is nil then an interactive account completion is performed
using the 'timecard-account-alist' for an alist of valid account strings.
The string variable 'timecard-account-default' may hold a default account,
variable 'timecard-account-must-match' forces the user to take one of the
accounts out of 'timecard-account-alist'."
  (interactive "P")
  (if timecard-autosave-timecard ;; Reload timecard-file from disk (ftp!!!)
     (let ((buffer (get-file-buffer timecard-file)))
          (if buffer (save-excursion (set-buffer buffer)
                     (if (not (buffer-modified-p)) (revert-buffer t t))))))
  (timecard-checkout nil)
  (let ((name (substring (or account (timecard-account-completion)) 1)))
       (insert name)
       (insert (read-from-minibuffer (concat "Add comment: *" name)))
  (if timecard-autosave-timecard (timecard-save-timecard-file))
))

(defun timecard-account-completion (&optional account-alist)
  "Get timecard account name out of an interactive account completion using the
optional ACCOUNT-ALIST (default is 'timecard-account-alist') as a list of valid
account strings.
The string variable 'timecard-account-default' may hold a default account,
a non-nil 'timecard-account-must-match' forces the user to take one of the
accounts out of 'timecard-account-alist'."
  (let*((completion-ignore-case t)
        (alist (sort (copy-alist (or account-alist timecard-account-alist))
               (function (lambda (x y) (string-lessp (car x) (car y)))))))
      (completing-read "Account name: " alist nil timecard-account-must-match
                                            (cons timecard-account-default 1))
))

;------------------------------------------------------------------------
(defvar timecard-key-account-alist '(
  (?h . "*Holiday*")
  (?l . "*Lunch*")
  (?m . "*MAINT*")
  (?p . "*PROJECT*")
  (?s . "*SUPPORT*")
  (?t . "*TOOLS*")
) "List of key characters associated with standard account texts.")
;;
(defun timecard-checkout-key-account ()
"Perform a 'timecard-checkout-account' with an account determined from the last
typed key (last-command-char) in association with 'timecard-key-account-alist'"
  (interactive)
  (let ((account (cdr (assoc last-command-char timecard-key-account-alist))))
       (if account (timecard-checkout-account account)
          (beep)
          (message "No 'timecard-key-account-alist' entry associated to key '%c' !"
                    last-command-char))
))
;------------------------------------------------------------------------
(defun timecard-month-summary ()
  (interactive)
  (beginning-of-line 1)
  (or (re-search-forward timecard-entry-regex nil 'go)
      (re-search-backward timecard-entry-regex nil 'go))
  (let*((day (buffer-substring (match-beginning 1) (match-end 1)))
        (month (substring day 3 6))
        (timecard (current-buffer))
        (buffer (concat "*" month "-95*"))
        (save))
       ;;
       (unconditional-new-buffer buffer)
       (set-buffer buffer) (insert-buffer timecard)
       ;;
       (re-search-forward "^--------" nil 'go)
       (beginning-of-line 2) (insert "\n")
       ;;
       (setq save (point))
       (re-search-forward (concat "^..-" month) nil 'go)
       (re-search-backward "^----" nil 'go)
       (beginning-of-line 1)
       (delete-region save (point))
       ;;
       (end-of-buffer) (setq save (point))
       (re-search-backward (concat "^..-" month) nil 'go)
       (beginning-of-line 1)
       (delete-region (point) save)
       ;;
       (beginning-of-buffer)
       (delete-matching-lines "^---------------------------------* *")
       (beginning-of-buffer)
       (delete-matching-lines (concat "^..-" month "::"))
       ;;
       (beginning-of-buffer)
       (while (re-search-forward "^ARBEITSZEIT: " nil 'go)
              (delete-region (match-beginning 0)
                     (re-search-forward "^Gesamtstunden.*$\\|\\'" nil 'go)))
       ;;
       (beginning-of-buffer) (delete-double-lines)
       ;;
       (timecard-mode) (timecard-sum-week)
       (beginning-of-buffer)
       (re-search-forward "^ARBEITSZEIT: " nil 'go)
       (setq save (buffer-substring (match-beginning 0)
                                    (re-search-forward "^Datum.*$" nil 'go)))
       (re-search-forward "^Arbeitszeit - " nil 'go)
       (goto-char (match-beginning 0))
       (insert save) (insert "\n")
       ;;
       (switch-to-buffer buffer) (delete-other-windows)
       (goto-char (match-beginning 0)) (recenter)
))
;;----------------------------------------------------------------------------
(provide 'gold_timecard)
