;;; permas_fmt.el --- PERMAS FMT mode (edit/view PERMAS format files)

;; Copyright (C) 1999 Free Software Foundation, Inc.

;; Author: Markus Ast <ast@intes.de>
;; Last Change: 29-Nov-10
;; Keywords: PERMAS, formats, editing

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; permas-fmt-mode is a major mode to edit/view PERMAS format files.

;;; Code:

;;;###autoload
(defun permas-fmt-mode ()
  "Edit/view mode for PERMAS format files (*.fmt)."
  (interactive) (text-mode) ; start with text-mode initialization
  (setq mode-name  "PERMAS-FMT")
  (setq major-mode 'permas-fmt-mode)
;;
  (make-local-variable 'comment-start) (setq comment-start "!")
  (make-local-variable 'kf-def-regexp) (setq kf-def-regexp "^[^!]*[$]")
  ;;
  (local-set-key "\e\C-a"    'permas-fmt-prev)
  (local-set-key "\e\C-e"    'permas-fmt-next)
;;
  (setq context-help-function   'permas-fmt-help)
; (setq context-update-function 'permas-Fmt-context-update)
; (setq compile-buffer-function 'permas-compile-buffer)
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(permas-fmt-font-lock-keywords t t))
  (run-hooks 'permas-fmt-mode-hook))


;; font-lock support (list of regular expressions for highlighting)
;;-----------------------------------------------------------------
(setq permas-fmt-font-lock-keywords
  (list
;;;;;;;;;;
;   '("^ZYXWVU.*" (0 'font-lock-string-face t))
    '("^</grp>"   (0 'font-lock-string-face t))
    '("^</fmt>"   (0 'font-lock-string-face t))
    '("^!.*$"     (0 'font-lock-comment-face t))
    '("^! *$"     (0 'font-lock-string-face t))
    '("^[!<].*[^X\n]*\\(XXX+\\)" (1 'font-lock-mismatch-face t))
    '("^[!<].*[^X\n]*\\(XXXXXXXXXXXXXXXXX+\\)" (1 'font-lock-separator-face t))
;   '("^[!c]err [^9]*\\(99+\\)"  (1 'font-lock-mismatch-face t))
;;
;   '("^\\(ELT\\) *'\\([0-9]+\\)\\([0-9][0-9][0-9][0-9]\\)' *\\(READ\\)"
;    (1 'font-lock-function-name-face) (2 'font-lock-reference-face)
;    (3 'font-lock-gold-face) (4 'font-lock-string-face))
;;
    '("^\\(<grp\\) *id=\\([0-9]+\\)\\(>\\)"
     (1 'font-lock-function-name-face) (2 'font-lock-reference-face)
     (3 'font-lock-function-name-face))
;;
    '("<! +\\([0-9]+ formats\\)" (1 'font-lock-gold-face))
;;
    '("^\\(<fmt\\) *id=\\([0-9]+\\)\\([0-9][0-9][0-9][0-9]\\)\\(>\\)"
     (1 'font-lock-function-name-face) (2 'font-lock-reference-face)
     (3 'font-lock-gold-face) (4 'font-lock-function-name-face))
;;
    '("^ \\([0-9][0-9][0-9]\\) \\([0-9][0-9][0-9][0-9]\\) "
     (1 'font-lock-reference-face) (2 'font-lock-gold-face))
;;
    '("^!?\\(<signature>\\)\\(.*\\)\\(</signature>\\)"
     (1 'font-lock-function-name-face t) (2 'font-lock-reference-face t)
     (3 'font-lock-function-name-face t))
;;
    ;; FMT-formats
;   '("'.*[^%]\\(%%\\)[^%]" (1 'font-lock-keyword-face t))
    '("%&"                                                (0 'font-lock-keyword-face t))
    '("%[#0<=>+-]* ?[#0<=>+-]*[0-9.]*[DLP]?[sefgibox]\\b" (0 'font-lock-keyword-face t))
;   '("%[ #0<=>+-]*[0-9.]*[DhlLP]?[bcdefginopsux]\\b"     (0 'font-lock-keyword-face t))
    '("[^%]\\(%[#+-<=> ]?[0-9.]*([^)]*)\\)"               (1 'font-lock-keyword-face t))
))


;; Next/prev header line
;;-----------------------
(defun permas-fmt-next (&optional arg)
  "Move forward to next format definition. With argument, do it that many times.
Negative argument -N means move back to Nth preceding end of xydata."
  (interactive "p") (if (or (null arg) (= arg 0)) (setq arg 1))
  (forward-char 1) (permas-fmt-prev (- arg)))
;;
(defun permas-fmt-prev (&optional arg)
"Move to start of current format definition block. With argument, do it that many times.
Negative arg -N means move forward to Nth following start of data block."
  (interactive "p") (or (bolp) (end-of-line 1))
  (and (re-search-backward "^<grp" nil 'move (or arg 1))
       (progn (beginning-of-line)
          (if (looking-at "^ *[$]EXIT")
              (if (> arg 0) (permas-fmt-prev) (permas-fmt-next)))
        t)))

;;----------------------------------------------------------------------------
;;;###autoload
(defun show-permas-fmt-group (group) "Show help for PERMAS FMT Group."
  (interactive "sShow PERMAS FMT Group (3 digits): ")
  (if (equal group "09") (setq group "009"))
  (if (equal group "10") (setq group "010"))
  (if (equal group "11") (setq group "011"))
  (if (equal group "12") (setq group "012"))
  (if (equal group "13") (setq group "013"))
  (if (equal group "14") (setq group "014"))
  (if (equal group "25") (setq group "025"))
  (if (permas-formats-buffer)
      (if (search-forward (format "GRP ID=%s" group) nil t)
          (progn (line-to-top-of-window) (forward-char 2))
          (message "Can not find group %s !" group))))
;;
;;;###autoload
(defun permas-formats-buffer () "Switch to [reloaded] PERMAS/PERMNEW formats buffer."
   (if (or (if (get-buffer "*Formats*") (switch-to-buffer-other-window "*Formats*"))
           (load-permas-formats)) (goto-char 0) (eval nil)))
;;
(defun load-permas-formats () "Load and ReFormat PERMAS/PERMNEW formats file."
  (let ((file (format "%s/formats" permas-files-dir group)))
       (if (file-readable-p file)
           (progn (unconditional-new-buffer "*Formats*" 'other)
                  (message (format "Loading %s to *Formats* buffer" file))
                  (insert-file-contents file)
                  (while (< (forward-line 1) 1)        ; next: not last-line
                         (if (looking-at "<GRP")     (permas-grpx))
                         (if (looking-at "<FMT ID=") (permas-fmtx)))
                  (goto-char 0) (permas-fmt-mode) (message " ")
                  (set-buffer-modified-p nil) (setq buffer-read-only t))
           (beep) (message "Can not find or read file %s !" file) (eval nil))))
;;
(defun permas-grpx () 
  "ReFormate a <GRP>-Line from FORMATS file for FMT-Help-Buffer."
  (while (looking-at "<GRP ID=\\([0-9]+\\)")
         ;; Read list of group-members ?????
         (forward-line 2)
         (if (looking-at " *$"  ) (delete-current-line 1)) ; remove blank-line
))
;;
(defun permas-fmtx () 
  "ReFormate a Block of <FMT>-Lines from FORMATS file for FMT-Help-Buffer."
  (while (looking-at "<FMT ID=\\([0-9][0-9][0-9]\\)\\([0-9][0-9][0-9][0-9]\\).*$")
         (replace-match " \\1 \\2 ") (delete-char 1) ; 1st line of format
         (while (and (< (forward-line 1) 1)          ; nxt: not End-Of-File
                     (not (looking-at "</FMT"))      ; nxt: not End-Of-FMT
                     (not (looking-at " *$")))       ; nxt: not blank-line
                (insert "          "))               ; indent continuation line
         (if (looking-at "</FMT") (delete-current-line 1)) ; remove </FMT> line
         (if (looking-at " *$"  ) (delete-current-line 1)) ; remove blank-line
))
;;
;;----------------------------------------------------------------------------
;(defun PERMAS-fmt-help (&optional query)
;"Show help for PERMAS FMT input command of actual line (info from um450).
;Ask interactively for keyword if QUERY is non-nil (short cut otherwise)."
; (interactive)
; (let ((key (keyword-at-point)) (uci))
;      (if query (let ((enable-recursive-minibuffers t))
;                (setq key (read-with-default "PERMAS input command" key))))
;      (help-uci-key key)
;))
;;;----------------------------------------------------------------------------
(defun permas-fmt-help ()
"Format fields for PERMAS error messages ...

%i  - integer
%f  - float
%E  - exponent
%G  - float or exponent
%s  - character string

%(name) fields... "
      (interactive)
      (save-window-excursion
           (switch-to-buffer "*Help*")
           (setq buffer-read-only nil) (erase-buffer)
           (insert (documentation 'permas-fmt-help))
           (call-interactively 'GOLD-quit-help)))

;;;----------------------------------------------------------------------------
(provide 'permas_fmt)

;;; permas_fmt.el ends here
