;;; permas_log.el --- PERMAS LOG mode (viewing PERMAS log files)

;; Copyright (C) 1999 Free Software Foundation, Inc.

;; Author: Markus Ast <ast@intes.de>
;; Last Change: 26-Sep-08
;; Keywords: PERMAS, log, protocol, viewing

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; permas-log-mode is a major mode for viewing PERMAS log files.

;;; Code:

;;;###autoload
(defun permas-log-mode ()
  "Viewing mode for PERMAS log files (*.log)."
  (interactive) (text-mode) ; start with text-mode initialization
  (setq mode-name  "PERMAS-LOG")
  (setq major-mode 'permas-log-mode)
  (local-set-key "\e\C-a" 'permas-log-prev)
  (local-set-key "\e\C-e" 'permas-log-next)
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(permas-log-font-lock-keywords t t))
  (run-hooks 'permas-log-mode-hook))


;; font-lock support (list of regular expressions for highlighting)
;;-----------------------------------------------------------------
(setq permas-log-font-lock-keywords
  (list
   ;;
   ;; UCI echos
   '("^\\(_[0-9]+>\\)? ?\\(> .*\\)$"   (2 'font-lock-comment-face))
   ;;
   ;; Warnings and Errors
;? '((permerr-regexp "C") . (2 'font-lock-comment-face)))
;? '(eval . (cons (permerr-regexp "C") (2 'font-lock-comment-face)))
;? '("^\\(_[0-9]+>\\)? ?[*][CWEFX][*] " . anchored-highlighter...
;;
   '("^\\(_[0-9]+>\\)? ?\\([*]C[*] .*\\(\n\\(_[0-9]+>\\)?           .*\\)*\\)"
                               (2 'font-lock-comment-face))
   '("^\\(_[0-9]+>\\)? ?\\([*]W[*] .*\\(\n\\(_[0-9]+>\\)?           .*\\)*\\)"
                               (2 'font-lock-warning-face))
   '("^\\(_[0-9]+>\\)? ?\\([*]E[*] .*\\(\n\\(_[0-9]+>\\)?           .*\\)*\\)"
                               (2 'font-lock-separator-face))
   '("^\\(_[0-9]+>\\)? ?\\([*][FX][*] .*\\(\n\\(_[0-9]+>\\)?           .*\\)*\\)"
                               (2 'font-lock-mismatch-face))
;   '("^_?[0-9]*>? ?[*]C[*] .*\\(\n           .*\\)*" . 'font-lock-comment-face)
;   '("^_?[0-9]*>? ?[*]W[*] .*\\(\n           .*\\)*" . 'font-lock-warning-face)
;   '("^_?[0-9]*>? ?[*]E[*] .*\\(\n           .*\\)*" . 'font-lock-separator-face)
;   '("^_?[0-9]*>? ?[*]F[*] .*\\(\n           .*\\)*" . 'font-lock-mismatch-face)
;   '("^_?[0-9]*>? ?[*]X[*] .*\\(\n           .*\\)*" . 'font-lock-mismatch-face)
   '("^\\(_[0-9]+>\\)? ?[*]C[*] *\\([0-9]+/ *[0-9>]+\\)" (2 'font-lock-string-face t))
   '("^\\(_[0-9]+>\\)? ?[*]W[*] *\\([0-9]+/ *[0-9>]+\\)" (2 'font-lock-comment-face t))
   '("^\\(_[0-9]+>\\) +"  (0 'default t))
   '("<see list>"         (0 'font-lock-list-face t))
   '("<[0-9]+ items>"     (0 'font-lock-list-face t))
   ;;
   ;; switch = value
   '("\\([a-z#?+][a-z.0-9_#-]*\\) *= *\\([a-z.'0-9/_+-]+\\)"
                                           (1 'font-lock-variable-name-face)
                                           (2 'font-lock-type-face))
   ;; Iteration messages (suspit)
   '("=> *\\([0-9]+\\).*\\([0-9][.][0-9ed+-]*\\)"
                                           (1 'font-lock-variable-name-face)
                                           (2 'font-lock-type-face))
   ;; OUT tables
;  '("^ [|│║].*[|│║:].*[|│║]"                . 'font-lock-string-face)
   '("[|│║+][._=+-]+[|│║+]"                  . 'font-lock-string-face)
   '("[^═]═+[^═]"                            . 'font-lock-string-face)
   '("[|│║].* \\(:\\) "                     (1 'font-lock-string-face))
   '("[|│║]"                                 . 'font-lock-string-face)
   ;;
   ;; timer
   '("^\\(_[0-9][0-9]>\\)  ?\\([0-2][0-9]\\)\\([0-5][0-9]\\)\\([0-5][0-9]\\) "
                  (3 'font-lock-comment-face)
                  (4 'font-lock-variable-name-face))
   ;;
   ;'("  !.*!"      . 'font-lock-comment-face)
   ;'(" ----+"      . 'font-lock-comment-face)
   ;'(" ====+"      . 'font-lock-comment-face)
   ;;
   ;; CASTATUS-LOG (new)
   '("\\(Outer\\) \\(Inner\\) \\(SlipIt\\) +: +\\(Passive\\) +\\(Force\\) +\\(Active\\) +\\(Stick\\)/\\(Slipping\\)"
     (1 'font-lock-function-name-face)
     (2 'font-lock-variable-name-face)
     (3 'font-lock-comment-face)
     (4 'font-lock-type-face)
     (5 'font-lock-variable-name-face)
     (6 'font-lock-function-name-face)
     (7 'font-lock-keyword-face)
     (8 'font-lock-comment-face))
   '(" [|│║] +\\([0-9]+\\) +\\([0-9]+\\) \\([ 0-9]+\\) +: +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) "
     (1 'default) (2 'default) (3 'default)
     (4 'font-lock-type-face)
     (5 'font-lock-variable-name-face)
     (6 'font-lock-function-name-face)
     (7 'font-lock-keyword-face)
     (8 'font-lock-comment-face))
   '(" [|│║] +\\([0-9]+\\) +\\([gsk]+ +[0-9]+\\) +: +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) "
     (1 'default)
     (2 'font-lock-list-face)
     (3 'font-lock-type-face)
     (4 'font-lock-variable-name-face)
     (5 'font-lock-function-name-face)
     (6 'font-lock-keyword-face)
     (7 'font-lock-comment-face))
   ;;
   ;; CASTATUS-LOG (old)
   '("\\(Outer\\) \\(Inner\\) \\(SlipIt\\) : +\\(Passive\\) +\\(Active\\) +\\(Stick\\)/\\(Slipping\\)"
     (1 'font-lock-variable-name-face) (2 'font-lock-variable-name-face)
     (3 'font-lock-comment-face) (4 'font-lock-type-face) (6 'font-lock-function-name-face)
     (7 'font-lock-keyword-face) (8 'font-lock-comment-face))
   '(" [|│║] +\\([0-9]+\\) +\\([0-9]+\\) \\([ 0-9]+\\) : +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) "
     (1 'default) (2 'default)
     (3 'default) (4 'font-lock-type-face) (5 'font-lock-function-name-face)
     (6 'font-lock-keyword-face) (7 'font-lock-comment-face))
   ;;
   ;; Pro/Step-Log
   '("\\(Act.\\|Step\\|PrL.\\|Proc\\|Sub.\\|Trac\\) \\([^ ]+\\) *\\(Start\\|Exit\\)"
     (1 font-lock-keyword-face) (2 font-lock-function-name-face) (3 font-lock-keyword-face))
   '("\\(Act.\\|Step\\|PrL.\\) \\([^ ]+\\) *\\(.*\\)"
     (1 font-lock-keyword-face) (2 font-lock-function-name-face) (3 font-lock-comment-face))
;   '("\\(Sub.\\) \\([^ ]+\\)"
;     (1 font-lock-keyword-face) (2 font-lock-function-name-face) (3 font-lock-comment-face))
   '("\\(Component\\|Situation\\) *\\([^, ]*\\)" . 'font-lock-string-face)
   ;;
   ;; Pro/Step-Arg
   '("\\(Create\\|Gen\\|Mod\\|Modify\\|Read\\|Scr\\|Scratch\\) *\\(Book\\|[C#]?Table\\|Sequence\\|VirtBook\\) +\\([^ ]*\\)"
     (1 font-lock-type-face) (2 font-lock-type-face) (3 font-lock-variable-name-face))
   ;;
   ;; Object-Arg
   '("\\(Copy\\|Delete\\|Link\\|Refund\\|Rename\\) *\\(Book\\|[C#]?Table\\|Sequence\\|VirtBook\\) +\\([^ \n]+\\)"
     (1 font-lock-type-face) (2 font-lock-type-face) (3 font-lock-variable-name-face))
   '("\\(Copy\\|Link\\|Rename\\) *\\(Book\\|Table\\|Sequence\\|VirtBook\\).*\\(from\\|to\\) +\\([^ \n]+\\)"
     (3 font-lock-type-face) (4 font-lock-variable-name-face))
   ;;
   '("\\(Sub.\\) \\([^ ]+\\) *\\(Copy\\|Create\\|Delete\\|Link\\|Refund\\|Rename\\) *\\(Book\\|[C#]?Table\\|Sequence\\|VirtBook\\) +\\([^ \n]+\\)"
     (1 font-lock-keyword-face) (2 font-lock-function-name-face)
     (3 font-lock-type-face) (4 font-lock-type-face) (5 font-lock-variable-name-face))
   ;;
   ;; Trac-Log
   '("Trac " . 'font-lock-keyword-face)
   ;;
   ;; Time strings/Counters
   '("\\<\\(Elapsed\\|Sys\\|User\\)\\>" . 'font-lock-builtin-face)
   ;;
   ;; Special key-lines
   '("Accuracy Limit .*" . 'font-lock-comment-face)
   '("Starting CA-.*"    . 'font-lock-comment-face)
   '("ESTM Resource .*"  . 'font-lock-comment-face)
   '("\\(Size:\\|Time:\\) *\\([0-9.]+\\) (.*), *\\([0-9.]+\\) ("
                        (1 'font-lock-variable-name-face)
                        (2 'font-lock-type-face)
                        (3 'font-lock-type-face))
;;
   '("Version +\\([0-9.]*\\) "               . (1 'font-lock-match-face))
   '("[|║] +[|║] *\\( [^ ]+ \\) *[|║] +[|║]" . (1 'font-lock-gold-face))
;;
;; Date
   '("\\b\\([0-3][0-9]\\)/\\([0-1][0-9]\\)/\\([0-9][0-9]\\)\\b"
        (2 'font-lock-comment-face) (3 'font-lock-variable-name-face))
   '("\\b\\([0-3][0-9]\\)-\\([a-z][a-z][a-z]\\)-\\([0-9][0-9]\\)\\b"
        (2 'font-lock-comment-face) (3 'font-lock-variable-name-face))
;; Time
   '("\\b\\([0-2][0-9]\\):\\([0-5][0-9]\\):\\([0-5][0-9]\\)\\b"
       (2 'font-lock-comment-face) (3 'font-lock-variable-name-face))

   ;; Final ParLOG-Prefix
   '("^_01>"      . (0 'font-lock-list-face     t))
   '("^_0[2468]>" . (0 'font-lock-doc-face      t))
   '("^_0[3579]>" . (0 'font-lock-constant-face t))
   ;;
))


;; Next/prev step
;;----------------
(defun permas-log-next (&optional arg)
  "Move forward to next step log. With argument, do it that many times.
Negative argument -N means move back to Nth preceding end of step."
  (interactive "p") (if (or (null arg) (= arg 0)) (setq arg 1))
  (forward-char 1) (permas-log-prev (- arg)))
;;
(defun permas-log-prev (&optional arg)
"Move to start of current step log block. With argument, do it that many times.
Negative arg -N means move forward to Nth following start of step."
  (interactive "p") (or (bolp) (end-of-line 1))
  (if (re-search-backward "^ *[0-9]+ +Step [^ ]+ +[^E ]" nil 'move (or arg 1))
      (re-search-forward "Step")))
;;
(provide 'permas_log)

;;; permas_log.el ends here
