;;; permas_pro.el --- PERMAS PRO mode (viewing PERMAS protocol files)

;; Copyright (C) 1999 Free Software Foundation, Inc.

;; Author: Markus Ast <ast@intes.de>
;; Last Change: 26-Sep-08
;; Keywords: PERMAS, protocol, viewing

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; permas-pro-mode is a major mode for viewing PERMAS protocol files.

;;; Code:

;;;###autoload
(defun permas-pro-mode ()
  "Viewing mode for PERMAS protocol files (*.pro)."
  (interactive) (text-mode) ; start with text-mode initialization
  (setq mode-name  "PERMAS-PRO")
  (setq major-mode 'permas-pro-mode)
  (local-set-key "\e\C-a" 'permas-pro-prev)
  (local-set-key "\e\C-e" 'permas-pro-next)
  (setq context-help-function 'permas-pro-help)
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(permas-pro-font-lock-keywords t t))
  (run-hooks 'permas-pro-mode-hook))


;; font-lock support (list of regular expressions for highlighting)
;;-----------------------------------------------------------------
(setq permas-pro-font-lock-keywords
  (list
   ;;
   ;; UCI echos
   '("^ ?> .*$"                             . 'font-lock-comment-face)
   ;;
   ;; Warnings and Errors
   '("^ ?[*]C[*] .*\\(\n             .*\\)*" . 'font-lock-comment-face)
   '("^ ?[*]W[*] .*\\(\n             .*\\)*" . 'font-lock-warning-face)
   '("^ ?[*]E[*] .*\\(\n             .*\\)*" . 'font-lock-separator-face)
   '("^ ?[*]F[*] .*\\(\n             .*\\)*" . 'font-lock-mismatch-face)
   '("^ ?[*]X[*] .*\\(\n             .*\\)*" . 'font-lock-mismatch-face)
   '("^ ?[*]C[*] *\\([0-9]+/ *[0-9>]+\\)"   (1 'font-lock-string-face t))
   '("^ ?[*]W[*] *\\([0-9]+/ *[0-9>]+\\)"   (1 'font-lock-comment-face t))
;  '("^ ?[*].[*]\\( *[0-9]+/ *[0-9>]+ ?\\)" (1 'font-lock-string-face t))
;  '("^ ?[*].[*] *\\([0-9]+/ *[0-9>]+\\)" (1 'fringe t))
   '("^ +" (0 'default t))
   '("<see list>"     . (0 'font-lock-list-face t))
   '("<[0-9]+ items>" . (0 'font-lock-list-face t))
   ;;
   ;; switch = value
   '("\\([a-z#?+][a-z.0-9_#-]*\\) *= *\\([a-z.'0-9_%()*/+-]+\\)"
                                           (1 'font-lock-variable-name-face)
                                           (2 'font-lock-type-face))
   ;; Iteration messages (suspit)
   '("=> *\\([0-9]+\\).*\\([0-9][.][0-9ed+-]*\\)"
                                           (1 'font-lock-variable-name-face)
                                           (2 'font-lock-type-face))
   ;; OUT tables
;  '("^ ?|.* [|:].*|"                       . 'font-lock-border-face)
   '("[|+][._=+-]+[|+]"                     . 'font-lock-border-face)
   '("|.* \\(:\\) "                        (1 'font-lock-border-face))
   '("^ ?\\(---*[+]---*\\) *$"           . (1 'font-lock-border-face))
   '("|"                                    . 'font-lock-border-face)
   '("[|│║] *\\([a-z#?+][a-z.0-9_#-]*\\) +\\([0-9+-]+\\)"
                                           (1 'font-lock-variable-name-face)
                                           (2 'font-lock-type-face))
   '("^ ?[|│║]\\( *[0-9]+ *\\)[|│║]"      . (1 'font-lock-list-face))
   '("^ ?[|│║] *[0-9]+01 *[|│║]\\(.*\\)[|│║] *$" . (1 'font-lock-list-face))
   '("^ ?[|│║] +1 [|│║]\\(.*\\)[|│║] *[\n] ?[|│║]" . (1 'font-lock-list-face))
   '("^ ?[|│║] \\([^ ]+= *1\\) [|│║]\\(.*\\)[|│║] *[\n] ?[|│║]" (1 'font-lock-list-face) (2 'font-lock-list-face))
   ;;
   ;; OUT-Frame:
   ;'("^ ?\\([|+]\\).*\\([|+]\\)"      (1 'font-lock-list-face t))
   ;'("^ ?\\([|+]\\).*\\([|+]\\)"      (2 'font-lock-list-face t))
   ;'("^ ?[+][-]+[+]"                  (0 'font-lock-list-face t))
   ;;
   ;; timer
   '("^  ?\\([0-2][0-9]\\)\\([0-5][0-9]\\)\\([0-5][0-9]\\) "
                  (2 'font-lock-comment-face)
                  (3 'font-lock-variable-name-face))
   ;;
   ;; CASTATUS-LOG (new)
   '("\\(Outer\\) \\(Inner\\) \\(SlipIt\\) +[:│] +\\(Passive\\) +\\(Force\\) +\\(Active\\) +\\(Stick\\)/\\(Slipping\\)"
     (1 'font-lock-function-name-face)
     (2 'font-lock-variable-name-face)
     (3 'font-lock-comment-face)
     (4 'font-lock-type-face)
     (5 'font-lock-variable-name-face)
     (6 'font-lock-function-name-face)
     (7 'font-lock-keyword-face)
     (8 'font-lock-comment-face))
   '(" [|│║] +\\([0-9]+\\) +\\([0-9]+\\) \\([ 0-9]+\\) +[:│] +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) "
     (1 'default) (2 'default) (3 'default)
     (4 'font-lock-type-face)
     (5 'font-lock-variable-name-face)
     (6 'font-lock-function-name-face)
     (7 'font-lock-keyword-face)
     (8 'font-lock-comment-face))
   '(" [|│║] +\\([0-9]+\\) +\\([gsk]+ +[0-9]+\\) +[:│] +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) "
     (1 'default)
     (2 'font-lock-list-face)
     (3 'font-lock-type-face)
     (4 'font-lock-variable-name-face)
     (5 'font-lock-function-name-face)
     (6 'font-lock-keyword-face)
     (7 'font-lock-comment-face))
   ;;
   ;; CASTATUS-LOG (old)
   '("\\(Outer\\) \\(Inner\\) \\(SlipIt\\) [:│] +\\(Passive\\) +\\(Active\\) +\\(Stick\\)/\\(Slipping\\)"
     (1 'font-lock-variable-name-face) (2 'font-lock-variable-name-face)
     (3 'font-lock-comment-face) (4 'font-lock-type-face) (6 'font-lock-function-name-face)
     (7 'font-lock-keyword-face) (8 'font-lock-comment-face))
   '(" [|│║] +\\([0-9]+\\) +\\([0-9]+\\) \\([ 0-9]+\\) [:│] +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) +\\([0-9]+\\) "
     (1 'default) (2 'default)
     (3 'default) (4 'font-lock-type-face) (5 'font-lock-function-name-face)
     (6 'font-lock-keyword-face) (7 'font-lock-comment-face))
   ;;
   ;; PSUM-Table
   '("^ +calls \\([^ ]+\\) +\\(user\\+sys\\).*\\(elapsed\\).*\\(eff%\\)"
     (1 'font-lock-function-name-face) (2 'font-lock-variable-name-face)
     (3 'font-lock-comment-face) (4 'font-lock-type-face))
   '("^ *\\([0-9.E+-]+\\) \\([.]*\\)\\([a-z][^. ]*\\) *\\([0-9.]+\\) ([^()]*) +\\([0-9.]+\\) ([^()]*) +\\([0-9.-]+\\)"
     (1 'default)
     (2 'font-lock-constant-face)
     (3 'font-lock-function-name-face)
     (4 'font-lock-variable-name-face)
     (5 'font-lock-comment-face)
     (6 'font-lock-type-face))
   '("^      +\\(total\\|sum\\) +\\([0-9.]+\\) +\\[sec\\]...... +\\([0-9.]+\\) +\\[sec\\]...... +\\([0-9.-]+\\)"
     (1 'font-lock-function-name-face) (2 'font-lock-variable-name-face)
     (3 'font-lock-comment-face) (4 'font-lock-type-face))
   ;;
   ;; PTOPCODE-Table
   '("^ +.tasks. \\([^ ]+\\) +\\(Tinr[^ ]*\\).*\\(Tout[^ ]*\\).*\\(eff%\\)"
     (1 'font-lock-function-name-face) (2 'font-lock-variable-name-face)
     (3 'font-lock-comment-face) (4 'font-lock-type-face))
   '("^ *\\([0-9.E+-]+\\) \\([^ ]+\\) +\\([0-9.]+\\) +[0-9.E+-]+ +\\([0-9.]+\\) +[0-9.E+-]+ +\\([0-9.-]+\\)"
     (1 'default)
     (2 'font-lock-function-name-face)
     (3 'font-lock-variable-name-face)
     (4 'font-lock-comment-face)
     (5 'font-lock-type-face))
   '("^         +\\(Summary\\) +\\([0-9.]+\\) +[0-9.E+-]+ +\\([0-9.]+\\) +[0-9.E+-]+ +\\([0-9.-]+\\)"
     (1 'font-lock-function-name-face) (2 'font-lock-variable-name-face)
     (3 'font-lock-comment-face) (4 'font-lock-type-face))
   ;;
   ;; Pro/Step-Log
   '("\\(Act.\\|Door\\|Func\\|PrL.\\|Proc\\|Step\\|Sub.\\|Trac\\) \\([^ ]+\\) *\\(Start\\|Exit\\|<[^ ]+\\)"
     (1 font-lock-keyword-face) (2 font-lock-function-name-face) (3 font-lock-keyword-face))
   '("<\\([^ <=>]+\\)" (1 font-lock-string-face))
   '("\\(Act.\\|Step\\|PrL.\\) \\([^ ]+\\) *\\(.*\\)"
     (1 font-lock-keyword-face) (2 font-lock-function-name-face) (3 font-lock-comment-face))
   '("\\(Module\\)   ? ?\\([^ ]+\\)"
     (1 font-lock-keyword-face) (2 font-lock-function-name-face))
;   '("\\(Sub.\\) \\([^ ]+\\)"
;     (1 font-lock-keyword-face) (2 font-lock-function-name-face))
   '("\\(Component\\|Situation\\) *\\([^, ]*\\)" . 'font-lock-string-face)
   ;;
   ;; Pro/Step-Arg
   '("\\(Create\\|Gen\\|Mod\\|Modify\\|Read\\|Scr\\|Scratch\\) *\\(Book\\|[C#]?Table\\|Sequence\\|VirtBook\\) +\\([^ \n]+\\)"
     (1 font-lock-type-face) (2 font-lock-type-face) (3 font-lock-variable-name-face))
   ;;
   ;; Object-Arg
   '("\\(Sub.\\|Door\\) \\([^ ]+\\) *\\(Copy\\|Create\\|Delete\\|Link\\|Modify\\|Read\\|Refund\\|Rename\\|Touch\\) *\\(Book\\|[C#]?Table\\|Sequence\\|VirtBook\\) +\\([^ \n]+\\)"
     (1 font-lock-keyword-face) (2 font-lock-function-name-face)
     (3 font-lock-type-face) (4 font-lock-type-face) (5 font-lock-variable-name-face))
   '("\\(Copy\\|Link\\|Rename\\) *\\(Book\\|Table\\|Sequence\\|VirtBook\\).*\\(from\\|to\\) +\\([^ \n]+\\)"
     (3 font-lock-type-face) (4 font-lock-variable-name-face))
   '("\\(Touch\\) *\\(Book\\|Table\\|Sequence\\|VirtBook\\).*\\(by\\) +\\([^ \n]+\\)"
     (3 font-lock-keyword-face) (4 font-lock-function-name-face))
   ;;
   ;; Trac-Log
   '("Trac " . 'font-lock-keyword-face)
   ;;
   ;; Time strings/Counters
   '("\\<\\(Elapsed\\|Sys\\|User\\)\\>" . 'font-lock-builtin-face)
   ;;
   ;; Special key-lines
   '("Accuracy Limit .*" . 'font-lock-comment-face)
   '("Starting CA-.*"    . 'font-lock-comment-face)
   '("ESTM Resource .*"  . 'font-lock-comment-face)
   '("\\(Size:\\|Time:\\) *\\([0-9.]+\\) (.*), *\\([0-9.]+\\) ("
                        (1 'font-lock-variable-name-face)
                        (2 'font-lock-type-face)
                        (3 'font-lock-type-face))
;;
   '("Version +\\([0-9.]*\\) "               . (1 'font-lock-match-face))
   '("[|║] +[|║] *\\( [^ ]+ \\) *[|║] +[|║]" . (1 'font-lock-gold-face))
;;
;; Date
   '("\\b\\([0-3][0-9]\\)/\\([0-1][0-9]\\)/\\([0-9][0-9]\\)\\b"
        (2 'font-lock-comment-face) (3 'font-lock-variable-name-face))
   '("\\b\\([0-3][0-9]\\)-\\([a-z][a-z][a-z]\\)-\\([0-9][0-9]\\)\\b"
        (2 'font-lock-comment-face) (3 'font-lock-variable-name-face))
;; Time
   '("\\b\\([0-2][0-9]\\):\\([0-5][0-9]\\):\\([0-5][0-9]\\)\\b"
       (2 'font-lock-comment-face) (3 'font-lock-variable-name-face))
))


;; Next/prev step
;;----------------
(defun permas-pro-next (&optional arg)
  "Move forward to next step log. With argument, do it that many times.
Negative argument -N means move back to Nth preceding end of step."
  (interactive "p") (if (or (null arg) (= arg 0)) (setq arg 1))
  (forward-char 1) (permas-pro-prev (- arg)))
;;
(defun permas-pro-prev (&optional arg)
"Move to start of current step log block. With argument, do it that many times.
Negative arg -N means move forward to Nth following start of step."
  (interactive "p") (or (bolp) (beginning-of-line))
  (if (re-search-backward "^ *[0-9]+ +Step [^ ]+ +[^E ]" nil 'move (or arg 1))
      (re-search-forward "Step")))


;; PRO-Help
;;----------
;;;###autoload
(defun permas-pro-help (&optional query)
  (interactive)
  (if (save-excursion (re-search-backward " [^< ]" nil t)
                      (looking-at " *\\([$a-z.0-9_-]+\\) +[<a-z0-9_]*<\\([0-9]+\\) *$"))
      (goto-prcount (match-string 2) (match-string 1))
      (permas-logtime)))

(defun goto-prcount (&optional iprcount object)
  "Jump to processor with given IPRCOUNT (ask for target if IPRCOUNT is nil)."
  (interactive)
  (let*((save (point))
        (objnam (or object "Data Object"))
        (count  (or iprcount (read-integer-at-point "Jump to PR-Counter")))
        (icount (string-to-int count))
        (count0 (int-to-string icount)) ; Cut away leading zeros
        (regexp (concat "PRcount=0*" count0 "\\b")))
       (if (re-search-backward regexp nil t)
           (progn (forward-char 7)
                  (message (format "%s was modified by this Processor with PR-Count=%i" objnam icount)))
           (setq count0 (int-to-string (+ icount 1)))
           (setq regexp (concat "PRcount=0*" count0 "\\b"))
           (if (re-search-backward regexp nil t)
               (progn (beginning-of-line)
               (message (format "%s was modified before this Processor with PR-Count=%i" objnam icount)))
               (message (concat "Can not find PR-Count \"" count "\""))
               (beep) (goto-char save)))))
;;
;;------
(provide 'permas_pro)

;;; permas_pro.el ends here
