;;; permas_psum.el --- PERMAS PSUM mode (viewing PERMAS protocol-summaries)

;; Copyright (C) 1999 Free Software Foundation, Inc.

;; Author: Markus Ast <ast@intes.de>
;; Last Change: 28-Aug-06
;; Keywords: PERMAS, protocol-summaries, viewing

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; permas-psum-mode is a major mode for viewing PERMAS psum files.

;;; Code:

;;;###autoload
(defun permas-psum-mode ()
  "Viewing mode for PERMAS protocol-summary files (*.psum)."
  (interactive) (text-mode) ; start with text-mode initialization
  (setq mode-name  "PERMAS-PSUM")
  (setq major-mode 'permas-psum-mode)
;  (local-set-key "\e\C-a" 'permas-psum-prev)
;  (local-set-key "\e\C-e" 'permas-psum-next)
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(permas-psum-font-lock-keywords t t))
  (run-hooks 'permas-psum-mode-hook))


;; font-lock support (list of regular expressions for highlighting)
;;-----------------------------------------------------------------
(setq permas-psum-font-lock-keywords
  (list
   ;; UCI echos
   '("^ ?> .*$"                                . 'font-lock-comment-face)
   ;;
   ;; Warnings and Errors
   '("^ *[*]C[*] .*\\(\n             .*\\)*" . 'font-lock-comment-face)
   '("^ *[*]W[*] .*\\(\n             .*\\)*" . 'font-lock-warning-face)
   '("^ *[*]E[*] .*\\(\n             .*\\)*" . 'font-lock-separator-face)
   '("^ *[*]F[*] .*\\(\n             .*\\)*" . 'font-lock-mismatch-face)
   '("^ *[*]X[*] .*\\(\n             .*\\)*" . 'font-lock-mismatch-face)
   ;;
   ;; switch = value
   '("\\([a-z][^ ]*\\) *= *\\([a-z.0-9_+-]*\\)"
                                            (1 'font-lock-variable-name-face)
                                            (2 'font-lock-type-face))
   ;; OUT tables
   '("^ ?|.* [|:].*|"                       . 'font-lock-border-face)
   '("[|+][._=+-]+[|+]"                     . 'font-lock-border-face)
   '("|.* \\(:\\) "                        (1 'font-lock-border-face))
   '("^ ?\\(---*[+]---*\\) *$"           . (1 'font-lock-border-face))
   '("|"                                    . 'font-lock-border-face)
   '("[|│║] *\\([a-z#?+][a-z.0-9_#-]*\\) +\\([0-9+-]+\\)"
                                           (1 'font-lock-variable-name-face)
                                           (2 'font-lock-type-face))
   '("^ ?[|│║]\\( *[0-9]+ *\\)[|│║]"      . (1 'font-lock-list-face))
   '("^ ?[|│║] *[0-9]+01 *[|│║]\\(.*\\)[|│║] *$" . (1 'font-lock-list-face))
   '("^ ?[|│║] +1 [|│║]\\(.*\\)[|│║] *[\n] ?[|│║]" . (1 'font-lock-list-face))
   ;;
   ;; timer
   ;'("^ \\([ 0-9][0-9][0-9][0-9][0-9][0-9]\\) " . 'font-lock-string-face)
   ;;
   ;; PSUM-Table
   '("^ +calls \\([^ ]+\\) +\\(user\\+sys\\).*\\(elapsed\\).*\\(eff%\\)"
     (1 'font-lock-function-name-face) (2 'font-lock-variable-name-face)
     (3 'font-lock-comment-face) (4 'font-lock-type-face))
   '("^ *\\([0-9.E+-]+\\) \\([.]*\\)\\([a-z][^. ]*\\) *\\([0-9.]+\\) ([^()]*) +\\([0-9.]+\\) ([^()]*) +\\([0-9.-]+\\)"
     (1 'default)
     (2 'font-lock-constant-face)
     (3 'font-lock-function-name-face)
     (4 'font-lock-variable-name-face)
     (5 'font-lock-comment-face)
     (6 'font-lock-type-face))
   '("^      +\\(total\\|sum\\) +\\([0-9.]+\\) +\\[sec\\]...... +\\([0-9.]+\\) +\\[sec\\]...... +\\([0-9.-]+\\)"
     (1 'font-lock-function-name-face) (2 'font-lock-variable-name-face)
     (3 'font-lock-comment-face) (4 'font-lock-type-face))
   ;;
   ;; Time strings/Counters
   '("\\<\\(Elapsed\\|Sys\\|User\\)\\>" . 'font-lock-builtin-face)
   ;;
   ;; Special key-lines
   '("ESTM Resource .*"  . 'font-lock-comment-face)
   '("Accuracy Limit .*" . 'font-lock-comment-face)
   '("Starting CA-.*"    . 'font-lock-comment-face)
   '("Outer Inner SlipIt"              . 'font-lock-function-name-face)
   '("Passive  Active  Stick/Slipping" . 'font-lock-function-name-face)
;;
   '("Version +\\([0-9.]*\\) "               . (1 'font-lock-match-face))
   '("[|║] +[|║] *\\( [^ ]+ \\) *[|║] +[|║]" . (1 'font-lock-gold-face))
))

(provide 'permas_psum)

;;; permas_psum.el ends here
