;;; permas_res.el --- PERMAS RES mode (viewing PERMAS result files)

;; Copyright (C) 1999 Free Software Foundation, Inc.

;; Author: Markus Ast <ast@intes.de>
;; Last Change: 26-Sep-08
;; Keywords: PERMAS, result, viewing

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; permas-res-mode is a major mode for viewing PERMAS result files.

;;; Code:

;;;###autoload
(defun permas-res-mode ()
  "Viewing mode for PERMAS result files (*.res)."
  (interactive) (text-mode) ; start with text-mode initialization
  (setq mode-name  "PERMAS-RES")
  (setq major-mode 'permas-res-mode)
  (setq context-help-function 'permas-res-help)
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(permas-res-font-lock-keywords t t))
  (run-hooks 'permas-res-mode-hook))


;; font-lock support (list of regular expressions for highlighting)
;;-----------------------------------------------------------------
(setq permas-res-font-lock-keywords
  (list
   ;;
   ;; UCI echos and OUT tables
   '("^ ?> .*$"                             . 'font-lock-comment-face)
   '("^ ?\\(>\\) [^ ]"                   . (1 'font-lock-border-face t))
   ;;
   ;; Warnings and Errors
   '("^ ?[*]C[*] .*\\(\n             .*\\)*" . 'font-lock-comment-face)
   '("^ ?[*]W[*] .*\\(\n             .*\\)*" . 'font-lock-warning-face)
   '("^ ?[*]E[*] .*\\(\n             .*\\)*" . 'font-lock-separator-face)
   '("^ ?[*]F[*] .*\\(\n             .*\\)*" . 'font-lock-mismatch-face)
   '("^ ?[*]X[*] .*\\(\n             .*\\)*" . 'font-lock-mismatch-face)
   '("^ ?[*]C[*] *\\([0-9]+/ *[0-9>]+\\)"   (1 'font-lock-string-face t))
   '("^ ?[*]W[*] *\\([0-9]+/ *[0-9>]+\\)"   (1 'font-lock-comment-face t))
;  '("^ ?[*].[*] *\\([0-9]+/ *[0-9>]+\\)"  (1 'fringe t))
   '("^ +" (0 'default t))
   '("<see list>"     . (0 'font-lock-list-face t))
   '("<[0-9]+ items>" . (0 'font-lock-list-face t))
   ;;
   ;; switch = value
   '("\\([a-z#?+][a-z.0-9_#-]*\\) *= *\\([a-z.'0-9/_+-]+\\)"
                                           (1 'font-lock-variable-name-face)
                                           (2 'font-lock-type-face))
   ;; OUT tables
;  '("^ ?|.* [|:].*|"                       . 'font-lock-border-face)
   '("[|+][._=+-]+[|+]"                     . 'font-lock-border-face)
   '("|.* \\(:\\) "                        (1 'font-lock-border-face))
   '("^ ?\\(---*[+]---*\\) *$"           . (1 'font-lock-border-face))
   '("|"                                    . 'font-lock-border-face)
   '("[|│║] *\\([a-z#?+][a-z.0-9_#-]*\\) +\\([0-9+-]+\\)"
                                           (1 'font-lock-variable-name-face)
                                           (2 'font-lock-type-face))
   '("^ ?[|│║]\\( *[0-9]+ ? ?\\)[|│║]"             . (1 'font-lock-list-face))
   '("^ ?[|│║] *[0-9]+01 *[|│║]\\(.*\\)[|│║] *$"   . (1 'font-lock-list-face))
   '("^ ?[|│║] +1 [|│║]\\(.*\\)[|│║] *[\n] ?[|│║]" . (1 'font-lock-list-face))
   '("^ ?[|│║] \\([^ ]+= *1\\) [|│║]\\(.*\\)[|│║] *[\n] ?[|│║]" (1 'font-lock-list-face) (2 'font-lock-list-face))
   ;;
   ;; PRXNORM
   '("^ \\(Book\\|Tbl.\\|Seq.\\|CTbl\\|Bed\\) +\\([^ \n]+\\) +\\([$][0-9A-F]+\\) +\\([$][0-9A-F]+\\) +\\([0-9.DE+-]+\\) +[^\n]+ +\\(Proc\\|Door\\|Func\\) +\\([^ \n]+\\)"
     (1 font-lock-type-face) (2 font-lock-variable-name-face) ; (3 font-lock-reference-face) 
     (4 font-lock-comment-face) (5 font-lock-type-face)
     (6 font-lock-keyword-face) (7 font-lock-function-name-face))
   ;;
   ;; DMS Dump
   '("^ \\(Book\\|Table\\|Sequence\\):.*"          . (0 'font-lock-gold-face t))
   '("^ ?> \\([^ ].*\\)"                 . (1 'font-lock-reference-face t))
   '("^[> ]*Current level [123]: \\(I.*\\)"    . (1 'font-lock-gold-face t))
   '("^[> ]*Current level [123] parameters \\(.*\\)"
                                         . (1 'font-lock-gold-face t))
   ;;
   ;; TGM Dump
   '("^[> ]*\\(...-Dump of.*:\\)"  . (1 'font-lock-gold-face t))
   '("^[> ]*\\(POM-I/O-Dump.*:\\)" . (1 'font-lock-gold-face t))
   '("^[> ]*\\(Dump of Core.*:\\)" . (1 'font-lock-gold-face t))
   '("^ [|│║]? ?\\(column.*\\) *[|│║]\\(.+\\)[|│║] *$"
     (1 'font-lock-comment-face t) (2 'font-lock-reference-face t))
   '("^ [|│║]? ?\\(ixtask\\|ixop\\|ixobj\\|ixrec\\|firstobj\\|lastuser\\|cmslot\\|nreq\\) *[|│║]\\(.+\\)[|│║] *$"
     (1 'font-lock-gold-face t) (2 'font-lock-function-name-face t))
   '("^ [|│║]? ?\\(next\\|nextid\\|nextobj\\|nextop\\|nextuser\\|olnext\\) *[|│║]\\(.+\\)[|│║] *$"
     (1 'font-lock-keyword-face) (2 'font-lock-variable-name-face t))
   '("^ [|│║]? ?\\(prev\\|previd\\|prevobj\\|prevop\\|prevuser\\|olprev\\) *[|│║]\\(.+\\)[|│║] *$"
     (1 'font-lock-keyword-face) (2 'font-lock-variable-name-face t))
   '("^ [|│║]? ?\\([rnlc]ov\\) *[|│║]\\(.+\\)[|│║] *$"
     (1 'font-lock-field-name-face) (2 'font-lock-field-name-face t))
   '("[*]ts\\(done\\|ready\\|exec\\)" (0 'font-lock-gold-face t))
;;
   '("Version +\\([0-9.]*\\) "               . (1 'font-lock-match-face))
   '("[|║] +[|║] *\\( [^ ]+ \\) *[|║] +[|║]" . (1 'font-lock-gold-face))
;;
;; Date
   '("\\b\\([0-3][0-9]\\)/\\([0-1][0-9]\\)/\\([0-9][0-9]\\)\\b"
        (2 'font-lock-comment-face) (3 'font-lock-variable-name-face))
   '("\\b\\([0-3][0-9]\\)-\\([a-z][a-z][a-z]\\)-\\([0-9][0-9]\\)\\b"
        (2 'font-lock-comment-face) (3 'font-lock-variable-name-face))
;; Time
   '("\\b\\([0-2][0-9]\\):\\([0-5][0-9]\\):\\([0-5][0-9]\\)\\b"
       (2 'font-lock-comment-face) (3 'font-lock-variable-name-face))
;;
   '("^ +" (0 'default t))
))

;;;----------------------------------------------------------------------------
(defun permas-res-help (&optional query)
  (interactive)
  (let ((key (save-excursion (beginning-of-line) (forward-char 3)
                             (upcase (keyword-at-point)))))
       (cond
       ((equal key "IXTASK"  ) (tgm-task    (integer-at-point)))
       ((equal key "IXOP"    ) (tgm-operand (integer-at-point)))
       ((equal key "IXOBJ"   ) (tgm-object  (integer-at-point)))
       ((equal key "IXREC"   ) (tgm-record  (integer-at-point)))
       ((equal key "LASTUSER") (tgm-operand (integer-at-point)))
       ((equal key "FIRSTOBJ") (tgm-object  (integer-at-point)))
       ((equal key "CMSLOT"  ) (cm-slot     (integer-at-point)))
       ((equal key "NREQ"    ) (tgm-record  (integer-at-point)))
       ((equal key "NEXTID"  ) (DUMP-next-column (integer-at-point)  1 ))
       ((equal key "PREVID"  ) (DUMP-next-column (integer-at-point)  2 ))
       ((equal key "NEXTOP"  ) (DUMP-next-column (integer-at-point)  1 ))
       ((equal key "NEXT"    ) (DUMP-next-column (integer-at-point)  1 ))
       ((equal key "PREV"    ) (DUMP-next-column (integer-at-point)  2 ))
       ((equal key "NEXTUSER") (DUMP-next-column (integer-at-point)  7 ))
       ((equal key "PREVUSER") (DUMP-next-column (integer-at-point)  8 ))
       ((equal key "NEXTOBJ" ) (DUMP-next-column (integer-at-point)  7 ))
       ((equal key "PREVOBJ" ) (DUMP-next-column (integer-at-point)  8 ))
       ((equal key "OLNEXT"  ) (DUMP-next-column (integer-at-point) 24 ))
       ((equal key "OLPREV"  ) (DUMP-next-column (integer-at-point) 25 ))
       (t (beep) (message (concat "No help for \"" key "\"")))
)))
;(defun permas-res-help (&optional query)
;"Show help for keyword at PERMAS RES position (or jump to referred position)
;Ask interactively for keyword if QUERY is non-nil (short cut otherwise)."
;;;----------------------------------------------------------------------------
(defun tgm-task (&optional ixtask)
  "Jump to given TASK in TGM dump (ask for target if IXTASK is nil)."
  (interactive) (DUMP-goto-dump-column "TGM-Dump of TASKS array"
  (or ixtask (read-integer-at-point "Jump to TGM task IXTASK")) 3 ))
;;
(defun tgm-operand (&optional ixop)
  "Jump to given OPERAND in TGM dump (ask for target if IXOP is nil)."
  (interactive) (DUMP-goto-dump-column "TGM-Dump of OPERANDS array"
  (or ixop (read-integer-at-point "Jump to TGM operand IXOP")) 1 ))
;;
(defun tgm-object (&optional ixobj)
  "Jump to given OBJECT in TGM dump (ask for target if IXOBJ is nil)."
  (interactive) (DUMP-goto-dump-column "TGM-Dump of OBJECTS array"
  (or ixobj (read-integer-at-point "Jump to TGM object IXOBJ")) 3 ))
;;
(defun tgm-record (&optional ixrec)
  "Jump to given RECORD in TGM dump (ask for target if IXREC is nil)."
  (interactive) (DUMP-goto-dump-column "TGM-Dump of RECORDS array"
  (or ixrec (read-integer-at-point "Jump to TGM record IXREC")) 3 ))
;;
(defun xpu-call (&optional ixpc)
  "Jump to given XPU-Call in TGM dump (ask for target if IXPC is nil)."
  (interactive) (DUMP-goto-dump-column "TGM-Dump of XPUCALLS array"
  (or ixpc (read-integer-at-point "Jump to XPU call IXPC")) 3 ))
;;
(defun cm-slot (&optional cmslot)
  "Jump to given SLOT in CM dump (ask for target if CMSLOT is nil)."
  (interactive) (DUMP-goto-dump-column "Dump of Core Slot Management DLL"
  (or cmslot (read-integer-at-point "Jump to CM slot CMSLOT")) 4 ))
;;
(defun io-req (&optional ixreq)
  "Jump to given Request in I/O-Req dump (ask for target if IXREQ is nil)."
  (interactive) (DUMP-goto-dump-column "POM-I/O-Dump of I/O request structure"
  (or ixreq (read-integer-at-point "Jump to I/O request IXREQ")) 3 ))
;;
;;------
;;
(defun DUMP-goto-dump-column (title icol &optional row)
  (let ((save (point)))
       (if (or (search-forward title nil t)
               (search-backward title nil t))
           (if (re-search-forward (concat "^[|│║ ]*column.* " icol "\\b") nil t)
               (progn (backward-char 1) (recenter 0)
                      (if row (let ((col (current-column)))
                               (forward-line (+ row 1)) (move-to-column col))))
               (message (concat "Can not find column " icol " for \"" title "\""))
               (beep) (goto-char save))
       (message (concat "Can not find \"" title "\""))
       (beep) (goto-char save))))
;;
(defun DUMP-next-column (icol &optional row)
  (let ((save (point)))
       (if (search-backward "Dump" nil t)
           (if (re-search-forward (concat "^[|│║ ]*column.* " icol "\\b") nil t)
               (progn (backward-char 1) (recenter 0)
                      (if row (let ((col (current-column)))
                               (forward-line (+ row 1)) (move-to-column col))))
               (message (concat "Can not find NEXT/PREV column " icol))
               (beep) (goto-char save))
       (message (concat "Can not find \"Dump\" title"))
       (beep) (goto-char save))))


(provide 'permas_res)

;;; permas_res.el ends here
