;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Special permas-t19-mode for Marc ASCII Result Files (t19) of PERMAS ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "permas_t19")
(defun permas-t19-mode ()
  "Editing mode for Marc (t19) result files for PERMAS."
  (interactive)
  (kill-all-local-variables) (text-mode) ;;; ??? (setq overwrite-mode t)
  (use-local-map permas-t19-mode-map)
  (make-local-variable 'tab-stop-list)
  (setq tab-stop-list '( 9 19 29 39 49 59 69 79))
  (setq mode-name "MARC") (setq major-mode 'permas-t19-mode)
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(permas-t19-font-lock-keywords t t))
  (setq context-help-function 'permas-t19-help-block)
  (run-hooks 'permas-t19-mode-hook))
;
(defvar permas-t19-mode-map nil "Local keymap used in permas-t19-mode (MARC-t19-Files).")
(if     permas-t19-mode-map () (setq permas-t19-mode-map (make-sparse-keymap))
  (define-key permas-t19-mode-map ""    'permas-t19-next-block)
  (define-key permas-t19-mode-map ""    'permas-t19-prev-block)
)


;; font-lock support (list of regular expressions for highlighting)
;;-----------------------------------------------------------------
(setq permas-t19-font-lock-keywords
  (list
   '("^=beg=.*$"   . 'font-lock-separator-face)
   '("^\*\*\*\*$"  . 'font-lock-match-face)
   '("^\+\+\+\+$"  . 'show-paren-mismatch-face)
   '("^----$"      . 'show-paren-mismatch-face)
))

;; Next/prev step
;;----------------
(defun permas-t19-prev-block () "Move to start of current block"
  (interactive)
  (re-search-backward "^=beg=" nil "t")
)
(defun permas-t19-next-block () "Move to start of next block"
  (interactive)
  (if (re-search-forward "^=beg=" nil "t"
         (if (looking-at "^=beg=") 2 1 ))
      (beginning-of-line))
)

;; Helps
;;-------
(defun permas-t19-help-block () "Show description/name for current Block."
  (interactive)
  (let ((save (point))) (beginning-of-line)
       (if (not (looking-at "^=beg=" ))
           (re-search-backward "^=beg=" nil t))
       (if (not (looking-at "^=beg=\\([1-9][0-9][0-9]\\)"))
           (message "No Block type number found")
           (forward-char 5)
           (let*((id (match-string 1))
                 (help-text (cdr (assoc (string-to-int id) permas-t19-help-alist))))
                (if help-text (message (concat "Block " id "nn: " help-text))
                              (message (concat "No help for block " id)))))
  (goto-char save)))
;;;
(defun permas-t19-show-block () "Show all Blocks (originally sorted)."
   (interactive)
   (with-output-to-temp-buffer "*Help*"
        (princ "Known Marc t19 Blocks are:\n")
        (let ((list permas-t19-help-alist) entry)
             (while list (setq entry (car list))
                   (princ (format "%4d - %s\n" (car entry) (cdr entry)))
                   (setq list (cdr list))))))

(defvar permas-t19-help-alist nil
 "AList for marc t19 file block help texts.")
(setq permas-t19-help-alist '(;;;
( 501 . "Analysis Title")
( 502 . "Analysis verification Data")
( 504 . "Dummy")
( 505 . "Domain Decomposition Information")
( 506 . "Element Variable Postcodes")
( 507 . "Element Connectivities")
( 508 . "Nodal Coordinates")
( 509 . "Spring Data")
( 510 . "Nodal Codes and Transformation ID")
( 511 . "Ties Due to Meshing")
( 512 . "Transformation Matrices")
( 513 . "Set Definition")
( 514 . "Contact Geometry Data")
( 515 . "Flow Line Data")
( 516 . "Loadcase Title")
( 517 . "Integer Increment Verification Data")
( 518 . "Real Increment Verification Data")
( 519 . "New Model")
( 520 . "Magnitude of Distributed Loads")
( 521 . "Magnitude of Spring Forces")
( 522 . "Contact Body Results")
( 523 . "Element Integration Point Values")
( 524 . "Nodal Results")
( 525 . "Response Gradients")
( 526 . "Element Contribution to Response")
( 527 . "Design Variable Values")
( 528 . "Flow Line Updates")
( 528 . "Flow Line Updates")
( 529 . "Global Variables")
))
;;----------------------------------------------------------------------------
(provide 'permas_t19)
